@@ -48,16 +48,15 @@ static void* freerdp_channels_find_static_entry_in_table(const STATIC_ENTRY_TABL
                                                          const char* identifier)
 {
 	size_t index = 0;
-	const STATIC_ENTRY* pEntry = &table->table[index++];
+	const STATIC_ENTRY* pEntry = &table->table.cse[index++];
 
 	while (pEntry->entry != NULL)
 	{
+		static_entry_fn_t fkt = pEntry->entry;
 		if (strcmp(pEntry->name, identifier) == 0)
-		{
-			return (void*)pEntry->entry;
-		}
+			return WINPR_FUNC_PTR_CAST(fkt, void*);
 
-		pEntry = &table->table[index++];
+		pEntry = &table->table.cse[index++];
 	}
 
 	return NULL;
@@ -68,7 +67,7 @@ void* freerdp_channels_client_find_static_entry(const char* name, const char* id
 	size_t index = 0;
 	const STATIC_ENTRY_TABLE* pEntry = &CLIENT_STATIC_ENTRY_TABLES[index++];
 
-	while (pEntry->table != NULL)
+	while (pEntry->table.cse != NULL)
 	{
 		if (strcmp(pEntry->name, name) == 0)
 		{
@@ -488,15 +487,15 @@ PVIRTUALCHANNELENTRY freerdp_channels_load_static_addin_entry(LPCSTR pszName, LP
 					     0) || /* we only want to know if strnlen is > 0 */
 					    (strncmp(subsystems->name, pszSubsystem, MAX_PATH) == 0))
 					{
+						static_subsystem_entry_fn_t fkt = subsystems->entry;
+
 						if (pszType)
 						{
 							if (strncmp(subsystems->type, pszType, MAX_PATH) == 0)
-								return (PVIRTUALCHANNELENTRY)subsystems->entry;
+								return WINPR_FUNC_PTR_CAST(fkt, PVIRTUALCHANNELENTRY);
 						}
 						else
-						{
-							return (PVIRTUALCHANNELENTRY)subsystems->entry;
-						}
+							return WINPR_FUNC_PTR_CAST(fkt, PVIRTUALCHANNELENTRY);
 					}
 				}
 			}
@@ -508,7 +507,7 @@ PVIRTUALCHANNELENTRY freerdp_channels_load_static_addin_entry(LPCSTR pszName, LP
 						return NULL;
 				}
 
-				return (PVIRTUALCHANNELENTRY)table->entry;
+				return table->entry.csevc;
 			}
 		}
 	}
@@ -595,7 +594,7 @@ static void channel_client_handler_free(msg_proc_internals* internals)
 		return;
 
 	if (internals->thread)
-		CloseHandle(internals->thread);
+		(void)CloseHandle(internals->thread);
 	MessageQueue_Free(internals->queue);
 	Stream_Free(internals->data_in, TRUE);
 	free(internals->channel_name);
