@@ -65,6 +65,8 @@ static bool is_mac_os_sonoma_or_later(void)
 	int major = 0;
 	int minor = 0;
 	int patch = 0;
+
+	// NOLINTNEXTLINE(cert-err34-c)
 	const int rc = sscanf(str, "%d.%d.%d", &major, &minor, &patch);
 	if (rc != 3)
 	{
@@ -110,7 +112,7 @@ static void printer_cups_get_printjob_name(char* buf, size_t size, size_t id)
 	WINPR_ASSERT(buf);
 	WINPR_ASSERT(size > 0);
 
-	(void)sprintf_s(buf, size - 1, "FreeRDP Print %04d-%02d-%02d %02d-%02d-%02d - Job %" PRIdz,
+	(void)sprintf_s(buf, size - 1, "FreeRDP Print %04d-%02d-%02d %02d-%02d-%02d - Job %" PRIuz,
 	                t->tm_year + 1900, t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec,
 	                id);
 }
@@ -147,10 +149,6 @@ static UINT write_printjob(rdpPrintJob* printjob, const BYTE* data, size_t size)
  */
 static UINT printer_cups_write_printjob(rdpPrintJob* printjob, const BYTE* data, size_t size)
 {
-	rdpCupsPrintJob* cups_printjob = (rdpCupsPrintJob*)printjob;
-
-	WINPR_ASSERT(cups_printjob);
-
 	return write_printjob(printjob, data, size);
 }
 
@@ -366,7 +364,7 @@ static rdpPrinter** printer_cups_enum_printers(rdpPrinterDriver* driver)
 	if (!printers)
 		return NULL;
 
-	for (size_t i = 0; i < num_dests; i++)
+	for (size_t i = 0; i < (size_t)num_dests; i++)
 	{
 		const cups_dest_t* dest = &dests[i];
 		if (dest->instance == NULL)
@@ -432,7 +430,7 @@ static void printer_cups_release_ref_driver(rdpPrinterDriver* driver)
 		cups_driver->references--;
 }
 
-FREERDP_ENTRY_POINT(UINT cups_freerdp_printer_client_subsystem_entry(void* arg))
+FREERDP_ENTRY_POINT(UINT VCAPITYPE cups_freerdp_printer_client_subsystem_entry(void* arg))
 {
 	rdpPrinterDriver** ppPrinter = (rdpPrinterDriver**)arg;
 	if (!ppPrinter)
