@@ -223,7 +223,7 @@ static BOOL rdpdr_load_drive(rdpdrPlugin* rdpdr, const char* name, const char* p
 
 fail:
 	freerdp_device_free(drive.device);
-	return rc;
+	return rc == CHANNEL_RC_OK;
 }
 
 /**
@@ -514,19 +514,19 @@ typedef struct
  */
 static UINT handle_hotplug(rdpdrPlugin* rdpdr)
 {
-	struct dirent* pDirent;
-	DIR* pDir;
-	char fullpath[PATH_MAX];
+	struct dirent* pDirent = NULL;
+	char fullpath[PATH_MAX] = { 0 };
 	char* szdir = (char*)"/Volumes";
-	struct stat buf;
-	hotplug_dev dev_array[MAX_USB_DEVICES];
-	int count;
-	DEVICE_DRIVE_EXT* device_ext;
+	struct stat buf = { 0 };
+	hotplug_dev dev_array[MAX_USB_DEVICES] = { 0 };
+	int count = 0;
+	DEVICE_DRIVE_EXT* device_ext = NULL;
 	ULONG_PTR* keys = NULL;
 	int size = 0;
-	UINT error;
+	UINT error = ERROR_INTERNAL_ERROR;
 	UINT32 ids[1];
-	pDir = opendir(szdir);
+
+	DIR* pDir = opendir(szdir);
 
 	if (pDir == NULL)
 	{
@@ -706,7 +706,7 @@ static DWORD WINAPI drive_hotplug_thread_func(LPVOID arg)
 out:
 	if (rdpdr->stopEvent)
 	{
-		CloseHandle(rdpdr->stopEvent);
+		(void)CloseHandle(rdpdr->stopEvent);
 		rdpdr->stopEvent = NULL;
 	}
 	ExitThread(CHANNEL_RC_OK);
@@ -737,12 +737,15 @@ static BOOL isAutomountLocation(const char* path)
 		const char* location = automountLocations[x];
 		size_t length = 0;
 
+		WINPR_PRAGMA_DIAG_PUSH
+		WINPR_PRAGMA_DIAG_IGNORED_FORMAT_NONLITERAL
 		if (strstr(location, "%lu"))
-			snprintf(buffer, sizeof(buffer), location, (unsigned long)uid);
+			(void)snprintf(buffer, sizeof(buffer), location, (unsigned long)uid);
 		else if (strstr(location, "%s"))
-			snprintf(buffer, sizeof(buffer), location, uname);
+			(void)snprintf(buffer, sizeof(buffer), location, uname);
 		else
-			snprintf(buffer, sizeof(buffer), "%s", location);
+			(void)snprintf(buffer, sizeof(buffer), "%s", location);
+		WINPR_PRAGMA_DIAG_POP
 
 		length = strnlen(buffer, sizeof(buffer));
 
@@ -834,9 +837,25 @@ static UINT handle_platform_mounts_bsd(wLog* log, hotplug_dev* dev_array, size_t
 
 #if defined(__LINUX__) || defined(__linux__)
 #include <mntent.h>
+static struct mntent* getmntent_x(FILE* f, struct mntent* buffer, char* pathbuffer,
+                                  size_t pathbuffersize)
+{
+#if defined(FREERDP_HAVE_GETMNTENT_R)
+	WINPR_ASSERT(pathbuffersize <= INT32_MAX);
+	return getmntent_r(f, buffer, pathbuffer, (int)pathbuffersize);
+#else
+	(void)buffer;
+	(void)pathbuffer;
+	(void)pathbuffersize;
+	return getmntent(f);
+#endif
+}
+
 static UINT handle_platform_mounts_linux(wLog* log, hotplug_dev* dev_array, size_t* size)
 {
 	FILE* f = NULL;
+	struct mntent mnt = { 0 };
+	char pathbuffer[PATH_MAX] = { 0 };
 	struct mntent* ent = NULL;
 	f = winpr_fopen("/proc/mounts", "r");
 	if (f == NULL)
@@ -844,11 +863,11 @@ static UINT handle_platform_mounts_linux(wLog* log, hotplug_dev* dev_array, size
 		WLog_Print(log, WLOG_ERROR, "fopen failed!");
 		return ERROR_OPEN_FAILED;
 	}
-	while ((ent = getmntent(f)) != NULL)
+	while ((ent = getmntent_x(f, &mnt, pathbuffer, sizeof(pathbuffer))) != NULL)
 	{
 		handle_mountpoint(dev_array, size, ent->mnt_dir);
 	}
-	fclose(f);
+	(void)fclose(f);
 	return ERROR_SUCCESS;
 }
 #endif
@@ -1064,7 +1083,7 @@ out:
 
 	if (rdpdr->stopEvent)
 	{
-		CloseHandle(rdpdr->stopEvent);
+		(void)CloseHandle(rdpdr->stopEvent);
 		rdpdr->stopEvent = NULL;
 	}
 
@@ -1090,7 +1109,7 @@ static UINT drive_hotplug_thread_terminate(rdpdrPlugin* rdpdr)
 	{
 #if !defined(_WIN32)
 		if (rdpdr->stopEvent)
-			SetEvent(rdpdr->stopEvent);
+			(void)SetEvent(rdpdr->stopEvent);
 #endif
 #ifdef __MACOSX__
 		CFRunLoopStop(rdpdr->runLoop);
@@ -1104,7 +1123,7 @@ static UINT drive_hotplug_thread_terminate(rdpdrPlugin* rdpdr)
 			return error;
 		}
 
-		CloseHandle(rdpdr->hotplugThread);
+		(void)CloseHandle(rdpdr->hotplugThread);
 		rdpdr->hotplugThread = NULL;
 	}
 
@@ -1157,7 +1176,8 @@ static UINT rdpdr_process_connect(rdpdrPlugin* rdpdr)
 			if (!drive->Path)
 				continue;
 
-			BOOL hotplugAll = strncmp(drive->Path, "*", 2) == 0;
+			const char wildcard[] = "*";
+			BOOL hotplugAll = strncmp(drive->Path, wildcard, sizeof(wildcard)) == 0;
 			BOOL hotplugLater = strncmp(drive->Path, DynamicDrives, sizeof(DynamicDrives)) == 0;
 
 			if (hotplugAll || hotplugLater)
@@ -1249,7 +1269,7 @@ static UINT rdpdr_send_client_announce_reply(rdpdrPlugin* rdpdr)
 	Stream_Write_UINT16(s, PAKID_CORE_CLIENTID_CONFIRM); /* PacketId (2 bytes) */
 	Stream_Write_UINT16(s, rdpdr->clientVersionMajor);
 	Stream_Write_UINT16(s, rdpdr->clientVersionMinor);
-	Stream_Write_UINT32(s, (UINT32)rdpdr->clientID);
+	Stream_Write_UINT32(s, rdpdr->clientID);
 	return rdpdr_send(rdpdr, s);
 }
 
@@ -1296,7 +1316,7 @@ static UINT rdpdr_send_client_name_request(rdpdrPlugin* rdpdr)
 	Stream_Write_UINT32(s, 0);                      /* codePage, must be set to zero */
 	Stream_Write_UINT32(s,
 	                    (UINT32)computerNameLenW); /* computerNameLen, including null terminator */
-	Stream_Write(s, computerNameW, (size_t)computerNameLenW);
+	Stream_Write(s, computerNameW, computerNameLenW);
 	free(computerNameW);
 	return rdpdr_send(rdpdr, s);
 }
@@ -1626,7 +1646,7 @@ static const char* state_str(size_t count, va_list ap, char* buffer, size_t size
 static BOOL rdpdr_state_check(rdpdrPlugin* rdpdr, UINT16 packetid, enum RDPDR_CHANNEL_STATE next,
                               size_t count, ...)
 {
-	va_list ap;
+	va_list ap = { 0 };
 	WINPR_ASSERT(rdpdr);
 
 	va_start(ap, count);
@@ -1859,7 +1879,7 @@ static UINT rdpdr_process_receive(rdpdrPlugin* rdpdr, wStream* s)
 UINT rdpdr_send(rdpdrPlugin* rdpdr, wStream* s)
 {
 	UINT status = 0;
-	rdpdrPlugin* plugin = (rdpdrPlugin*)rdpdr;
+	rdpdrPlugin* plugin = rdpdr;
 
 	if (!s)
 	{
@@ -2014,8 +2034,6 @@ static VOID VCAPITYPE rdpdr_virtual_channel_open_event_ex(LPVOID lpUserParam, DW
 	if (error && rdpdr && rdpdr->rdpcontext)
 		setChannelError(rdpdr->rdpcontext, error,
 		                "rdpdr_virtual_channel_open_event_ex reported an error");
-
-	return;
 }
 
 static DWORD WINAPI rdpdr_virtual_channel_client_thread(LPVOID arg)
@@ -2176,7 +2194,7 @@ static UINT rdpdr_virtual_channel_event_disconnected(rdpdrPlugin* rdpdr)
 	}
 
 	if (rdpdr->thread)
-		CloseHandle(rdpdr->thread);
+		(void)CloseHandle(rdpdr->thread);
 	MessageQueue_Free(rdpdr->queue);
 	rdpdr->queue = NULL;
 	rdpdr->thread = NULL;
@@ -2311,7 +2329,8 @@ FREERDP_ENTRY_POINT(BOOL VCAPITYPE VirtualChannelEntryEx(PCHANNEL_ENTRY_POINTS p
 
 	rdpdr->channelDef.options =
 	    CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP | CHANNEL_OPTION_COMPRESS_RDP;
-	sprintf_s(rdpdr->channelDef.name, ARRAYSIZE(rdpdr->channelDef.name), RDPDR_SVC_CHANNEL_NAME);
+	(void)sprintf_s(rdpdr->channelDef.name, ARRAYSIZE(rdpdr->channelDef.name),
+	                RDPDR_SVC_CHANNEL_NAME);
 	rdpdr->sequenceId = 0;
 	pEntryPointsEx = (CHANNEL_ENTRY_POINTS_FREERDP_EX*)pEntryPoints;
 
