@@ -19,11 +19,8 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <freerdp/config.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -37,23 +34,18 @@
 #include <winpr/synch.h>
 #include <winpr/thread.h>
 #include <winpr/wlog.h>
+#include <winpr/assert.h>
 
 #include <freerdp/freerdp.h>
 #include <freerdp/channels/rdpdr.h>
 #include <freerdp/channels/log.h>
+#include <freerdp/utils/rdpdr_utils.h>
 
 #define TAG CHANNELS_TAG("serial.client")
 
-/* TODO: all #ifdef __linux__ could be removed once only some generic
- * functions will be used. Replace CommReadFile by ReadFile,
- * CommWriteFile by WriteFile etc..  */
-#if defined __linux__ && !defined ANDROID
-
 #define MAX_IRP_THREADS 5
 
-typedef struct _SERIAL_DEVICE SERIAL_DEVICE;
-
-struct _SERIAL_DEVICE
+typedef struct
 {
 	DEVICE device;
 	BOOL permissive;
@@ -66,20 +58,18 @@ struct _SERIAL_DEVICE
 
 	/* one thread per pending IRP and indexed according their CompletionId */
 	wListDictionary* IrpThreads;
-	UINT32 IrpThreadToBeTerminatedCount;
 	CRITICAL_SECTION TerminatingIrpThreadsLock;
 	rdpContext* rdpcontext;
-};
+} SERIAL_DEVICE;
 
-typedef struct _IRP_THREAD_DATA IRP_THREAD_DATA;
-
-struct _IRP_THREAD_DATA
+typedef struct
 {
 	SERIAL_DEVICE* serial;
 	IRP* irp;
-};
+} IRP_THREAD_DATA;
 
-static UINT32 _GetLastErrorToIoStatus(SERIAL_DEVICE* serial)
+static void close_terminated_irp_thread_handles(SERIAL_DEVICE* serial, BOOL forceClose);
+static UINT32 GetLastErrorToIoStatus(SERIAL_DEVICE* serial)
 {
 	/* http://msdn.microsoft.com/en-us/library/ff547466%28v=vs.85%29.aspx#generic_status_values_for_serial_device_control_requests
 	 */
@@ -126,12 +116,15 @@ static UINT32 _GetLastErrorToIoStatus(SERIAL_DEVICE* serial)
 
 static UINT serial_process_irp_create(SERIAL_DEVICE* serial, IRP* irp)
 {
-	DWORD DesiredAccess;
-	DWORD SharedAccess;
-	DWORD CreateDisposition;
-	UINT32 PathLength;
+	DWORD DesiredAccess = 0;
+	DWORD SharedAccess = 0;
+	DWORD CreateDisposition = 0;
+	UINT32 PathLength = 0;
 
-	if (Stream_GetRemainingLength(irp->input) < 32)
+	WINPR_ASSERT(serial);
+	WINPR_ASSERT(irp);
+
+	if (!Stream_CheckAndLogRequiredLengthWLog(serial->log, irp->input, 32))
 		return ERROR_INVALID_DATA;
 
 	Stream_Read_UINT32(irp->input, DesiredAccess);     /* DesiredAccess (4 bytes) */
@@ -142,11 +135,10 @@ static UINT serial_process_irp_create(SERIAL_DEVICE* serial, IRP* irp)
 	Stream_Seek_UINT32(irp->input);                    /* CreateOptions (4 bytes) */
 	Stream_Read_UINT32(irp->input, PathLength);        /* PathLength (4 bytes) */
 
-	if (Stream_GetRemainingLength(irp->input) < PathLength)
+	if (!Stream_SafeSeek(irp->input, PathLength)) /* Path (variable) */
 		return ERROR_INVALID_DATA;
 
-	Stream_Seek(irp->input, PathLength); /* Path (variable) */
-	assert(PathLength == 0);             /* MS-RDPESP 2.2.2.2 */
+	WINPR_ASSERT(PathLength == 0); /* MS-RDPESP 2.2.2.2 */
 #ifndef _WIN32
 	/* Windows 2012 server sends on a first call :
 	 *     DesiredAccess     = 0x00100080: SYNCHRONIZE | FILE_READ_ATTRIBUTES
@@ -158,9 +150,9 @@ static UINT serial_process_irp_create(SERIAL_DEVICE* serial, IRP* irp)
 	 * FILE_READ_EA | FILE_READ_DATA SharedAccess      = 0x00000007: FILE_SHARE_DELETE |
 	 * FILE_SHARE_WRITE | FILE_SHARE_READ CreateDisposition = 0x00000001: CREATE_NEW
 	 *
-	 * assert(DesiredAccess == (GENERIC_READ | GENERIC_WRITE));
-	 * assert(SharedAccess == 0);
-	 * assert(CreateDisposition == OPEN_EXISTING);
+	 * WINPR_ASSERT(DesiredAccess == (GENERIC_READ | GENERIC_WRITE));
+	 * WINPR_ASSERT(SharedAccess == 0);
+	 * WINPR_ASSERT(CreateDisposition == OPEN_EXISTING);
 	 *
 	 */
 	WLog_Print(serial->log, WLOG_DEBUG,
@@ -195,7 +187,7 @@ static UINT serial_process_irp_create(SERIAL_DEVICE* serial, IRP* irp)
 	/* GetCommState(serial->hComm, &dcb); */
 	/* dcb.fBinary = TRUE; */
 	/* SetCommState(serial->hComm, &dcb); */
-	assert(irp->FileId == 0);
+	WINPR_ASSERT(irp->FileId == 0);
 	irp->FileId = irp->devman->id_sequence++; /* FIXME: why not ((WINPR_COMM*)hComm)->fd? */
 	irp->IoStatus = STATUS_SUCCESS;
 	WLog_Print(serial->log, WLOG_DEBUG, "%s (DeviceId: %" PRIu32 ", FileId: %" PRIu32 ") created.",
@@ -208,11 +200,16 @@ error_handle:
 
 static UINT serial_process_irp_close(SERIAL_DEVICE* serial, IRP* irp)
 {
-	if (Stream_GetRemainingLength(irp->input) < 32)
+	WINPR_ASSERT(serial);
+	WINPR_ASSERT(irp);
+
+	if (!Stream_CheckAndLogRequiredLengthWLog(serial->log, irp->input, 32))
 		return ERROR_INVALID_DATA;
 
 	Stream_Seek(irp->input, 32); /* Padding (32 bytes) */
 
+	close_terminated_irp_thread_handles(serial, TRUE);
+
 	if (!CloseHandle(serial->hComm))
 	{
 		WLog_Print(serial->log, WLOG_WARN, "CloseHandle failure: %s (%" PRIu32 ") closed.",
@@ -223,9 +220,9 @@ static UINT serial_process_irp_close(SERIAL_DEVICE* serial, IRP* irp)
 
 	WLog_Print(serial->log, WLOG_DEBUG, "%s (DeviceId: %" PRIu32 ", FileId: %" PRIu32 ") closed.",
 	           serial->device.name, irp->device->id, irp->FileId);
-	serial->hComm = NULL;
 	irp->IoStatus = STATUS_SUCCESS;
 error_handle:
+	serial->hComm = NULL;
 	Stream_Zero(irp->output, 5); /* Padding (5 bytes) */
 	return CHANNEL_RC_OK;
 }
@@ -237,16 +234,21 @@ error_handle:
  */
 static UINT serial_process_irp_read(SERIAL_DEVICE* serial, IRP* irp)
 {
-	UINT32 Length;
-	UINT64 Offset;
+	UINT32 Length = 0;
+	UINT64 Offset = 0;
 	BYTE* buffer = NULL;
 	DWORD nbRead = 0;
 
-	if (Stream_GetRemainingLength(irp->input) < 32)
+	WINPR_ASSERT(serial);
+	WINPR_ASSERT(irp);
+
+	if (!Stream_CheckAndLogRequiredLengthWLog(serial->log, irp->input, 32))
 		return ERROR_INVALID_DATA;
 
 	Stream_Read_UINT32(irp->input, Length); /* Length (4 bytes) */
 	Stream_Read_UINT64(irp->input, Offset); /* Offset (8 bytes) */
+	(void)Offset; /* [MS-RDPESP] 3.2.5.1.4 Processing a Server Read Request Message
+	               * ignored */
 	Stream_Seek(irp->input, 20);            /* Padding (20 bytes) */
 	buffer = (BYTE*)calloc(Length, sizeof(BYTE));
 
@@ -257,7 +259,7 @@ static UINT serial_process_irp_read(SERIAL_DEVICE* serial, IRP* irp)
 	}
 
 	/* MS-RDPESP 3.2.5.1.4: If the Offset field is not set to 0, the value MUST be ignored
-	 * assert(Offset == 0);
+	 * WINPR_ASSERT(Offset == 0);
 	 */
 	WLog_Print(serial->log, WLOG_DEBUG, "reading %" PRIu32 " bytes from %s", Length,
 	           serial->device.name);
@@ -272,7 +274,7 @@ static UINT serial_process_irp_read(SERIAL_DEVICE* serial, IRP* irp)
 		WLog_Print(serial->log, WLOG_DEBUG,
 		           "read failure to %s, nbRead=%" PRIu32 ", last-error: 0x%08" PRIX32 "",
 		           serial->device.name, nbRead, GetLastError());
-		irp->IoStatus = _GetLastErrorToIoStatus(serial);
+		irp->IoStatus = GetLastErrorToIoStatus(serial);
 	}
 
 	WLog_Print(serial->log, WLOG_DEBUG, "%" PRIu32 " bytes read from %s", nbRead,
@@ -284,7 +286,7 @@ error_handle:
 	{
 		if (!Stream_EnsureRemainingCapacity(irp->output, nbRead))
 		{
-			WLog_ERR(TAG, "Stream_EnsureRemainingCapacity failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "Stream_EnsureRemainingCapacity failed!");
 			free(buffer);
 			return CHANNEL_RC_NO_MEMORY;
 		}
@@ -298,18 +300,25 @@ error_handle:
 
 static UINT serial_process_irp_write(SERIAL_DEVICE* serial, IRP* irp)
 {
-	UINT32 Length;
-	UINT64 Offset;
+	UINT32 Length = 0;
+	UINT64 Offset = 0;
 	DWORD nbWritten = 0;
 
-	if (Stream_GetRemainingLength(irp->input) < 32)
+	WINPR_ASSERT(serial);
+	WINPR_ASSERT(irp);
+
+	if (!Stream_CheckAndLogRequiredLengthWLog(serial->log, irp->input, 32))
 		return ERROR_INVALID_DATA;
 
 	Stream_Read_UINT32(irp->input, Length); /* Length (4 bytes) */
 	Stream_Read_UINT64(irp->input, Offset); /* Offset (8 bytes) */
-	Stream_Seek(irp->input, 20);            /* Padding (20 bytes) */
+	(void)Offset; /* [MS-RDPESP] 3.2.5.1.4 Processing a Server Read Request Message
+	               * ignored */
+	if (!Stream_SafeSeek(irp->input, 20))   /* Padding (20 bytes) */
+		return ERROR_INVALID_DATA;
+
 	/* MS-RDPESP 3.2.5.1.5: The Offset field is ignored
-	 * assert(Offset == 0);
+	 * WINPR_ASSERT(Offset == 0);
 	 *
 	 * Using a serial printer, noticed though this field could be
 	 * set.
@@ -317,8 +326,11 @@ static UINT serial_process_irp_write(SERIAL_DEVICE* serial, IRP* irp)
 	WLog_Print(serial->log, WLOG_DEBUG, "writing %" PRIu32 " bytes to %s", Length,
 	           serial->device.name);
 
+	const void* ptr = Stream_ConstPointer(irp->input);
+	if (!Stream_SafeSeek(irp->input, Length))
+		return ERROR_INVALID_DATA;
 	/* FIXME: CommWriteFile to be replaced by WriteFile */
-	if (CommWriteFile(serial->hComm, Stream_Pointer(irp->input), Length, &nbWritten, NULL))
+	if (CommWriteFile(serial->hComm, ptr, Length, &nbWritten, NULL))
 	{
 		irp->IoStatus = STATUS_SUCCESS;
 	}
@@ -327,7 +339,7 @@ static UINT serial_process_irp_write(SERIAL_DEVICE* serial, IRP* irp)
 		WLog_Print(serial->log, WLOG_DEBUG,
 		           "write failure to %s, nbWritten=%" PRIu32 ", last-error: 0x%08" PRIX32 "",
 		           serial->device.name, nbWritten, GetLastError());
-		irp->IoStatus = _GetLastErrorToIoStatus(serial);
+		irp->IoStatus = GetLastErrorToIoStatus(serial);
 	}
 
 	WLog_Print(serial->log, WLOG_DEBUG, "%" PRIu32 " bytes written to %s", nbWritten,
@@ -344,14 +356,17 @@ static UINT serial_process_irp_write(SERIAL_DEVICE* serial, IRP* irp)
  */
 static UINT serial_process_irp_device_control(SERIAL_DEVICE* serial, IRP* irp)
 {
-	UINT32 IoControlCode;
-	UINT32 InputBufferLength;
+	UINT32 IoControlCode = 0;
+	UINT32 InputBufferLength = 0;
 	BYTE* InputBuffer = NULL;
-	UINT32 OutputBufferLength;
+	UINT32 OutputBufferLength = 0;
 	BYTE* OutputBuffer = NULL;
 	DWORD BytesReturned = 0;
 
-	if (Stream_GetRemainingLength(irp->input) < 32)
+	WINPR_ASSERT(serial);
+	WINPR_ASSERT(irp);
+
+	if (!Stream_CheckAndLogRequiredLengthWLog(serial->log, irp->input, 32))
 		return ERROR_INVALID_DATA;
 
 	Stream_Read_UINT32(irp->input, OutputBufferLength); /* OutputBufferLength (4 bytes) */
@@ -359,7 +374,7 @@ static UINT serial_process_irp_device_control(SERIAL_DEVICE* serial, IRP* irp)
 	Stream_Read_UINT32(irp->input, IoControlCode);      /* IoControlCode (4 bytes) */
 	Stream_Seek(irp->input, 20);                        /* Padding (20 bytes) */
 
-	if (Stream_GetRemainingLength(irp->input) < InputBufferLength)
+	if (!Stream_CheckAndLogRequiredLengthWLog(serial->log, irp->input, InputBufferLength))
 		return ERROR_INVALID_DATA;
 
 	OutputBuffer = (BYTE*)calloc(OutputBufferLength, sizeof(BYTE));
@@ -398,21 +413,21 @@ static UINT serial_process_irp_device_control(SERIAL_DEVICE* serial, IRP* irp)
 		           "CommDeviceIoControl failure: IoControlCode=[0x%" PRIX32
 		           "] %s, last-error: 0x%08" PRIX32 "",
 		           IoControlCode, _comm_serial_ioctl_name(IoControlCode), GetLastError());
-		irp->IoStatus = _GetLastErrorToIoStatus(serial);
+		irp->IoStatus = GetLastErrorToIoStatus(serial);
 	}
 
 error_handle:
 	/* FIXME: find out whether it's required or not to get
 	 * BytesReturned == OutputBufferLength when
 	 * CommDeviceIoControl returns FALSE */
-	assert(OutputBufferLength == BytesReturned);
+	WINPR_ASSERT(OutputBufferLength == BytesReturned);
 	Stream_Write_UINT32(irp->output, BytesReturned); /* OutputBufferLength (4 bytes) */
 
 	if (BytesReturned > 0)
 	{
 		if (!Stream_EnsureRemainingCapacity(irp->output, BytesReturned))
 		{
-			WLog_ERR(TAG, "Stream_EnsureRemainingCapacity failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "Stream_EnsureRemainingCapacity failed!");
 			free(InputBuffer);
 			free(OutputBuffer);
 			return CHANNEL_RC_NO_MEMORY;
@@ -442,9 +457,12 @@ error_handle:
 static UINT serial_process_irp(SERIAL_DEVICE* serial, IRP* irp)
 {
 	UINT error = CHANNEL_RC_OK;
-	WLog_Print(serial->log, WLOG_DEBUG,
-	           "IRP MajorFunction: 0x%08" PRIX32 " MinorFunction: 0x%08" PRIX32 "\n",
-	           irp->MajorFunction, irp->MinorFunction);
+
+	WINPR_ASSERT(serial);
+	WINPR_ASSERT(irp);
+
+	WLog_Print(serial->log, WLOG_DEBUG, "IRP MajorFunction: %s, MinorFunction: 0x%08" PRIX32 "\n",
+	           rdpdr_irp_string(irp->MajorFunction), irp->MinorFunction);
 
 	switch (irp->MajorFunction)
 	{
@@ -457,9 +475,7 @@ static UINT serial_process_irp(SERIAL_DEVICE* serial, IRP* irp)
 			break;
 
 		case IRP_MJ_READ:
-			if ((error = serial_process_irp_read(serial, irp)))
-				WLog_ERR(TAG, "serial_process_irp_read failed with error %" PRIu32 "!", error);
-
+			error = serial_process_irp_read(serial, irp);
 			break;
 
 		case IRP_MJ_WRITE:
@@ -467,10 +483,7 @@ static UINT serial_process_irp(SERIAL_DEVICE* serial, IRP* irp)
 			break;
 
 		case IRP_MJ_DEVICE_CONTROL:
-			if ((error = serial_process_irp_device_control(serial, irp)))
-				WLog_ERR(TAG, "serial_process_irp_device_control failed with error %" PRIu32 "!",
-				         error);
-
+			error = serial_process_irp_device_control(serial, irp);
 			break;
 
 		default:
@@ -478,23 +491,37 @@ static UINT serial_process_irp(SERIAL_DEVICE* serial, IRP* irp)
 			break;
 	}
 
+	DWORD level = WLOG_TRACE;
+	if (error)
+		level = WLOG_WARN;
+
+	WLog_Print(serial->log, level,
+	           "[%s|0x%08" PRIx32 "] completed with %s [0x%08" PRIx32 "] (IoStatus %s [0x%08" PRIx32
+	           "])",
+	           rdpdr_irp_string(irp->MajorFunction), irp->MajorFunction, WTSErrorToString(error),
+	           error, NtStatus2Tag(irp->IoStatus), irp->IoStatus);
+
 	return error;
 }
 
 static DWORD WINAPI irp_thread_func(LPVOID arg)
 {
 	IRP_THREAD_DATA* data = (IRP_THREAD_DATA*)arg;
-	UINT error;
+	UINT error = 0;
+
+	WINPR_ASSERT(data);
+	WINPR_ASSERT(data->serial);
+	WINPR_ASSERT(data->irp);
 
 	/* blocks until the end of the request */
 	if ((error = serial_process_irp(data->serial, data->irp)))
 	{
-		WLog_ERR(TAG, "serial_process_irp failed with error %" PRIu32 "", error);
+		WLog_Print(data->serial->log, WLOG_ERROR,
+		           "serial_process_irp failed with error %" PRIu32 "", error);
 		goto error_out;
 	}
 
 	EnterCriticalSection(&data->serial->TerminatingIrpThreadsLock);
-	data->serial->IrpThreadToBeTerminatedCount++;
 	error = data->irp->Complete(data->irp);
 	LeaveCriticalSection(&data->serial->TerminatingIrpThreadsLock);
 error_out:
@@ -510,75 +537,71 @@ error_out:
 	return error;
 }
 
+static void close_unterminated_irp_thread(wListDictionary* list, wLog* log, ULONG_PTR id)
+{
+	WINPR_ASSERT(list);
+	HANDLE self = _GetCurrentThread();
+	HANDLE cirpThread = ListDictionary_GetItemValue(list, (void*)id);
+	if (self == cirpThread)
+		WLog_Print(log, WLOG_DEBUG, "Skipping termination of own IRP thread");
+	else
+		ListDictionary_Remove(list, (void*)id);
+}
+
+static void close_terminated_irp_thread(wListDictionary* list, wLog* log, ULONG_PTR id)
+{
+	WINPR_ASSERT(list);
+
+	HANDLE cirpThread = ListDictionary_GetItemValue(list, (void*)id);
+	/* FIXME: not quite sure a zero timeout is a good thing to check whether a thread is
+	 * stil alived or not */
+	const DWORD waitResult = WaitForSingleObject(cirpThread, 0);
+
+	if (waitResult == WAIT_OBJECT_0)
+		ListDictionary_Remove(list, (void*)id);
+	else if (waitResult != WAIT_TIMEOUT)
+	{
+		/* unexpected thread state */
+		WLog_Print(log, WLOG_WARN, "WaitForSingleObject, got an unexpected result=0x%" PRIX32 "\n",
+		           waitResult);
+	}
+}
+
+void close_terminated_irp_thread_handles(SERIAL_DEVICE* serial, BOOL forceClose)
+{
+	WINPR_ASSERT(serial);
+
+	EnterCriticalSection(&serial->TerminatingIrpThreadsLock);
+
+	ULONG_PTR* ids = NULL;
+	const size_t nbIds = ListDictionary_GetKeys(serial->IrpThreads, &ids);
+
+	for (size_t i = 0; i < nbIds; i++)
+	{
+		ULONG_PTR id = ids[i];
+		if (forceClose)
+			close_unterminated_irp_thread(serial->IrpThreads, serial->log, id);
+		else
+			close_terminated_irp_thread(serial->IrpThreads, serial->log, id);
+	}
+
+	free(ids);
+
+	LeaveCriticalSection(&serial->TerminatingIrpThreadsLock);
+}
+
 static void create_irp_thread(SERIAL_DEVICE* serial, IRP* irp)
 {
 	IRP_THREAD_DATA* data = NULL;
-	HANDLE irpThread;
-	HANDLE previousIrpThread;
-	uintptr_t key;
-	/* for a test/debug purpose, uncomment the code below to get a
-	 * single thread for all IRPs. NB: two IRPs could not be
-	 * processed at the same time, typically two concurent
-	 * Read/Write operations could block each other. */
-	/* serial_process_irp(serial, irp); */
-	/* irp->Complete(irp); */
-	/* return; */
-	/* NOTE: for good or bad, this implementation relies on the
-	 * server to avoid a flooding of requests. see also _purge().
-	 */
-	EnterCriticalSection(&serial->TerminatingIrpThreadsLock);
+	HANDLE irpThread = NULL;
+	HANDLE previousIrpThread = NULL;
+	uintptr_t key = 0;
 
-	while (serial->IrpThreadToBeTerminatedCount > 0)
-	{
-		/* Cleaning up termitating and pending irp
-		 * threads. See also: irp_thread_func() */
-		HANDLE irpThread;
-		ULONG_PTR* ids;
-		int i, nbIds;
-		nbIds = ListDictionary_GetKeys(serial->IrpThreads, &ids);
+	WINPR_ASSERT(serial);
+	WINPR_ASSERT(irp);
 
-		for (i = 0; i < nbIds; i++)
-		{
-			/* Checking if ids[i] is terminating or pending */
-			DWORD waitResult;
-			ULONG_PTR id = ids[i];
-			irpThread = ListDictionary_GetItemValue(serial->IrpThreads, (void*)id);
-			/* FIXME: not quite sure a zero timeout is a good thing to check whether a thread is
-			 * stil alived or not */
-			waitResult = WaitForSingleObject(irpThread, 0);
+	close_terminated_irp_thread_handles(serial, FALSE);
 
-			if (waitResult == WAIT_OBJECT_0)
-			{
-				/* terminating thread */
-				/* WLog_Print(serial->log, WLOG_DEBUG, "IRP thread with CompletionId=%"PRIuz"
-				 * naturally died", id); */
-				CloseHandle(irpThread);
-				ListDictionary_Remove(serial->IrpThreads, (void*)id);
-				serial->IrpThreadToBeTerminatedCount--;
-			}
-			else if (waitResult != WAIT_TIMEOUT)
-			{
-				/* unexpected thread state */
-				WLog_Print(serial->log, WLOG_WARN,
-				           "WaitForSingleObject, got an unexpected result=0x%" PRIX32 "\n",
-				           waitResult);
-				assert(FALSE);
-			}
-
-			/* pending thread (but not yet terminating thread) if waitResult == WAIT_TIMEOUT */
-		}
-
-		if (serial->IrpThreadToBeTerminatedCount > 0)
-		{
-			WLog_Print(serial->log, WLOG_DEBUG, "%" PRIu32 " IRP thread(s) not yet terminated",
-			           serial->IrpThreadToBeTerminatedCount);
-			Sleep(1); /* 1 ms */
-		}
-
-		free(ids);
-	}
-
-	LeaveCriticalSection(&serial->TerminatingIrpThreadsLock);
 	/* NB: At this point and thanks to the synchronization we're
 	 * sure that the incoming IRP uses well a recycled
 	 * CompletionId or the server sent again an IRP already posted
@@ -589,7 +612,7 @@ static void create_irp_thread(SERIAL_DEVICE* serial, IRP* irp)
 	 * FIXME: behavior documented somewhere? behavior not yet
 	 * observed with FreeRDP).
 	 */
-	key = irp->CompletionId;
+	key = irp->CompletionId + 1ull;
 	previousIrpThread = ListDictionary_GetItemValue(serial->IrpThreads, (void*)key);
 
 	if (previousIrpThread)
@@ -598,8 +621,8 @@ static void create_irp_thread(SERIAL_DEVICE* serial, IRP* irp)
 		WLog_Print(serial->log, WLOG_DEBUG,
 		           "IRP recall: IRP with the CompletionId=%" PRIu32 " not yet completed!",
 		           irp->CompletionId);
-		assert(FALSE); /* unimplemented */
-		/* TODO: asserts that previousIrpThread handles well
+		WINPR_ASSERT(FALSE); /* unimplemented */
+		/* TODO: WINPR_ASSERTs that previousIrpThread handles well
 		 * the same request by checking more details. Need an
 		 * access to the IRP object used by previousIrpThread
 		 */
@@ -619,14 +642,14 @@ static void create_irp_thread(SERIAL_DEVICE* serial, IRP* irp)
 	if (ListDictionary_Count(serial->IrpThreads) >= MAX_IRP_THREADS)
 	{
 		WLog_Print(serial->log, WLOG_WARN,
-		           "Number of IRP threads threshold reached: %d, keep on anyway",
+		           "Number of IRP threads threshold reached: %" PRIuz ", keep on anyway",
 		           ListDictionary_Count(serial->IrpThreads));
-		assert(FALSE); /* unimplemented */
-		               /* TODO: MAX_IRP_THREADS has been thought to avoid a
-		                * flooding of pending requests. Use
-		                * WaitForMultipleObjects() when available in winpr
-		                * for threads.
-		                */
+		WINPR_ASSERT(FALSE); /* unimplemented */
+		                     /* TODO: MAX_IRP_THREADS has been thought to avoid a
+		                      * flooding of pending requests. Use
+		                      * WaitForMultipleObjects() when available in winpr
+		                      * for threads.
+		                      */
 	}
 
 	/* error_handle to be used ... */
@@ -641,7 +664,7 @@ static void create_irp_thread(SERIAL_DEVICE* serial, IRP* irp)
 	data->serial = serial;
 	data->irp = irp;
 	/* data freed by irp_thread_func */
-	irpThread = CreateThread(NULL, 0, irp_thread_func, (void*)data, 0, NULL);
+	irpThread = CreateThread(NULL, 0, irp_thread_func, (void*)data, CREATE_SUSPENDED, NULL);
 
 	if (irpThread == INVALID_HANDLE_VALUE)
 	{
@@ -649,77 +672,52 @@ static void create_irp_thread(SERIAL_DEVICE* serial, IRP* irp)
 		goto error_handle;
 	}
 
-	key = irp->CompletionId;
+	key = irp->CompletionId + 1ull;
 
 	if (!ListDictionary_Add(serial->IrpThreads, (void*)key, irpThread))
 	{
-		WLog_ERR(TAG, "ListDictionary_Add failed!");
+		WLog_Print(serial->log, WLOG_ERROR, "ListDictionary_Add failed!");
 		goto error_handle;
 	}
 
+	ResumeThread(irpThread);
+
 	return;
 error_handle:
+	if (irpThread)
+		(void)CloseHandle(irpThread);
 	irp->IoStatus = STATUS_NO_MEMORY;
 	irp->Complete(irp);
 	free(data);
 }
 
-static void terminate_pending_irp_threads(SERIAL_DEVICE* serial)
-{
-	ULONG_PTR* ids;
-	int i, nbIds;
-	nbIds = ListDictionary_GetKeys(serial->IrpThreads, &ids);
-	WLog_Print(serial->log, WLOG_DEBUG, "Terminating %d IRP thread(s)", nbIds);
-
-	for (i = 0; i < nbIds; i++)
-	{
-		HANDLE irpThread;
-		ULONG_PTR id = ids[i];
-		irpThread = ListDictionary_GetItemValue(serial->IrpThreads, (void*)id);
-		TerminateThread(irpThread, 0);
-
-		if (WaitForSingleObject(irpThread, INFINITE) == WAIT_FAILED)
-		{
-			WLog_ERR(TAG, "WaitForSingleObject failed!");
-			continue;
-		}
-
-		CloseHandle(irpThread);
-		WLog_Print(serial->log, WLOG_DEBUG, "IRP thread terminated, CompletionId %p", (void*)id);
-	}
-
-	ListDictionary_Clear(serial->IrpThreads);
-	free(ids);
-}
-
 static DWORD WINAPI serial_thread_func(LPVOID arg)
 {
-	IRP* irp;
-	wMessage message;
+	IRP* irp = NULL;
+	wMessage message = { 0 };
 	SERIAL_DEVICE* serial = (SERIAL_DEVICE*)arg;
 	UINT error = CHANNEL_RC_OK;
 
+	WINPR_ASSERT(serial);
+
 	while (1)
 	{
 		if (!MessageQueue_Wait(serial->MainIrpQueue))
 		{
-			WLog_ERR(TAG, "MessageQueue_Wait failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "MessageQueue_Wait failed!");
 			error = ERROR_INTERNAL_ERROR;
 			break;
 		}
 
 		if (!MessageQueue_Peek(serial->MainIrpQueue, &message, TRUE))
 		{
-			WLog_ERR(TAG, "MessageQueue_Peek failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "MessageQueue_Peek failed!");
 			error = ERROR_INTERNAL_ERROR;
 			break;
 		}
 
 		if (message.id == WMQ_QUIT)
-		{
-			terminate_pending_irp_threads(serial);
 			break;
-		}
 
 		irp = (IRP*)message.wParam;
 
@@ -727,6 +725,7 @@ static DWORD WINAPI serial_thread_func(LPVOID arg)
 			create_irp_thread(serial, irp);
 	}
 
+	ListDictionary_Clear(serial->IrpThreads);
 	if (error && serial->rdpcontext)
 		setChannelError(serial->rdpcontext, error, "serial_thread_func reported an error");
 
@@ -742,7 +741,8 @@ static DWORD WINAPI serial_thread_func(LPVOID arg)
 static UINT serial_irp_request(DEVICE* device, IRP* irp)
 {
 	SERIAL_DEVICE* serial = (SERIAL_DEVICE*)device;
-	assert(irp != NULL);
+	WINPR_ASSERT(irp != NULL);
+	WINPR_ASSERT(serial);
 
 	if (irp == NULL)
 		return CHANNEL_RC_OK;
@@ -754,7 +754,7 @@ static UINT serial_irp_request(DEVICE* device, IRP* irp)
 
 	if (!MessageQueue_Post(serial->MainIrpQueue, NULL, 0, (void*)irp, NULL))
 	{
-		WLog_ERR(TAG, "MessageQueue_Post failed!");
+		WLog_Print(serial->log, WLOG_ERROR, "MessageQueue_Post failed!");
 		return ERROR_INTERNAL_ERROR;
 	}
 
@@ -768,22 +768,28 @@ static UINT serial_irp_request(DEVICE* device, IRP* irp)
  */
 static UINT serial_free(DEVICE* device)
 {
-	UINT error;
+	UINT error = 0;
 	SERIAL_DEVICE* serial = (SERIAL_DEVICE*)device;
-	WLog_Print(serial->log, WLOG_DEBUG, "freeing");
-	MessageQueue_PostQuit(serial->MainIrpQueue, 0);
+	if (!serial)
+		return CHANNEL_RC_OK;
 
-	if (WaitForSingleObject(serial->MainThread, INFINITE) == WAIT_FAILED)
+	WLog_Print(serial->log, WLOG_DEBUG, "freeing");
+	if (serial->MainIrpQueue)
+		MessageQueue_PostQuit(serial->MainIrpQueue, 0);
+
+	if (serial->MainThread)
 	{
-		error = GetLastError();
-		WLog_ERR(TAG, "WaitForSingleObject failed with error %" PRIu32 "!", error);
-		return error;
+		if (WaitForSingleObject(serial->MainThread, INFINITE) == WAIT_FAILED)
+		{
+			error = GetLastError();
+			WLog_Print(serial->log, WLOG_ERROR,
+			           "WaitForSingleObject failed with error %" PRIu32 "!", error);
+		}
+		(void)CloseHandle(serial->MainThread);
 	}
 
-	CloseHandle(serial->MainThread);
-
 	if (serial->hComm)
-		CloseHandle(serial->hComm);
+		(void)CloseHandle(*serial->hComm);
 
 	/* Clean up resources */
 	Stream_Free(serial->device.data, TRUE);
@@ -794,56 +800,76 @@ static UINT serial_free(DEVICE* device)
 	return CHANNEL_RC_OK;
 }
 
-#endif /* __linux__ */
+static void serial_message_free(void* obj)
+{
+	wMessage* msg = obj;
+	if (!msg)
+		return;
+	if (msg->id != 0)
+		return;
 
-#ifdef BUILTIN_CHANNELS
-#define DeviceServiceEntry serial_DeviceServiceEntry
-#else
-#define DeviceServiceEntry FREERDP_API DeviceServiceEntry
-#endif
+	IRP* irp = (IRP*)msg->wParam;
+	if (!irp)
+		return;
+	WINPR_ASSERT(irp->Discard);
+	irp->Discard(irp);
+}
+
+static void irp_thread_close(void* arg)
+{
+	HANDLE hdl = arg;
+	if (hdl)
+	{
+		HANDLE thz = _GetCurrentThread();
+		if (thz == hdl)
+			WLog_WARN(TAG, "closing self, ignoring...");
+		else
+		{
+			(void)TerminateThread(hdl, 0);
+			(void)WaitForSingleObject(hdl, INFINITE);
+			(void)CloseHandle(hdl);
+		}
+	}
+}
 
 /**
  * Function description
  *
  * @return 0 on success, otherwise a Win32 error code
  */
-UINT DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints)
+FREERDP_ENTRY_POINT(
+    UINT VCAPITYPE serial_DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints))
 {
-	char* name;
-	char* path;
-	char* driver;
-	RDPDR_SERIAL* device;
-#if defined __linux__ && !defined ANDROID
-	size_t i, len;
-	SERIAL_DEVICE* serial;
-#endif /* __linux__ */
+	size_t len = 0;
+	SERIAL_DEVICE* serial = NULL;
 	UINT error = CHANNEL_RC_OK;
-	device = (RDPDR_SERIAL*)pEntryPoints->device;
-	name = device->Name;
-	path = device->Path;
-	driver = device->Driver;
+
+	WINPR_ASSERT(pEntryPoints);
+
+	RDPDR_SERIAL* device = (RDPDR_SERIAL*)pEntryPoints->device;
+	WINPR_ASSERT(device);
+
+	wLog* log = WLog_Get(TAG);
+	const char* name = device->device.Name;
+	const char* path = device->Path;
+	const char* driver = device->Driver;
 
 	if (!name || (name[0] == '*'))
 	{
 		/* TODO: implement auto detection of serial ports */
+		WLog_Print(log, WLOG_WARN,
+		           "Serial port autodetection not implemented, nothing will be redirected!");
 		return CHANNEL_RC_OK;
 	}
 
 	if ((name && name[0]) && (path && path[0]))
 	{
-		wLog* log;
-		log = WLog_Get("com.freerdp.channel.serial.client");
-		WLog_Print(log, WLOG_DEBUG, "initializing");
-#ifndef __linux__ /* to be removed */
-		WLog_Print(log, WLOG_WARN, "Serial ports redirection not supported on this platform.");
-		return CHANNEL_RC_INITIALIZATION_ERROR;
-#else /* __linux __ */
 		WLog_Print(log, WLOG_DEBUG, "Defining %s as %s", name, path);
 
 		if (!DefineCommDevice(name /* eg: COM1 */, path /* eg: /dev/ttyS0 */))
 		{
 			DWORD status = GetLastError();
-			WLog_ERR(TAG, "DefineCommDevice failed with %08" PRIx32, status);
+			WLog_Print(log, WLOG_ERROR, "DefineCommDevice failed with %08" PRIx32, status);
 			return ERROR_INTERNAL_ERROR;
 		}
 
@@ -851,7 +877,7 @@ UINT DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints)
 
 		if (!serial)
 		{
-			WLog_ERR(TAG, "calloc failed!");
+			WLog_Print(log, WLOG_ERROR, "calloc failed!");
 			return CHANNEL_RC_NO_MEMORY;
 		}
 
@@ -866,12 +892,12 @@ UINT DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints)
 
 		if (!serial->device.data)
 		{
-			WLog_ERR(TAG, "calloc failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "calloc failed!");
 			error = CHANNEL_RC_NO_MEMORY;
 			goto error_out;
 		}
 
-		for (i = 0; i <= len; i++)
+		for (size_t i = 0; i <= len; i++)
 			Stream_Write_UINT8(serial->device.data, name[i] < 0 ? '_' : name[i]);
 
 		if (driver != NULL)
@@ -884,10 +910,10 @@ UINT DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints)
 				serial->ServerSerialDriverId = SerialDriverSerCx2Sys;
 			else
 			{
-				assert(FALSE);
-				WLog_Print(serial->log, WLOG_DEBUG,
-				           "Unknown server's serial driver: %s. SerCx2 will be used", driver);
-				serial->ServerSerialDriverId = SerialDriverSerialSys;
+				WLog_Print(serial->log, WLOG_WARN, "Unknown server's serial driver: %s.", driver);
+				WLog_Print(serial->log, WLOG_WARN,
+				           "Valid options are: 'Serial' (default), 'SerCx' and 'SerCx2'");
+				goto error_out;
 			}
 		}
 		else
@@ -904,60 +930,67 @@ UINT DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints)
 			}
 			else
 			{
-				WLog_Print(serial->log, WLOG_DEBUG, "Unknown flag: %s", device->Permissive);
-				assert(FALSE);
+				WLog_Print(serial->log, WLOG_WARN, "Unknown flag: %s", device->Permissive);
+				goto error_out;
 			}
 		}
 
 		WLog_Print(serial->log, WLOG_DEBUG, "Server's serial driver: %s (id: %d)", driver,
 		           serial->ServerSerialDriverId);
-		/* TODO: implement auto detection of the server's serial driver */
+
 		serial->MainIrpQueue = MessageQueue_New(NULL);
 
 		if (!serial->MainIrpQueue)
 		{
-			WLog_ERR(TAG, "MessageQueue_New failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "MessageQueue_New failed!");
 			error = CHANNEL_RC_NO_MEMORY;
 			goto error_out;
 		}
 
+		{
+			wObject* obj = MessageQueue_Object(serial->MainIrpQueue);
+			WINPR_ASSERT(obj);
+			obj->fnObjectFree = serial_message_free;
+		}
+
 		/* IrpThreads content only modified by create_irp_thread() */
 		serial->IrpThreads = ListDictionary_New(FALSE);
 
 		if (!serial->IrpThreads)
 		{
-			WLog_ERR(TAG, "ListDictionary_New failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "ListDictionary_New failed!");
 			error = CHANNEL_RC_NO_MEMORY;
 			goto error_out;
 		}
 
-		serial->IrpThreadToBeTerminatedCount = 0;
+		{
+			wObject* obj = ListDictionary_ValueObject(serial->IrpThreads);
+			WINPR_ASSERT(obj);
+			obj->fnObjectFree = irp_thread_close;
+		}
+
 		InitializeCriticalSection(&serial->TerminatingIrpThreadsLock);
 
-		if ((error = pEntryPoints->RegisterDevice(pEntryPoints->devman, (DEVICE*)serial)))
+		error = pEntryPoints->RegisterDevice(pEntryPoints->devman, &serial->device);
+		if (error != CHANNEL_RC_OK)
 		{
-			WLog_ERR(TAG, "EntryPoints->RegisterDevice failed with error %" PRIu32 "!", error);
+			WLog_Print(serial->log, WLOG_ERROR,
+			           "EntryPoints->RegisterDevice failed with error %" PRIu32 "!", error);
 			goto error_out;
 		}
 
-		if (!(serial->MainThread =
-		          CreateThread(NULL, 0, serial_thread_func, (void*)serial, 0, NULL)))
+		serial->MainThread = CreateThread(NULL, 0, serial_thread_func, serial, 0, NULL);
+		if (!serial->MainThread)
 		{
-			WLog_ERR(TAG, "CreateThread failed!");
+			WLog_Print(serial->log, WLOG_ERROR, "CreateThread failed!");
 			error = ERROR_INTERNAL_ERROR;
 			goto error_out;
 		}
-
-#endif /* __linux __ */
 	}
 
 	return error;
 error_out:
-#ifdef __linux__ /* to be removed */
-	ListDictionary_Free(serial->IrpThreads);
-	MessageQueue_Free(serial->MainIrpQueue);
-	Stream_Free(serial->device.data, TRUE);
-	free(serial);
-#endif /* __linux __ */
+	if (serial)
+		serial_free(&serial->device);
 	return error;
 }
