@@ -139,7 +139,7 @@ static BOOL log_libusb_result_(wLog* log, DWORD lvl, WINPR_FORMAT_ARG const char
 	if (error < 0)
 	{
 		char buffer[8192] = { 0 };
-		va_list ap;
+		va_list ap = { 0 };
 		va_start(ap, error);
 		(void)vsnprintf(buffer, sizeof(buffer), fmt, ap);
 		va_end(ap);
@@ -461,7 +461,9 @@ static int func_config_release_all_interface(URBDRC_PLUGIN* urbdrc,
                                              LIBUSB_DEVICE_HANDLE* libusb_handle,
                                              UINT32 NumInterfaces)
 {
-	for (UINT32 i = 0; i < NumInterfaces; i++)
+	if (NumInterfaces > INT32_MAX)
+		return -1;
+	for (INT32 i = 0; i < (INT32)NumInterfaces; i++)
 	{
 		int ret = libusb_release_interface(libusb_handle, i);
 
@@ -864,7 +866,7 @@ static UINT32 libusb_udev_control_query_device_text(IUDEVICE* idev, UINT32 TextT
 				           msg, ret, devDescriptor->iProduct);
 
 				len = MIN(sizeof(strDesc), inSize);
-				for (ssize_t i = 0; i < len; i++)
+				for (size_t i = 0; i < len; i++)
 					text[i] = (WCHAR)strDesc[i];
 
 				*BufferSize = (BYTE)(len * 2);
@@ -899,7 +901,7 @@ static UINT32 libusb_udev_control_query_device_text(IUDEVICE* idev, UINT32 TextT
 
 			len = strnlen(deviceLocation,
 			              MIN(sizeof(deviceLocation), (inSize > 0) ? inSize - 1U : 0));
-			for (ssize_t i = 0; i < len; i++)
+			for (size_t i = 0; i < len; i++)
 				text[i] = (WCHAR)deviceLocation[i];
 			text[len++] = '\0';
 			*BufferSize = (UINT8)(len * sizeof(WCHAR));
