@@ -17,18 +17,22 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <freerdp/config.h>
 
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdint.h>
 
+#include <winpr/assert.h>
 #include <winpr/crt.h>
 #include <winpr/path.h>
+#include <winpr/assert.h>
 #include <winpr/collections.h>
 
+#include <freerdp/utils/string.h>
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -40,53 +44,78 @@
 
 #include "xf_keyboard.h"
 
+#include "xf_utils.h"
+
 #include <freerdp/log.h>
 #define TAG CLIENT_TAG("x11")
 
-static BOOL firstPressRightCtrl = TRUE;
-static BOOL ungrabKeyboardWithRightCtrl = TRUE;
+static void xf_keyboard_modifier_map_free(xfContext* xfc)
+{
+	WINPR_ASSERT(xfc);
+	if (xfc->modifierMap)
+	{
+		XFreeModifiermap(xfc->modifierMap);
+		xfc->modifierMap = NULL;
+	}
+}
+
+BOOL xf_keyboard_update_modifier_map(xfContext* xfc)
+{
+	WINPR_ASSERT(xfc);
+	xf_keyboard_modifier_map_free(xfc);
+	xfc->modifierMap = XGetModifierMapping(xfc->display);
+	return xfc->modifierMap != NULL;
+}
+
+static void xf_keyboard_send_key(xfContext* xfc, BOOL down, BOOL repeat, const XKeyEvent* ev);
+
+static BOOL xf_sync_kbd_state(xfContext* xfc)
+{
+	const UINT32 syncFlags = xf_keyboard_get_toggle_keys_state(xfc);
+
+	WINPR_ASSERT(xfc);
+	return freerdp_input_send_synchronize_event(xfc->common.context.input, syncFlags);
+}
+
+static void xf_keyboard_clear(xfContext* xfc)
+{
+	WINPR_ASSERT(xfc);
+	ZeroMemory(xfc->KeyboardState, sizeof(xfc->KeyboardState));
+}
+
+static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t size, void* user,
+                                    const char* what, const char* arg)
+{
+	WINPR_ASSERT(xfc);
+	WINPR_UNUSED(what);
+	WINPR_UNUSED(arg);
+
+	if (!buffer || (size == 0))
+		return TRUE;
+	return ArrayList_Append(xfc->keyCombinations, buffer);
+}
 
 static BOOL xf_keyboard_action_script_init(xfContext* xfc)
 {
-	FILE* keyScript;
-	char* keyCombination;
-	char buffer[1024] = { 0 };
-	char command[1024] = { 0 };
-	xfc->actionScriptExists = PathFileExistsA(xfc->context.settings->ActionScript);
+	wObject* obj = NULL;
+	const rdpSettings* settings = NULL;
 
-	if (!xfc->actionScriptExists)
-		return FALSE;
+	settings = xfc->common.context.settings;
+	WINPR_ASSERT(settings);
 
 	xfc->keyCombinations = ArrayList_New(TRUE);
 
 	if (!xfc->keyCombinations)
 		return FALSE;
 
-	ArrayList_Object(xfc->keyCombinations)->fnObjectFree = free;
-	sprintf_s(command, sizeof(command), "%s key", xfc->context.settings->ActionScript);
-	keyScript = popen(command, "r");
+	obj = ArrayList_Object(xfc->keyCombinations);
+	WINPR_ASSERT(obj);
+	obj->fnObjectNew = winpr_ObjectStringClone;
+	obj->fnObjectFree = winpr_ObjectStringFree;
 
-	if (!keyScript)
-	{
-		xfc->actionScriptExists = FALSE;
+	if (!run_action_script(xfc, "key", NULL, xf_action_script_append, NULL))
 		return FALSE;
-	}
 
-	while (fgets(buffer, sizeof(buffer), keyScript) != NULL)
-	{
-		strtok(buffer, "\n");
-		keyCombination = _strdup(buffer);
-
-		if (!keyCombination || ArrayList_Add(xfc->keyCombinations, keyCombination) < 0)
-		{
-			ArrayList_Free(xfc->keyCombinations);
-			xfc->actionScriptExists = FALSE;
-			pclose(keyScript);
-			return FALSE;
-		}
-	}
-
-	pclose(keyScript);
 	return xf_event_action_script_init(xfc);
 }
 
@@ -104,15 +133,21 @@ static void xf_keyboard_action_script_free(xfContext* xfc)
 
 BOOL xf_keyboard_init(xfContext* xfc)
 {
+	rdpSettings* settings = NULL;
+
+	WINPR_ASSERT(xfc);
+
+	settings = xfc->common.context.settings;
+	WINPR_ASSERT(settings);
+
 	xf_keyboard_clear(xfc);
-	xfc->KeyboardLayout = xfc->context.settings->KeyboardLayout;
-	xfc->KeyboardLayout = freerdp_keyboard_init(xfc->KeyboardLayout);
-	xfc->context.settings->KeyboardLayout = xfc->KeyboardLayout;
+	xfc->KeyboardLayout = freerdp_settings_get_uint32(settings, FreeRDP_KeyboardLayout);
+	xfc->KeyboardLayout = freerdp_keyboard_init_ex(
+	    xfc->KeyboardLayout, freerdp_settings_get_string(settings, FreeRDP_KeyboardRemappingList));
+	if (!freerdp_settings_set_uint32(settings, FreeRDP_KeyboardLayout, xfc->KeyboardLayout))
+		return FALSE;
 
-	if (xfc->modifierMap)
-		XFreeModifiermap(xfc->modifierMap);
-
-	if (!(xfc->modifierMap = XGetModifierMapping(xfc->display)))
+	if (!xf_keyboard_update_modifier_map(xfc))
 		return FALSE;
 
 	xf_keyboard_action_script_init(xfc);
@@ -121,84 +156,85 @@ BOOL xf_keyboard_init(xfContext* xfc)
 
 void xf_keyboard_free(xfContext* xfc)
 {
-	if (xfc->modifierMap)
-	{
-		XFreeModifiermap(xfc->modifierMap);
-		xfc->modifierMap = NULL;
-	}
-
+	xf_keyboard_modifier_map_free(xfc);
 	xf_keyboard_action_script_free(xfc);
 }
 
-void xf_keyboard_clear(xfContext* xfc)
+void xf_keyboard_key_press(xfContext* xfc, const XKeyEvent* event, KeySym keysym)
 {
-	ZeroMemory(xfc->KeyboardState, 256 * sizeof(BOOL));
-}
+	BOOL last = 0;
 
-void xf_keyboard_key_press(xfContext* xfc, BYTE keycode, KeySym keysym)
-{
-	if (keycode < 8)
-		return;
+	WINPR_ASSERT(xfc);
+	WINPR_ASSERT(event);
+	WINPR_ASSERT(event->keycode < ARRAYSIZE(xfc->KeyboardState));
 
-	xfc->KeyboardState[keycode] = TRUE;
+	last = xfc->KeyboardState[event->keycode];
+	xfc->KeyboardState[event->keycode] = TRUE;
 
 	if (xf_keyboard_handle_special_keys(xfc, keysym))
 		return;
 
-	xf_keyboard_send_key(xfc, TRUE, keycode);
+	xf_keyboard_send_key(xfc, TRUE, last, event);
 }
 
-void xf_keyboard_key_release(xfContext* xfc, BYTE keycode, KeySym keysym)
+void xf_keyboard_key_release(xfContext* xfc, const XKeyEvent* event, KeySym keysym)
 {
-	if (keycode < 8)
-		return;
+	WINPR_ASSERT(xfc);
+	WINPR_ASSERT(event);
+	WINPR_ASSERT(event->keycode < ARRAYSIZE(xfc->KeyboardState));
 
-	xfc->KeyboardState[keycode] = FALSE;
+	BOOL last = xfc->KeyboardState[event->keycode];
+	xfc->KeyboardState[event->keycode] = FALSE;
 	xf_keyboard_handle_special_keys_release(xfc, keysym);
-	xf_keyboard_send_key(xfc, FALSE, keycode);
+	xf_keyboard_send_key(xfc, FALSE, last, event);
 }
 
 void xf_keyboard_release_all_keypress(xfContext* xfc)
 {
-	size_t keycode;
-	DWORD rdp_scancode;
+	WINPR_ASSERT(xfc);
 
-	for (keycode = 0; keycode < ARRAYSIZE(xfc->KeyboardState); keycode++)
+	WINPR_STATIC_ASSERT(ARRAYSIZE(xfc->KeyboardState) <= UINT32_MAX);
+	for (size_t keycode = 0; keycode < ARRAYSIZE(xfc->KeyboardState); keycode++)
 	{
 		if (xfc->KeyboardState[keycode])
 		{
-			rdp_scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(keycode);
+			const DWORD rdp_scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(keycode);
 
 			// release tab before releasing the windows key.
 			// this stops the start menu from opening on unfocus event.
 			if (rdp_scancode == RDP_SCANCODE_LWIN)
-				freerdp_input_send_keyboard_event_ex(xfc->context.input, FALSE, RDP_SCANCODE_TAB);
+				freerdp_input_send_keyboard_event_ex(xfc->common.context.input, FALSE, FALSE,
+				                                     RDP_SCANCODE_TAB);
 
-			freerdp_input_send_keyboard_event_ex(xfc->context.input, FALSE, rdp_scancode);
+			freerdp_input_send_keyboard_event_ex(xfc->common.context.input, FALSE, FALSE,
+			                                     rdp_scancode);
 			xfc->KeyboardState[keycode] = FALSE;
 		}
 	}
+	xf_sync_kbd_state(xfc);
 }
 
 BOOL xf_keyboard_key_pressed(xfContext* xfc, KeySym keysym)
 {
 	KeyCode keycode = XKeysymToKeycode(xfc->display, keysym);
+	WINPR_ASSERT(keycode < ARRAYSIZE(xfc->KeyboardState));
 	return xfc->KeyboardState[keycode];
 }
 
-void xf_keyboard_send_key(xfContext* xfc, BOOL down, BYTE keycode)
+void xf_keyboard_send_key(xfContext* xfc, BOOL down, BOOL repeat, const XKeyEvent* event)
 {
-	DWORD rdp_scancode;
-	rdpInput* input;
-	input = xfc->context.input;
-	rdp_scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(keycode);
+	DWORD rdp_scancode = 0;
+	rdpInput* input = NULL;
 
-	if (rdp_scancode == RDP_SCANCODE_UNKNOWN)
-	{
-		WLog_ERR(TAG, "Unknown key with X keycode 0x%02" PRIx8 "", keycode);
-	}
-	else if (rdp_scancode == RDP_SCANCODE_PAUSE && !xf_keyboard_key_pressed(xfc, XK_Control_L) &&
-	         !xf_keyboard_key_pressed(xfc, XK_Control_R))
+	WINPR_ASSERT(xfc);
+	WINPR_ASSERT(event);
+
+	input = xfc->common.context.input;
+	WINPR_ASSERT(input);
+
+	rdp_scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(event->keycode);
+	if (rdp_scancode == RDP_SCANCODE_PAUSE && !xf_keyboard_key_pressed(xfc, XK_Control_L) &&
+	    !xf_keyboard_key_pressed(xfc, XK_Control_R))
 	{
 		/* Pause without Ctrl has to be sent as a series of keycodes
 		 * in a single input PDU.  Pause only happens on "press";
@@ -211,24 +247,61 @@ void xf_keyboard_send_key(xfContext* xfc, BOOL down, BYTE keycode)
 	}
 	else
 	{
-		freerdp_input_send_keyboard_event_ex(input, down, rdp_scancode);
+		if (freerdp_settings_get_bool(xfc->common.context.settings, FreeRDP_UnicodeInput))
+		{
+			wchar_t buffer[32] = { 0 };
+			int xwc = -1;
+
+			switch (rdp_scancode)
+			{
+				case RDP_SCANCODE_RETURN:
+					break;
+				default:
+				{
+					XIM xim = XOpenIM(xfc->display, 0, 0, 0);
+					XIC xic =
+					    XCreateIC(xim, XNInputStyle, XIMPreeditNothing | XIMStatusNothing, NULL);
+
+					KeySym ignore = { 0 };
+					Status return_status = 0;
+					XKeyEvent ev = *event;
+					ev.type = KeyPress;
+					xwc = XwcLookupString(xic, &ev, buffer, ARRAYSIZE(buffer), &ignore,
+					                      &return_status);
+				}
+				break;
+			}
+
+			if (xwc < 1)
+			{
+				if (rdp_scancode == RDP_SCANCODE_UNKNOWN)
+					WLog_ERR(TAG, "Unknown key with X keycode 0x%02" PRIx8 "", event->keycode);
+				else
+					freerdp_input_send_keyboard_event_ex(input, down, repeat, rdp_scancode);
+			}
+			else
+				freerdp_input_send_unicode_keyboard_event(input, down ? 0 : KBD_FLAGS_RELEASE,
+				                                          buffer[0]);
+		}
+		else if (rdp_scancode == RDP_SCANCODE_UNKNOWN)
+			WLog_ERR(TAG, "Unknown key with X keycode 0x%02" PRIx8 "", event->keycode);
+		else
+			freerdp_input_send_keyboard_event_ex(input, down, repeat, rdp_scancode);
 
 		if ((rdp_scancode == RDP_SCANCODE_CAPSLOCK) && (down == FALSE))
 		{
-			UINT32 syncFlags;
-			syncFlags = xf_keyboard_get_toggle_keys_state(xfc);
-			input->SynchronizeEvent(input, syncFlags);
+			xf_sync_kbd_state(xfc);
 		}
 	}
 }
 
 int xf_keyboard_read_keyboard_state(xfContext* xfc)
 {
-	int dummy;
-	Window wdummy;
+	int dummy = 0;
+	Window wdummy = 0;
 	UINT32 state = 0;
 
-	if (!xfc->remote_app)
+	if (!xfc->remote_app && xfc->window)
 	{
 		XQueryPointer(xfc->display, xfc->window->handle, &wdummy, &wdummy, &dummy, &dummy, &dummy,
 		              &dummy, &state);
@@ -244,17 +317,18 @@ int xf_keyboard_read_keyboard_state(xfContext* xfc)
 
 static int xf_keyboard_get_keymask(xfContext* xfc, int keysym)
 {
-	int modifierpos, key, keysymMask = 0;
+	int keysymMask = 0;
 	KeyCode keycode = XKeysymToKeycode(xfc->display, keysym);
 
 	if (keycode == NoSymbol)
 		return 0;
 
-	for (modifierpos = 0; modifierpos < 8; modifierpos++)
+	WINPR_ASSERT(xfc->modifierMap);
+	for (int modifierpos = 0; modifierpos < 8; modifierpos++)
 	{
 		int offset = xfc->modifierMap->max_keypermod * modifierpos;
 
-		for (key = 0; key < xfc->modifierMap->max_keypermod; key++)
+		for (int key = 0; key < xfc->modifierMap->max_keypermod; key++)
 		{
 			if (xfc->modifierMap->modifiermap[offset + key] == keycode)
 			{
@@ -278,12 +352,10 @@ BOOL xf_keyboard_get_key_state(xfContext* xfc, int state, int keysym)
 
 static BOOL xf_keyboard_set_key_state(xfContext* xfc, BOOL on, int keysym)
 {
-	int keysymMask;
-
 	if (!xfc->xkbAvailable)
 		return FALSE;
 
-	keysymMask = xf_keyboard_get_keymask(xfc, keysym);
+	const int keysymMask = xf_keyboard_get_keymask(xfc, keysym);
 
 	if (!keysymMask)
 	{
@@ -295,9 +367,8 @@ static BOOL xf_keyboard_set_key_state(xfContext* xfc, BOOL on, int keysym)
 
 UINT32 xf_keyboard_get_toggle_keys_state(xfContext* xfc)
 {
-	int state;
 	UINT32 toggleKeysState = 0;
-	state = xf_keyboard_read_keyboard_state(xfc);
+	const int state = xf_keyboard_read_keyboard_state(xfc);
 
 	if (xf_keyboard_get_key_state(xfc, state, XK_Scroll_Lock))
 		toggleKeysState |= KBD_SYNC_SCROLL_LOCK;
@@ -316,21 +387,19 @@ UINT32 xf_keyboard_get_toggle_keys_state(xfContext* xfc)
 
 static void xk_keyboard_update_modifier_keys(xfContext* xfc)
 {
-	int state;
-	size_t i;
-	KeyCode keycode;
-	int keysyms[] = { XK_Shift_L,   XK_Shift_R,   XK_Alt_L,   XK_Alt_R,
-		              XK_Control_L, XK_Control_R, XK_Super_L, XK_Super_R };
+	const int keysyms[] = { XK_Shift_L,   XK_Shift_R,   XK_Alt_L,   XK_Alt_R,
+		                    XK_Control_L, XK_Control_R, XK_Super_L, XK_Super_R };
 
 	xf_keyboard_clear(xfc);
 
-	state = xf_keyboard_read_keyboard_state(xfc);
+	const int state = xf_keyboard_read_keyboard_state(xfc);
 
-	for (i = 0; i < ARRAYSIZE(keysyms); i++)
+	for (size_t i = 0; i < ARRAYSIZE(keysyms); i++)
 	{
 		if (xf_keyboard_get_key_state(xfc, state, keysyms[i]))
 		{
-			keycode = XKeysymToKeycode(xfc->display, keysyms[i]);
+			const KeyCode keycode = XKeysymToKeycode(xfc->display, keysyms[i]);
+			WINPR_ASSERT(keycode < ARRAYSIZE(xfc->KeyboardState));
 			xfc->KeyboardState[keycode] = TRUE;
 		}
 	}
@@ -338,44 +407,83 @@ static void xk_keyboard_update_modifier_keys(xfContext* xfc)
 
 void xf_keyboard_focus_in(xfContext* xfc)
 {
-	rdpInput* input;
-	UINT32 syncFlags, state;
-	Window w;
-	int d, x, y;
+	UINT32 state = 0;
+	Window w = None;
+	int d = 0;
+	int x = 0;
+	int y = 0;
 
+	WINPR_ASSERT(xfc);
 	if (!xfc->display || !xfc->window)
 		return;
 
-	input = xfc->context.input;
-	syncFlags = xf_keyboard_get_toggle_keys_state(xfc);
-	input->FocusInEvent(input, syncFlags);
+	rdpInput* input = xfc->common.context.input;
+	WINPR_ASSERT(input);
+
+	const UINT32 syncFlags = xf_keyboard_get_toggle_keys_state(xfc);
+	freerdp_input_send_focus_in_event(input, syncFlags);
 	xk_keyboard_update_modifier_keys(xfc);
 
 	/* finish with a mouse pointer position like mstsc.exe if required */
 
-	if (xfc->remote_app)
+	if (xfc->remote_app || !xfc->window)
 		return;
 
 	if (XQueryPointer(xfc->display, xfc->window->handle, &w, &w, &d, &d, &x, &y, &state))
 	{
-		if (x >= 0 && x < xfc->window->width && y >= 0 && y < xfc->window->height)
+		if ((x >= 0) && (x < xfc->window->width) && (y >= 0) && (y < xfc->window->height))
 		{
 			xf_event_adjust_coordinates(xfc, &x, &y);
-			input->MouseEvent(input, PTR_FLAGS_MOVE, x, y);
+			freerdp_client_send_button_event(&xfc->common, FALSE, PTR_FLAGS_MOVE, x, y);
 		}
 	}
 }
 
+static BOOL action_script_run(xfContext* xfc, const char* buffer, size_t size, void* user,
+                              const char* what, const char* arg)
+{
+	WINPR_UNUSED(xfc);
+	WINPR_UNUSED(what);
+	WINPR_UNUSED(arg);
+	WINPR_ASSERT(user);
+	int* pstatus = user;
+
+	if (size == 0)
+	{
+		WLog_WARN(TAG, "ActionScript key: script did not return data");
+		return FALSE;
+	}
+
+	if (strcmp(buffer, "key-local") == 0)
+		*pstatus = 0;
+	else if (winpr_PathFileExists(buffer))
+	{
+		FILE* fp = popen(buffer, "w");
+		if (!fp)
+		{
+			WLog_ERR(TAG, "Failed to execute '%s'", buffer);
+			return FALSE;
+		}
+
+		*pstatus = pclose(fp);
+		if (*pstatus < 0)
+		{
+			WLog_ERR(TAG, "Command '%s' returned %d", buffer, *pstatus);
+			return FALSE;
+		}
+	}
+	else
+	{
+		WLog_WARN(TAG, "ActionScript key: no such file '%s'", buffer);
+		return FALSE;
+	}
+	return TRUE;
+}
+
 static int xf_keyboard_execute_action_script(xfContext* xfc, XF_MODIFIER_KEYS* mod, KeySym keysym)
 {
-	int index;
-	int count;
 	int status = 1;
-	FILE* keyScript;
-	const char* keyStr;
 	BOOL match = FALSE;
-	char* keyCombination;
-	char buffer[1024] = { 0 };
 	char command[2048] = { 0 };
 	char combination[1024] = { 0 };
 
@@ -388,7 +496,7 @@ static int xf_keyboard_execute_action_script(xfContext* xfc, XF_MODIFIER_KEYS* m
 		return 1;
 	}
 
-	keyStr = XKeysymToString(keysym);
+	const char* keyStr = XKeysymToString(keysym);
 
 	if (keyStr == 0)
 	{
@@ -396,23 +504,27 @@ static int xf_keyboard_execute_action_script(xfContext* xfc, XF_MODIFIER_KEYS* m
 	}
 
 	if (mod->Shift)
-		strcat(combination, "Shift+");
+		winpr_str_append("Shift", combination, sizeof(combination), "+");
 
 	if (mod->Ctrl)
-		strcat(combination, "Ctrl+");
+		winpr_str_append("Ctrl", combination, sizeof(combination), "+");
 
 	if (mod->Alt)
-		strcat(combination, "Alt+");
+		winpr_str_append("Alt", combination, sizeof(combination), "+");
 
 	if (mod->Super)
-		strcat(combination, "Super+");
+		winpr_str_append("Super", combination, sizeof(combination), "+");
 
-	strcat(combination, keyStr);
-	count = ArrayList_Count(xfc->keyCombinations);
+	winpr_str_append(keyStr, combination, sizeof(combination), "+");
 
-	for (index = 0; index < count; index++)
+	for (size_t i = 0; i < strnlen(combination, sizeof(combination)); i++)
+		combination[i] = tolower(combination[i]);
+
+	const size_t count = ArrayList_Count(xfc->keyCombinations);
+
+	for (size_t index = 0; index < count; index++)
 	{
-		keyCombination = (char*)ArrayList_GetItem(xfc->keyCombinations, index);
+		const char* keyCombination = (const char*)ArrayList_GetItem(xfc->keyCombinations, index);
 
 		if (_stricmp(keyCombination, combination) == 0)
 		{
@@ -424,24 +536,10 @@ static int xf_keyboard_execute_action_script(xfContext* xfc, XF_MODIFIER_KEYS* m
 	if (!match)
 		return 1;
 
-	sprintf_s(command, sizeof(command), "%s key %s", xfc->context.settings->ActionScript,
-	          combination);
-	keyScript = popen(command, "r");
-
-	if (!keyScript)
+	(void)sprintf_s(command, sizeof(command), "key %s", combination);
+	if (!run_action_script(xfc, command, NULL, action_script_run, &status))
 		return -1;
 
-	while (fgets(buffer, sizeof(buffer), keyScript) != NULL)
-	{
-		strtok(buffer, "\n");
-
-		if (strcmp(buffer, "key-local") == 0)
-			status = 0;
-	}
-
-	if (pclose(keyScript) == -1)
-		status = -1;
-
 	return status;
 }
 
@@ -471,54 +569,68 @@ BOOL xf_keyboard_handle_special_keys(xfContext* xfc, KeySym keysym)
 	// do not return anything such that the key could be used by client if ungrab is not the goal
 	if (keysym == XK_Control_R)
 	{
-		if (mod.RightCtrl && firstPressRightCtrl)
+		if (mod.RightCtrl && !xfc->wasRightCtrlAlreadyPressed)
 		{
 			// Right Ctrl is pressed, getting ready to ungrab
-			ungrabKeyboardWithRightCtrl = TRUE;
-			firstPressRightCtrl = FALSE;
+			xfc->ungrabKeyboardWithRightCtrl = TRUE;
+			xfc->wasRightCtrlAlreadyPressed = TRUE;
 		}
 	}
 	else
 	{
 		// some other key has been pressed, abort ungrabbing
-		if (ungrabKeyboardWithRightCtrl)
-			ungrabKeyboardWithRightCtrl = FALSE;
+		if (xfc->ungrabKeyboardWithRightCtrl)
+			xfc->ungrabKeyboardWithRightCtrl = FALSE;
 	}
 
-	if (!xf_keyboard_execute_action_script(xfc, &mod, keysym))
-	{
+	const int rc = xf_keyboard_execute_action_script(xfc, &mod, keysym);
+	if (rc < 0)
+		return FALSE;
+	if (rc == 0)
 		return TRUE;
-	}
 
 	if (!xfc->remote_app && xfc->fullscreen_toggle)
 	{
-		if (keysym == XK_Return)
+		switch (keysym)
 		{
-			if (mod.Ctrl && mod.Alt)
-			{
-				/* Ctrl-Alt-Enter: toggle full screen */
-				xf_toggle_fullscreen(xfc);
-				return TRUE;
-			}
+			case XK_Return:
+				if (mod.Ctrl && mod.Alt)
+				{
+					/* Ctrl-Alt-Enter: toggle full screen */
+					xf_toggle_fullscreen(xfc);
+					return TRUE;
+				}
+				break;
+			default:
+				break;
 		}
 	}
 
-	if ((keysym == XK_c) || (keysym == XK_C))
+	if (mod.Ctrl && mod.Alt)
 	{
-		if (mod.Ctrl && mod.Alt)
+		switch (keysym)
 		{
-			/* Ctrl-Alt-C: toggle control */
-			if (xf_toggle_control(xfc))
+			case XK_m:
+			case XK_M:
+				xf_minimize(xfc);
 				return TRUE;
+			case XK_c:
+			case XK_C:
+				/* Ctrl-Alt-C: toggle control */
+				if (freerdp_client_encomsp_toggle_control(xfc->common.encomsp))
+					return TRUE;
+				break;
+			default:
+				break;
 		}
 	}
 
 #if 0 /* set to 1 to enable multi touch gesture simulation via keyboard */
 #ifdef WITH_XRENDER
 
-	if (!xfc->remote_app && xfc->settings->MultiTouchGestures)
+    if (!xfc->remote_app && freerdp_settings_get_bool(xfc->common.context.settings, FreeRDP_MultiTouchGestures))
 	{
-		rdpContext* ctx = &xfc->context;
+		rdpContext* ctx = &xfc->common.context;
 
 		if (mod.Ctrl && mod.Alt)
 		{
@@ -529,20 +641,24 @@ BOOL xf_keyboard_handle_special_keys(xfContext* xfc, KeySym keysym)
 
 			switch (keysym)
 			{
-				case XK_0:	/* Ctrl-Alt-0: Reset scaling and panning */
-					xfc->scaledWidth = xfc->sessionWidth;
-					xfc->scaledHeight = xfc->sessionHeight;
+                case XK_0:	/* Ctrl-Alt-0: Reset scaling and panning */{
+                const UINT32 sessionWidth = freerdp_settings_get_uint32(xfc->common.context.settings, FreeRDP_DesktopWidth);
+                const UINT32 sessionHeight = freerdp_settings_get_uint32(xfc->common.context.settings, FreeRDP_DesktopHeight);
+
+                    xfc->scaledWidth = sessionWidth;
+                    xfc->scaledHeight = sessionHeight;
 					xfc->offset_x = 0;
 					xfc->offset_y = 0;
 
-					if (!xfc->fullscreen && (xfc->sessionWidth != xfc->window->width ||
-					                         xfc->sessionHeight != xfc->window->height))
+                    if (!xfc->fullscreen && (sessionWidth != xfc->window->width ||
+                                             sessionHeight != xfc->window->height))
 					{
-						xf_ResizeDesktopWindow(xfc, xfc->window, xfc->sessionWidth, xfc->sessionHeight);
+                        xf_ResizeDesktopWindow(xfc, xfc->window, sessionWidth, sessionHeight);
 					}
 
-					xf_draw_screen(xfc, 0, 0, xfc->sessionWidth, xfc->sessionHeight);
+                    xf_draw_screen(xfc, 0, 0, sessionWidth, sessionHeight);
 					return TRUE;
+}
 
 				case XK_1:	/* Ctrl-Alt-1: Zoom in */
 					zdx = zdy = 10;
@@ -601,9 +717,9 @@ void xf_keyboard_handle_special_keys_release(xfContext* xfc, KeySym keysym)
 	if (keysym != XK_Control_R)
 		return;
 
-	firstPressRightCtrl = TRUE;
+	xfc->wasRightCtrlAlreadyPressed = FALSE;
 
-	if (!ungrabKeyboardWithRightCtrl)
+	if (!xfc->ungrabKeyboardWithRightCtrl)
 		return;
 
 	// all requirements for ungrab are fulfilled, ungrabbing now
@@ -614,15 +730,15 @@ void xf_keyboard_handle_special_keys_release(xfContext* xfc, KeySym keysym)
 	{
 		if (!xfc->fullscreen)
 		{
-			xf_toggle_control(xfc);
+			freerdp_client_encomsp_toggle_control(xfc->common.encomsp);
 		}
 
 		xfc->mouse_active = FALSE;
-		XUngrabKeyboard(xfc->display, CurrentTime);
+		xf_ungrab(xfc);
 	}
 
 	// ungrabbed
-	ungrabKeyboardWithRightCtrl = FALSE;
+	xfc->ungrabKeyboardWithRightCtrl = FALSE;
 }
 
 BOOL xf_keyboard_set_indicators(rdpContext* context, UINT16 led_flags)
@@ -647,3 +763,12 @@ BOOL xf_keyboard_set_ime_status(rdpContext* context, UINT16 imeId, UINT32 imeSta
 	          imeId, imeState, imeConvMode);
 	return TRUE;
 }
+
+BOOL xf_ungrab(xfContext* xfc)
+{
+	WINPR_ASSERT(xfc);
+	XUngrabKeyboard(xfc->display, CurrentTime);
+	XUngrabPointer(xfc->display, CurrentTime);
+	xfc->common.mouse_grabbed = FALSE;
+	return TRUE;
+}
