@@ -57,13 +57,13 @@
 
 static const BYTE BOM_UTF16_LE[2] = { 0xFF, 0xFE };
 
-#define INVALID_INTEGER_VALUE 0xFFFFFFFF
+// #define INVALID_INTEGER_VALUE 0xFFFFFFFF
 
 #define RDP_FILE_LINE_FLAG_FORMATTED 0x00000001
-#define RDP_FILE_LINE_FLAG_STANDARD 0x00000002
+// #define RDP_FILE_LINE_FLAG_STANDARD 0x00000002
 #define RDP_FILE_LINE_FLAG_TYPE_STRING 0x00000010
 #define RDP_FILE_LINE_FLAG_TYPE_INTEGER 0x00000020
-#define RDP_FILE_LINE_FLAG_TYPE_BINARY 0x00000040
+// #define RDP_FILE_LINE_FLAG_TYPE_BINARY 0x00000040
 
 struct rdp_file_line
 {
@@ -1021,9 +1021,9 @@ BOOL freerdp_client_parse_rdp_file_ex(rdpFile* file, const char* name, rdp_file_
 		return FALSE;
 	}
 
-	_fseeki64(fp, 0, SEEK_END);
+	(void)_fseeki64(fp, 0, SEEK_END);
 	file_size = _ftelli64(fp);
-	_fseeki64(fp, 0, SEEK_SET);
+	(void)_fseeki64(fp, 0, SEEK_SET);
 
 	if (file_size < 1)
 	{
@@ -1117,7 +1117,6 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 	UINT32 LoadBalanceInfoLength = 0;
 	const char* GatewayHostname = NULL;
 	char* redirectCameras = NULL;
-	char* redirectUsb = NULL;
 
 	if (!file || !settings)
 		return FALSE;
@@ -1307,7 +1306,8 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 		file->RedirectCameras = redirectCameras;
 	}
 #ifdef CHANNEL_URBDRC_CLIENT
-	redirectUsb = freerdp_client_channel_args_to_string(settings, URBDRC_CHANNEL_NAME, "device:");
+	char* redirectUsb =
+	    freerdp_client_channel_args_to_string(settings, URBDRC_CHANNEL_NAME, "device:");
 	if (redirectUsb)
 		file->UsbDevicesToRedirect = redirectUsb;
 
@@ -1357,19 +1357,16 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 
 BOOL freerdp_client_write_rdp_file(const rdpFile* file, const char* name, BOOL unicode)
 {
-	FILE* fp = NULL;
-	size_t size = 0;
-	char* buffer = NULL;
 	int status = 0;
 	WCHAR* unicodestr = NULL;
 
 	if (!file || !name)
 		return FALSE;
 
-	size = freerdp_client_write_rdp_file_buffer(file, NULL, 0);
+	const size_t size = freerdp_client_write_rdp_file_buffer(file, NULL, 0);
 	if (size == 0)
 		return FALSE;
-	buffer = (char*)calloc((size_t)(size + 1), sizeof(char));
+	char* buffer = calloc(size + 1ULL, sizeof(char));
 
 	if (freerdp_client_write_rdp_file_buffer(file, buffer, size + 1) != size)
 	{
@@ -1378,7 +1375,7 @@ BOOL freerdp_client_write_rdp_file(const rdpFile* file, const char* name, BOOL u
 		return FALSE;
 	}
 
-	fp = winpr_fopen(name, "w+b");
+	FILE* fp = winpr_fopen(name, "w+b");
 
 	if (fp)
 	{
@@ -1428,7 +1425,7 @@ WINPR_ATTR_FORMAT_ARG(3, 4)
 static SSIZE_T freerdp_client_write_setting_to_buffer(char** buffer, size_t* bufferSize,
                                                       WINPR_FORMAT_ARG const char* fmt, ...)
 {
-	va_list ap;
+	va_list ap = { 0 };
 	SSIZE_T len = 0;
 	char* buf = NULL;
 	size_t bufSize = 0;
@@ -2378,23 +2375,27 @@ BOOL freerdp_client_populate_settings_from_rdp_file(const rdpFile* file, rdpSett
 		if (!args)
 			return FALSE;
 
+		BOOL status = TRUE;
 		if (~file->EncodeRedirectedVideoCapture)
 		{
-			char encode[64];
-			_snprintf(encode, sizeof(encode), "encode:%" PRIu32,
-			          file->EncodeRedirectedVideoCapture);
-			freerdp_addin_argv_add_argument(args, encode);
+			char encode[64] = { 0 };
+			(void)_snprintf(encode, sizeof(encode), "encode:%" PRIu32,
+			                file->EncodeRedirectedVideoCapture);
+			if (!freerdp_addin_argv_add_argument(args, encode))
+				status = FALSE;
 		}
 		if (~file->RedirectedVideoCaptureEncodingQuality)
 		{
-			char quality[64];
-			_snprintf(quality, sizeof(quality), "quality:%" PRIu32,
-			          file->RedirectedVideoCaptureEncodingQuality);
-			freerdp_addin_argv_add_argument(args, quality);
+			char quality[64] = { 0 };
+			(void)_snprintf(quality, sizeof(quality), "quality:%" PRIu32,
+			                file->RedirectedVideoCaptureEncodingQuality);
+			if (!freerdp_addin_argv_add_argument(args, quality))
+				status = FALSE;
 		}
 
 		cnv.c = args->argv;
-		const BOOL status = freerdp_client_add_dynamic_channel(settings, args->argc, cnv.cc);
+		if (status)
+			status = freerdp_client_add_dynamic_channel(settings, args->argc, cnv.cc);
 		freerdp_addin_argv_free(args);
 		if (!status)
 			return FALSE;
@@ -2593,7 +2594,8 @@ const char* freerdp_client_rdp_file_get_string_option(const rdpFile* file, const
 	LPSTR* value = NULL;
 	rdpFileLine* line = NULL;
 
-	if (freerdp_client_rdp_file_find_string_entry((rdpFile*)file, name, &value, &line))
+	rdpFile* wfile = WINPR_CAST_CONST_PTR_AWAY(file, rdpFile*);
+	if (freerdp_client_rdp_file_find_string_entry(wfile, name, &value, &line))
 	{
 		if (value && ~(size_t)(*value))
 			return *value;
@@ -2614,7 +2616,8 @@ int freerdp_client_rdp_file_get_integer_option(const rdpFile* file, const char*
 	DWORD* value = NULL;
 	rdpFileLine* line = NULL;
 
-	if (freerdp_client_rdp_file_find_integer_entry((rdpFile*)file, name, &value, &line))
+	rdpFile* wfile = WINPR_CAST_CONST_PTR_AWAY(file, rdpFile*);
+	if (freerdp_client_rdp_file_find_integer_entry(wfile, name, &value, &line))
 	{
 		if (value && ~(*value))
 			return *value;
