@@ -286,12 +286,12 @@ static BYTE* test_progressive_load_file(const char* path, const char* file, size
 	if (!fp)
 		return NULL;
 
-	_fseeki64(fp, 0, SEEK_END);
+	(void)_fseeki64(fp, 0, SEEK_END);
 	const INT64 pos = _ftelli64(fp);
 	WINPR_ASSERT(pos >= 0);
 	WINPR_ASSERT(pos <= SIZE_MAX);
 	*size = (size_t)pos;
-	_fseeki64(fp, 0, SEEK_SET);
+	(void)_fseeki64(fp, 0, SEEK_SET);
 	BYTE* buffer = (BYTE*)malloc(*size);
 
 	if (!buffer)
@@ -1065,6 +1065,8 @@ static BOOL test_encode_decode(const char* path)
 	rc = progressive_compress(progressiveEnc, image->data, image->scanline * image->height,
 	                          ColorFormat, image->width, image->height, image->scanline, NULL,
 	                          &dstData, &dstSize);
+	if (rc < 0)
+		goto fail;
 
 	// Progressive decode
 	rc = progressive_create_surface_context(progressiveDec, 0, image->width, image->height);
@@ -1120,6 +1122,7 @@ static BOOL read_cmd(FILE* fp, RDPGFX_SURFACE_COMMAND* cmd, UINT32* frameId)
 	WINPR_ASSERT(cmd);
 	WINPR_ASSERT(frameId);
 
+	// NOLINTBEGIN(cert-err34-c)
 	if (1 != fscanf(fp, "frameid: %" PRIu32 "\n", frameId))
 		return FALSE;
 	if (1 != fscanf(fp, "surfaceId: %" PRIu32 "\n", &cmd->surfaceId))
@@ -1144,6 +1147,7 @@ static BOOL read_cmd(FILE* fp, RDPGFX_SURFACE_COMMAND* cmd, UINT32* frameId)
 		return FALSE;
 	if (1 != fscanf(fp, "length: %" PRIu32 "\n", &cmd->length))
 		return FALSE;
+	// NOLINTEND(cert-err34-c)
 
 	char* data = NULL;
 
