@@ -78,7 +78,7 @@ struct rdp_assistance_file
 static const char* strrstr(const char* haystack, size_t len, const char* needle)
 {
 	if (*needle == '\0')
-		return (const char*)haystack;
+		return haystack;
 
 	char* result = NULL;
 	for (;;)
@@ -214,13 +214,16 @@ static BOOL update_connectionstring2_wchar(rdpAssistanceFile* file, const WCHAR*
  * Use the first n bytes of the result of step 5 as the derived key.
  */
 
-static BOOL freerdp_assistance_crypt_derive_key_sha1(BYTE* hash, size_t hashLength, BYTE* key,
+static BOOL freerdp_assistance_crypt_derive_key_sha1(const BYTE* hash, size_t hashLength, BYTE* key,
                                                      size_t keyLength)
 {
 	BOOL rc = FALSE;
 	BYTE pad1[64] = { 0 };
 	BYTE pad2[64] = { 0 };
 
+	if (hashLength == 0)
+		return FALSE;
+
 	memset(pad1, 0x36, sizeof(pad1));
 	memset(pad2, 0x5C, sizeof(pad2));
 
@@ -297,7 +300,8 @@ static BOOL freerdp_assistance_parse_address_list(rdpAssistanceFile* file, char*
 	char* s = ";";
 
 	// get the first token
-	char* token = strtok(strp, s);
+	char* saveptr = NULL;
+	char* token = strtok_s(strp, s, &saveptr);
 
 	// walk through other tokens
 	while (token != NULL)
@@ -311,7 +315,7 @@ static BOOL freerdp_assistance_parse_address_list(rdpAssistanceFile* file, char*
 		if (!append_address(file, token, port))
 			goto out;
 
-		token = strtok(NULL, s);
+		token = strtok_s(NULL, s, &saveptr);
 	}
 	rc = TRUE;
 out:
@@ -423,7 +427,9 @@ static BOOL freerdp_assistance_parse_attr(const char** opt, size_t* plength, con
 	char bkey[128] = { 0 };
 	const int rc = _snprintf(bkey, sizeof(bkey), "%s=\"", key);
 	WINPR_ASSERT(rc > 0);
-	WINPR_ASSERT(rc < sizeof(bkey));
+	WINPR_ASSERT((size_t)rc < sizeof(bkey));
+	if ((rc <= 0) || ((size_t)rc >= sizeof(bkey)))
+		return FALSE;
 
 	char* p = strstr(tag, bkey);
 	if (!p)
@@ -523,7 +529,9 @@ static char* freerdp_assistance_contains_element(char* input, size_t ilen, const
 	char bkey[128] = { 0 };
 	const int rc = _snprintf(bkey, sizeof(bkey), "<%s", key);
 	WINPR_ASSERT(rc > 0);
-	WINPR_ASSERT(rc < sizeof(bkey));
+	WINPR_ASSERT((size_t)rc < sizeof(bkey));
+	if ((rc < 0) || ((size_t)rc >= sizeof(bkey)))
+		return NULL;
 
 	char* tag = strstr(input, bkey);
 	if (!tag || (tag > input + ilen))
@@ -562,7 +570,9 @@ static char* freerdp_assistance_contains_element(char* input, size_t ilen, const
 		char ekey[128] = { 0 };
 		const int erc = _snprintf(ekey, sizeof(ekey), "</%s>", key);
 		WINPR_ASSERT(erc > 0);
-		WINPR_ASSERT(erc < sizeof(ekey));
+		WINPR_ASSERT((size_t)erc < sizeof(ekey));
+		if ((erc <= 0) || ((size_t)erc >= sizeof(ekey)))
+			return NULL;
 		const size_t offset = start - tag;
 		dend = end = strrstr(start, ilen - offset, ekey);
 		if (end)
@@ -775,24 +785,19 @@ out_fail:
 
 char* freerdp_assistance_construct_expert_blob(const char* name, const char* pass)
 {
-	size_t size = 0;
-	size_t nameLength = 0;
-	size_t passLength = 0;
-	char* ExpertBlob = NULL;
-
 	if (!name || !pass)
 		return NULL;
 
-	nameLength = strlen(name) + strlen("NAME=");
-	passLength = strlen(pass) + strlen("PASS=");
-	size = nameLength + passLength + 64;
-	ExpertBlob = (char*)calloc(1, size);
+	const size_t nameLength = strlen(name) + strlen("NAME=");
+	const size_t passLength = strlen(pass) + strlen("PASS=");
+	const size_t size = nameLength + passLength + 64;
+	char* ExpertBlob = (char*)calloc(1, size);
 
 	if (!ExpertBlob)
 		return NULL;
 
-	sprintf_s(ExpertBlob, size, "%" PRIdz ";NAME=%s%" PRIdz ";PASS=%s", nameLength, name,
-	          passLength, pass);
+	(void)sprintf_s(ExpertBlob, size, "%" PRIuz ";NAME=%s%" PRIuz ";PASS=%s", nameLength, name,
+	                passLength, pass);
 	return ExpertBlob;
 }
 
@@ -960,8 +965,8 @@ static BOOL freerdp_assistance_decrypt2(rdpAssistanceFile* file)
 		goto fail;
 
 	cbOut = cbFinal = 0;
-	cbIn = (size_t)file->EncryptedLHTicketLength;
-	pbIn = (BYTE*)file->EncryptedLHTicket;
+	cbIn = file->EncryptedLHTicketLength;
+	pbIn = file->EncryptedLHTicket;
 	pbOut = (BYTE*)calloc(1, cbIn + WINPR_AES_BLOCK_SIZE + 2);
 
 	if (!pbOut)
@@ -1236,14 +1241,14 @@ int freerdp_assistance_parse_file(rdpAssistanceFile* file, const char* name, con
 		return -1;
 	}
 
-	_fseeki64(fp, 0, SEEK_END);
+	(void)_fseeki64(fp, 0, SEEK_END);
 	fileSize.i64 = _ftelli64(fp);
-	_fseeki64(fp, 0, SEEK_SET);
+	(void)_fseeki64(fp, 0, SEEK_SET);
 
 	if (fileSize.i64 < 1)
 	{
 		WLog_ERR(TAG, "Failed to read ASSISTANCE file %s ", name);
-		fclose(fp);
+		(void)fclose(fp);
 		return -1;
 	}
 
@@ -1251,7 +1256,7 @@ int freerdp_assistance_parse_file(rdpAssistanceFile* file, const char* name, con
 
 	if (!buffer)
 	{
-		fclose(fp);
+		(void)fclose(fp);
 		return -1;
 	}
 
@@ -1263,7 +1268,7 @@ int freerdp_assistance_parse_file(rdpAssistanceFile* file, const char* name, con
 			readSize = fileSize.s;
 	}
 
-	fclose(fp);
+	(void)fclose(fp);
 
 	if (readSize < 1)
 	{
@@ -1342,8 +1347,13 @@ BOOL freerdp_assistance_populate_settings_from_assistance_file(rdpAssistanceFile
 	if (ports != addresses)
 		return FALSE;
 
-	const UINT32 port = (UINT32)ArrayList_GetItem(file->MachinePorts, 0);
-	if (!freerdp_settings_set_uint32(settings, FreeRDP_ServerPort, port))
+	union
+	{
+		UINT32 port;
+		void* data;
+	} cnv;
+	cnv.data = ArrayList_GetItem(file->MachinePorts, 0);
+	if (!freerdp_settings_set_uint32(settings, FreeRDP_ServerPort, cnv.port))
 		return FALSE;
 
 	if (!freerdp_target_net_adresses_reset(settings, ports))
@@ -1351,8 +1361,8 @@ BOOL freerdp_assistance_populate_settings_from_assistance_file(rdpAssistanceFile
 
 	for (size_t x = 0; x < ports; x++)
 	{
-		const UINT32 mport = (UINT32)ArrayList_GetItem(file->MachinePorts, x);
-		if (!freerdp_settings_set_pointer_array(settings, FreeRDP_TargetNetPorts, x, &mport))
+		cnv.data = ArrayList_GetItem(file->MachinePorts, x);
+		if (!freerdp_settings_set_pointer_array(settings, FreeRDP_TargetNetPorts, x, &cnv.port))
 			return FALSE;
 	}
 	for (size_t i = 0; i < addresses; i++)
@@ -1451,7 +1461,15 @@ void freerdp_assistance_print_file(rdpAssistanceFile* file, wLog* log, DWORD lev
 		const char* uri = NULL;
 		const char* addr = ArrayList_GetItem(file->MachineAddresses, x);
 		if (x < ArrayList_Count(file->MachinePorts))
-			port = (UINT32)ArrayList_GetItem(file->MachinePorts, x);
+		{
+			union
+			{
+				UINT32 port;
+				void* data;
+			} cnv;
+			cnv.data = ArrayList_GetItem(file->MachinePorts, x);
+			port = cnv.port;
+		}
 		if (x < ArrayList_Count(file->MachineUris))
 			uri = ArrayList_GetItem(file->MachineUris, x);
 
