@@ -17,13 +17,13 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <freerdp/config.h>
 
 #include <freerdp/log.h>
 
 #include <winpr/crt.h>
+#include <winpr/wtypes.h>
+#include <winpr/assert.h>
 #include <winpr/print.h>
 #include <winpr/synch.h>
 #include <winpr/thread.h>
@@ -35,22 +35,70 @@
 #include "rpc_bind.h"
 #include "rpc_fault.h"
 #include "rpc_client.h"
+#include "rts_signature.h"
+
+#include "../utils.h"
 #include "../rdp.h"
 #include "../proxy.h"
 
 #define TAG FREERDP_TAG("core.gateway.rpc")
 
+static const char* rpc_client_state_str(RPC_CLIENT_STATE state)
+{
+	// NOLINTNEXTLINE(clang-analyzer-deadcode.DeadStores)
+	const char* str = "RPC_CLIENT_STATE_UNKNOWN";
+
+	switch (state)
+	{
+		case RPC_CLIENT_STATE_INITIAL:
+			str = "RPC_CLIENT_STATE_INITIAL";
+			break;
+
+		case RPC_CLIENT_STATE_ESTABLISHED:
+			str = "RPC_CLIENT_STATE_ESTABLISHED";
+			break;
+
+		case RPC_CLIENT_STATE_WAIT_SECURE_BIND_ACK:
+			str = "RPC_CLIENT_STATE_WAIT_SECURE_BIND_ACK";
+			break;
+
+		case RPC_CLIENT_STATE_WAIT_UNSECURE_BIND_ACK:
+			str = "RPC_CLIENT_STATE_WAIT_UNSECURE_BIND_ACK";
+			break;
+
+		case RPC_CLIENT_STATE_WAIT_SECURE_ALTER_CONTEXT_RESPONSE:
+			str = "RPC_CLIENT_STATE_WAIT_SECURE_ALTER_CONTEXT_RESPONSE";
+			break;
+
+		case RPC_CLIENT_STATE_CONTEXT_NEGOTIATED:
+			str = "RPC_CLIENT_STATE_CONTEXT_NEGOTIATED";
+			break;
+
+		case RPC_CLIENT_STATE_WAIT_RESPONSE:
+			str = "RPC_CLIENT_STATE_WAIT_RESPONSE";
+			break;
+
+		case RPC_CLIENT_STATE_FINAL:
+			str = "RPC_CLIENT_STATE_FINAL";
+			break;
+		default:
+			break;
+	}
+	return str;
+}
+
 static void rpc_pdu_reset(RPC_PDU* pdu)
 {
 	pdu->Type = 0;
 	pdu->Flags = 0;
 	pdu->CallId = 0;
 	Stream_SetPosition(pdu->s, 0);
+	Stream_SetLength(pdu->s, 0);
 }
 
 static RPC_PDU* rpc_pdu_new(void)
 {
-	RPC_PDU* pdu;
+	RPC_PDU* pdu = NULL;
 	pdu = (RPC_PDU*)malloc(sizeof(RPC_PDU));
 
 	if (!pdu)
@@ -90,7 +138,7 @@ static int rpc_client_receive_pipe_write(RpcClient* client, const BYTE* buffer,
 		status += (int)length;
 
 	if (ringbuffer_used(&(client->ReceivePipe)) > 0)
-		SetEvent(client->PipeEvent);
+		(void)SetEvent(client->PipeEvent);
 
 	LeaveCriticalSection(&(client->PipeLock));
 	return status;
@@ -98,8 +146,7 @@ static int rpc_client_receive_pipe_write(RpcClient* client, const BYTE* buffer,
 
 int rpc_client_receive_pipe_read(RpcClient* client, BYTE* buffer, size_t length)
 {
-	int index = 0;
-	int status = 0;
+	size_t status = 0;
 	int nchunks = 0;
 	DataChunk chunks[2];
 
@@ -109,7 +156,7 @@ int rpc_client_receive_pipe_read(RpcClient* client, BYTE* buffer, size_t length)
 	EnterCriticalSection(&(client->PipeLock));
 	nchunks = ringbuffer_peek(&(client->ReceivePipe), chunks, length);
 
-	for (index = 0; index < nchunks; index++)
+	for (int index = 0; index < nchunks; index++)
 	{
 		CopyMemory(&buffer[status], chunks[index].data, chunks[index].size);
 		status += chunks[index].size;
@@ -119,62 +166,41 @@ int rpc_client_receive_pipe_read(RpcClient* client, BYTE* buffer, size_t length)
 		ringbuffer_commit_read_bytes(&(client->ReceivePipe), status);
 
 	if (ringbuffer_used(&(client->ReceivePipe)) < 1)
-		ResetEvent(client->PipeEvent);
+		(void)ResetEvent(client->PipeEvent);
 
 	LeaveCriticalSection(&(client->PipeLock));
-	return status;
+
+	if (status > INT_MAX)
+		return -1;
+	return (int)status;
 }
 
 static int rpc_client_transition_to_state(rdpRpc* rpc, RPC_CLIENT_STATE state)
 {
 	int status = 1;
-	const char* str = "RPC_CLIENT_STATE_UNKNOWN";
-
-	switch (state)
-	{
-		case RPC_CLIENT_STATE_INITIAL:
-			str = "RPC_CLIENT_STATE_INITIAL";
-			break;
-
-		case RPC_CLIENT_STATE_ESTABLISHED:
-			str = "RPC_CLIENT_STATE_ESTABLISHED";
-			break;
-
-		case RPC_CLIENT_STATE_WAIT_SECURE_BIND_ACK:
-			str = "RPC_CLIENT_STATE_WAIT_SECURE_BIND_ACK";
-			break;
-
-		case RPC_CLIENT_STATE_WAIT_UNSECURE_BIND_ACK:
-			str = "RPC_CLIENT_STATE_WAIT_UNSECURE_BIND_ACK";
-			break;
-
-		case RPC_CLIENT_STATE_WAIT_SECURE_ALTER_CONTEXT_RESPONSE:
-			str = "RPC_CLIENT_STATE_WAIT_SECURE_ALTER_CONTEXT_RESPONSE";
-			break;
-
-		case RPC_CLIENT_STATE_CONTEXT_NEGOTIATED:
-			str = "RPC_CLIENT_STATE_CONTEXT_NEGOTIATED";
-			break;
-
-		case RPC_CLIENT_STATE_WAIT_RESPONSE:
-			str = "RPC_CLIENT_STATE_WAIT_RESPONSE";
-			break;
-
-		case RPC_CLIENT_STATE_FINAL:
-			str = "RPC_CLIENT_STATE_FINAL";
-			break;
-	}
 
 	rpc->State = state;
-	WLog_DBG(TAG, "%s", str);
+	WLog_DBG(TAG, "%s", rpc_client_state_str(state));
 	return status;
 }
 
-static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
+static int rpc_client_recv_pdu_int(rdpRpc* rpc, RPC_PDU* pdu)
 {
 	int status = -1;
-	rpcconn_rts_hdr_t* rts;
-	rdpTsg* tsg = rpc->transport->tsg;
+	RtsPduSignature found = { 0 };
+
+	WINPR_ASSERT(rpc);
+	WINPR_ASSERT(pdu);
+
+	rdpTsg* tsg = transport_get_tsg(rpc->transport);
+
+	WLog_VRB(TAG, "client state %s, vc state %s", rpc_client_state_str(rpc->State),
+	         rpc_vc_state_str(rpc->VirtualConnection->State));
+
+	const BOOL rc = rts_match_pdu_signature_ex(&RTS_PDU_PING_SIGNATURE, pdu->s, NULL, &found, TRUE);
+	rts_print_pdu_signature(rpc->log, WLOG_TRACE, &found);
+	if (rc)
+		return rts_recv_ping_pdu(rpc, pdu->s);
 
 	if (rpc->VirtualConnection->State < VIRTUAL_CONNECTION_STATE_OPENED)
 	{
@@ -187,17 +213,15 @@ static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
 				break;
 
 			case VIRTUAL_CONNECTION_STATE_WAIT_A3W:
-				rts = (rpcconn_rts_hdr_t*)Stream_Buffer(pdu->s);
-
-				if (!rts_match_pdu_signature(&RTS_PDU_CONN_A3_SIGNATURE, rts))
+				if (memcmp(&found, &RTS_PDU_CONN_A3_SIGNATURE, sizeof(found)) != 0)
 				{
-					WLog_ERR(TAG, "unexpected RTS PDU: Expected CONN/A3");
+					wLog* log = WLog_Get(TAG);
+					WLog_Print(log, WLOG_ERROR, "unexpected RTS PDU: Expected CONN/A3");
+					rts_print_pdu_signature(log, WLOG_ERROR, &found);
 					return -1;
 				}
 
-				status = rts_recv_CONN_A3_pdu(rpc, Stream_Buffer(pdu->s), Stream_Length(pdu->s));
-
-				if (status < 0)
+				if (!rts_recv_CONN_A3_pdu(rpc, pdu->s))
 				{
 					WLog_ERR(TAG, "rts_recv_CONN_A3_pdu failure");
 					return -1;
@@ -209,17 +233,15 @@ static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
 				break;
 
 			case VIRTUAL_CONNECTION_STATE_WAIT_C2:
-				rts = (rpcconn_rts_hdr_t*)Stream_Buffer(pdu->s);
-
-				if (!rts_match_pdu_signature(&RTS_PDU_CONN_C2_SIGNATURE, rts))
+				if (memcmp(&found, &RTS_PDU_CONN_C2_SIGNATURE, sizeof(found)) != 0)
 				{
-					WLog_ERR(TAG, "unexpected RTS PDU: Expected CONN/C2");
+					wLog* log = WLog_Get(TAG);
+					WLog_Print(log, WLOG_ERROR, "unexpected RTS PDU: Expected CONN/C2");
+					rts_print_pdu_signature(log, WLOG_ERROR, &found);
 					return -1;
 				}
 
-				status = rts_recv_CONN_C2_pdu(rpc, Stream_Buffer(pdu->s), Stream_Length(pdu->s));
-
-				if (status < 0)
+				if (!rts_recv_CONN_C2_pdu(rpc, pdu->s))
 				{
 					WLog_ERR(TAG, "rts_recv_CONN_C2_pdu failure");
 					return -1;
@@ -229,7 +251,7 @@ static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
 				                                           VIRTUAL_CONNECTION_STATE_OPENED);
 				rpc_client_transition_to_state(rpc, RPC_CLIENT_STATE_ESTABLISHED);
 
-				if (rpc_send_bind_pdu(rpc) < 0)
+				if (rpc_send_bind_pdu(rpc, TRUE) < 0)
 				{
 					WLog_ERR(TAG, "rpc_send_bind_pdu failure");
 					return -1;
@@ -250,9 +272,9 @@ static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
 	{
 		if (rpc->State == RPC_CLIENT_STATE_WAIT_SECURE_BIND_ACK)
 		{
-			if (pdu->Type == PTYPE_BIND_ACK)
+			if (pdu->Type == PTYPE_BIND_ACK || pdu->Type == PTYPE_ALTER_CONTEXT_RESP)
 			{
-				if (rpc_recv_bind_ack_pdu(rpc, Stream_Buffer(pdu->s), Stream_Length(pdu->s)) <= 0)
+				if (!rpc_recv_bind_ack_pdu(rpc, pdu->s))
 				{
 					WLog_ERR(TAG, "rpc_recv_bind_ack_pdu failure");
 					return -1;
@@ -267,25 +289,38 @@ static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
 				return -1;
 			}
 
-			if (rpc_send_rpc_auth_3_pdu(rpc) < 0)
+			switch (rpc_bind_state(rpc))
 			{
-				WLog_ERR(TAG, "rpc_secure_bind: error sending rpc_auth_3 pdu!");
-				return -1;
-			}
+				case RPC_BIND_STATE_INCOMPLETE:
+					if (rpc_send_bind_pdu(rpc, FALSE) < 0)
+					{
+						WLog_ERR(TAG, "rpc_send_bind_pdu failure");
+						return -1;
+					}
+					break;
+				case RPC_BIND_STATE_LAST_LEG:
+					if (rpc_send_rpc_auth_3_pdu(rpc) < 0)
+					{
+						WLog_ERR(TAG, "rpc_secure_bind: error sending rpc_auth_3 pdu!");
+						return -1;
+					}
+					/* fallthrough */
+					WINPR_FALLTHROUGH
+				case RPC_BIND_STATE_COMPLETE:
+					rpc_client_transition_to_state(rpc, RPC_CLIENT_STATE_CONTEXT_NEGOTIATED);
 
-			rpc_client_transition_to_state(rpc, RPC_CLIENT_STATE_CONTEXT_NEGOTIATED);
-
-			if (!tsg_proxy_begin(tsg))
-			{
-				WLog_ERR(TAG, "tsg_proxy_begin failure");
-				return -1;
+					if (!tsg_proxy_begin(tsg))
+					{
+						WLog_ERR(TAG, "tsg_proxy_begin failure");
+						return -1;
+					}
 			}
 
 			status = 1;
 		}
 		else
 		{
-			WLog_ERR(TAG, "rpc_client_recv_pdu: invalid rpc->State: %d", rpc->State);
+			WLog_ERR(TAG, "invalid rpc->State: %d", rpc->State);
 		}
 	}
 	else if (rpc->State >= RPC_CLIENT_STATE_CONTEXT_NEGOTIATED)
@@ -299,91 +334,142 @@ static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
 	return status;
 }
 
+static int rpc_client_recv_pdu(rdpRpc* rpc, RPC_PDU* pdu)
+{
+	WINPR_ASSERT(rpc);
+	WINPR_ASSERT(pdu);
+
+	Stream_SealLength(pdu->s);
+	Stream_SetPosition(pdu->s, 0);
+
+	const size_t before = Stream_GetRemainingLength(pdu->s);
+	WLog_VRB(TAG, "RPC PDU parsing %" PRIuz " bytes", before);
+	const int rc = rpc_client_recv_pdu_int(rpc, pdu);
+	if (rc < 0)
+		return rc;
+	const size_t after = Stream_GetRemainingLength(pdu->s);
+	if (after > 0)
+	{
+		/* Just log so we do not fail if we have some unprocessed padding bytes */
+		WLog_WARN(TAG, "Incompletely parsed RPC PDU (%" PRIuz " bytes remain)", after);
+	}
+
+	return rc;
+}
+
 static int rpc_client_recv_fragment(rdpRpc* rpc, wStream* fragment)
 {
-	BYTE* buffer;
-	RPC_PDU* pdu;
-	UINT32 StubOffset;
-	UINT32 StubLength;
-	RpcClientCall* call;
-	rpcconn_hdr_t* header;
-	pdu = rpc->client->pdu;
-	buffer = (BYTE*)Stream_Buffer(fragment);
-	header = (rpcconn_hdr_t*)Stream_Buffer(fragment);
+	int rc = -1;
+	RPC_PDU* pdu = NULL;
+	size_t StubOffset = 0;
+	size_t StubLength = 0;
+	RpcClientCall* call = NULL;
+	rpcconn_hdr_t header = { 0 };
 
-	if (header->common.ptype == PTYPE_RESPONSE)
+	WINPR_ASSERT(rpc);
+	WINPR_ASSERT(rpc->client);
+	WINPR_ASSERT(fragment);
+
+	pdu = rpc->client->pdu;
+	WINPR_ASSERT(pdu);
+
+	Stream_SealLength(fragment);
+	Stream_SetPosition(fragment, 0);
+
+	if (!rts_read_pdu_header(fragment, &header))
+		goto fail;
+
+	if (header.common.ptype == PTYPE_RESPONSE)
 	{
-		rpc->VirtualConnection->DefaultOutChannel->BytesReceived += header->common.frag_length;
+		rpc->VirtualConnection->DefaultOutChannel->BytesReceived += header.common.frag_length;
 		rpc->VirtualConnection->DefaultOutChannel->ReceiverAvailableWindow -=
-		    header->common.frag_length;
+		    header.common.frag_length;
 
 		if (rpc->VirtualConnection->DefaultOutChannel->ReceiverAvailableWindow <
 		    (rpc->ReceiveWindow / 2))
 		{
-			if (rts_send_flow_control_ack_pdu(rpc) < 0)
-				return -1;
+			if (!rts_send_flow_control_ack_pdu(rpc))
+				goto fail;
 		}
 
-		if (!rpc_get_stub_data_info(rpc, buffer, &StubOffset, &StubLength))
+		if (!rpc_get_stub_data_info(rpc, &header, &StubOffset, &StubLength))
 		{
 			WLog_ERR(TAG, "expected stub");
-			return -1;
+			goto fail;
 		}
 
 		if (StubLength == 4)
 		{
-			if ((header->common.call_id == rpc->PipeCallId) &&
-			    (header->common.pfc_flags & PFC_LAST_FRAG))
+			if ((header.common.call_id == rpc->PipeCallId) &&
+			    (header.common.pfc_flags & PFC_LAST_FRAG))
 			{
 				/* End of TsProxySetupReceivePipe */
 				TerminateEventArgs e;
-				rpc->result = *((UINT32*)&buffer[StubOffset]);
-				freerdp_abort_connect(rpc->context->instance);
-				tsg_set_state(rpc->transport->tsg, TSG_STATE_TUNNEL_CLOSE_PENDING);
+				rdpContext* context = transport_get_context(rpc->transport);
+				rdpTsg* tsg = transport_get_tsg(rpc->transport);
+
+				WINPR_ASSERT(context);
+
+				if (Stream_Length(fragment) < StubOffset + 4)
+					goto fail;
+				Stream_SetPosition(fragment, StubOffset);
+				Stream_Read_UINT32(fragment, rpc->result);
+
+				utils_abort_connect(context->rdp);
+				tsg_set_state(tsg, TSG_STATE_TUNNEL_CLOSE_PENDING);
 				EventArgsInit(&e, "freerdp");
 				e.code = 0;
-				PubSub_OnTerminate(rpc->context->pubSub, rpc->context, &e);
-				return 0;
+				PubSub_OnTerminate(context->rdp->pubSub, context, &e);
+				rc = 0;
+				goto success;
 			}
 
-			if (header->common.call_id != rpc->PipeCallId)
+			if (header.common.call_id != rpc->PipeCallId)
 			{
 				/* Ignoring non-TsProxySetupReceivePipe Response */
-				return 0;
+				rc = 0;
+				goto success;
 			}
 		}
 
 		if (rpc->StubFragCount == 0)
-			rpc->StubCallId = header->common.call_id;
+			rpc->StubCallId = header.common.call_id;
 
-		if (rpc->StubCallId != header->common.call_id)
+		if (rpc->StubCallId != header.common.call_id)
 		{
 			WLog_ERR(TAG,
 			         "invalid call_id: actual: %" PRIu32 ", expected: %" PRIu32
 			         ", frag_count: %" PRIu32 "",
-			         rpc->StubCallId, header->common.call_id, rpc->StubFragCount);
+			         rpc->StubCallId, header.common.call_id, rpc->StubFragCount);
 		}
 
 		call = rpc_client_call_find_by_id(rpc->client, rpc->StubCallId);
 
 		if (!call)
-			return -1;
+			goto fail;
 
 		if (call->OpNum != TsProxySetupReceivePipeOpnum)
 		{
-			if (!Stream_EnsureCapacity(pdu->s, header->response.alloc_hint))
-				return -1;
+			const rpcconn_response_hdr_t* response =
+			    (const rpcconn_response_hdr_t*)&header.response;
+			if (!Stream_EnsureCapacity(pdu->s, response->alloc_hint))
+				goto fail;
 
-			Stream_Write(pdu->s, &buffer[StubOffset], StubLength);
+			if (Stream_Length(fragment) < StubOffset + StubLength)
+				goto fail;
+
+			Stream_SetPosition(fragment, StubOffset);
+			Stream_Write(pdu->s, Stream_ConstPointer(fragment), StubLength);
 			rpc->StubFragCount++;
 
-			if (header->response.alloc_hint == StubLength)
+			if (response->alloc_hint == StubLength)
 			{
 				pdu->Flags = RPC_PDU_FLAG_STUB;
 				pdu->Type = PTYPE_RESPONSE;
 				pdu->CallId = rpc->StubCallId;
-				Stream_SealLength(pdu->s);
-				rpc_client_recv_pdu(rpc, pdu);
+
+				if (rpc_client_recv_pdu(rpc, pdu) < 0)
+					goto fail;
 				rpc_pdu_reset(pdu);
 				rpc->StubFragCount = 0;
 				rpc->StubCallId = 0;
@@ -391,84 +477,95 @@ static int rpc_client_recv_fragment(rdpRpc* rpc, wStream* fragment)
 		}
 		else
 		{
-			rpc_client_receive_pipe_write(rpc->client, &buffer[StubOffset], (size_t)StubLength);
+			const rpcconn_response_hdr_t* response = &header.response;
+			if (Stream_Length(fragment) < StubOffset + StubLength)
+				goto fail;
+			Stream_SetPosition(fragment, StubOffset);
+			rpc_client_receive_pipe_write(rpc->client, Stream_ConstPointer(fragment), StubLength);
 			rpc->StubFragCount++;
 
-			if (header->response.alloc_hint == StubLength)
+			if (response->alloc_hint == StubLength)
 			{
 				rpc->StubFragCount = 0;
 				rpc->StubCallId = 0;
 			}
 		}
 
-		return 1;
+		goto success;
 	}
-	else if (header->common.ptype == PTYPE_RTS)
+	else if (header.common.ptype == PTYPE_RTS)
 	{
 		if (rpc->State < RPC_CLIENT_STATE_CONTEXT_NEGOTIATED)
 		{
 			pdu->Flags = 0;
-			pdu->Type = header->common.ptype;
-			pdu->CallId = header->common.call_id;
+			pdu->Type = header.common.ptype;
+			pdu->CallId = header.common.call_id;
 
-			if (!Stream_EnsureCapacity(pdu->s, Stream_Length(fragment)))
-				return -1;
+			const size_t len = Stream_Length(fragment);
+			if (!Stream_EnsureCapacity(pdu->s, len))
+				goto fail;
 
-			Stream_Write(pdu->s, buffer, Stream_Length(fragment));
-			Stream_SealLength(pdu->s);
+			Stream_Write(pdu->s, Stream_Buffer(fragment), len);
 
 			if (rpc_client_recv_pdu(rpc, pdu) < 0)
-				return -1;
+				goto fail;
 
 			rpc_pdu_reset(pdu);
 		}
 		else
 		{
-			if (rts_recv_out_of_sequence_pdu(rpc, buffer, header->common.frag_length) < 0)
-				return -1;
+			if (!rts_recv_out_of_sequence_pdu(rpc, fragment, &header))
+				goto fail;
 		}
 
-		return 1;
+		goto success;
 	}
-	else if (header->common.ptype == PTYPE_BIND_ACK)
+	else if (header.common.ptype == PTYPE_BIND_ACK ||
+	         header.common.ptype == PTYPE_ALTER_CONTEXT_RESP)
 	{
 		pdu->Flags = 0;
-		pdu->Type = header->common.ptype;
-		pdu->CallId = header->common.call_id;
+		pdu->Type = header.common.ptype;
+		pdu->CallId = header.common.call_id;
 
-		if (!Stream_EnsureCapacity(pdu->s, Stream_Length(fragment)))
-			return -1;
+		const size_t len = Stream_Length(fragment);
+		if (!Stream_EnsureCapacity(pdu->s, len))
+			goto fail;
 
-		Stream_Write(pdu->s, buffer, Stream_Length(fragment));
-		Stream_SealLength(pdu->s);
+		Stream_Write(pdu->s, Stream_Buffer(fragment), len);
 
 		if (rpc_client_recv_pdu(rpc, pdu) < 0)
-			return -1;
+			goto fail;
 
 		rpc_pdu_reset(pdu);
-		return 1;
+		goto success;
 	}
-	else if (header->common.ptype == PTYPE_FAULT)
+	else if (header.common.ptype == PTYPE_FAULT)
 	{
-		rpc_recv_fault_pdu(header->fault.status);
-		return -1;
+		const rpcconn_fault_hdr_t* fault = (const rpcconn_fault_hdr_t*)&header.fault;
+		rpc_recv_fault_pdu(fault->status);
+		goto fail;
 	}
 	else
 	{
-		WLog_ERR(TAG, "unexpected RPC PDU type 0x%02" PRIX8 "", header->common.ptype);
-		return -1;
+		WLog_ERR(TAG, "unexpected RPC PDU type 0x%02" PRIX8 "", header.common.ptype);
+		goto fail;
 	}
 
-	return 1;
+success:
+	rc = (rc < 0) ? 1 : 0; /* In case of default error return change to 1, otherwise we already set
+	                          the return code */
+fail:
+	rts_free_pdu_header(&header, FALSE);
+	return rc;
 }
 
-static int rpc_client_default_out_channel_recv(rdpRpc* rpc)
+static SSIZE_T rpc_client_default_out_channel_recv(rdpRpc* rpc)
 {
-	int status = -1;
-	UINT32 statusCode;
-	HttpResponse* response;
-	RpcInChannel* inChannel;
-	RpcOutChannel* outChannel;
+	SSIZE_T status = -1;
+	UINT32 statusCode = 0;
+	HttpResponse* response = NULL;
+	RpcInChannel* inChannel = NULL;
+	RpcOutChannel* outChannel = NULL;
 	HANDLE outChannelEvent = NULL;
 	RpcVirtualConnection* connection = rpc->VirtualConnection;
 	inChannel = connection->DefaultInChannel;
@@ -504,24 +601,28 @@ static int rpc_client_default_out_channel_recv(rdpRpc* rpc)
 				return -1;
 			}
 
-			rpc_ncacn_http_ntlm_uninit(&outChannel->common);
-			rpc_out_channel_transition_to_state(outChannel, CLIENT_OUT_CHANNEL_STATE_NEGOTIATED);
-
-			/* Send CONN/A1 PDU over OUT channel */
-
-			if (rts_send_CONN_A1_pdu(rpc) < 0)
+			if (rpc_ncacn_http_is_final_request(&outChannel->common))
 			{
-				http_response_free(response);
-				WLog_ERR(TAG, "rpc_send_CONN_A1_pdu error!");
-				return -1;
-			}
+				rpc_ncacn_http_auth_uninit(&outChannel->common);
+				rpc_out_channel_transition_to_state(outChannel,
+				                                    CLIENT_OUT_CHANNEL_STATE_NEGOTIATED);
 
-			rpc_out_channel_transition_to_state(outChannel, CLIENT_OUT_CHANNEL_STATE_OPENED);
+				/* Send CONN/A1 PDU over OUT channel */
 
-			if (inChannel->State == CLIENT_IN_CHANNEL_STATE_OPENED)
-			{
-				rpc_virtual_connection_transition_to_state(
-				    rpc, connection, VIRTUAL_CONNECTION_STATE_OUT_CHANNEL_WAIT);
+				if (!rts_send_CONN_A1_pdu(rpc))
+				{
+					http_response_free(response);
+					WLog_ERR(TAG, "rpc_send_CONN_A1_pdu error!");
+					return -1;
+				}
+
+				rpc_out_channel_transition_to_state(outChannel, CLIENT_OUT_CHANNEL_STATE_OPENED);
+
+				if (inChannel->State == CLIENT_IN_CHANNEL_STATE_OPENED)
+				{
+					rpc_virtual_connection_transition_to_state(
+					    rpc, connection, VIRTUAL_CONNECTION_STATE_OUT_CHANNEL_WAIT);
+				}
 			}
 
 			status = 1;
@@ -544,12 +645,12 @@ static int rpc_client_default_out_channel_recv(rdpRpc* rpc)
 
 		if (statusCode != HTTP_STATUS_OK)
 		{
-			WLog_ERR(TAG, "error! Status Code: %" PRIu32 "", statusCode);
-			http_response_print(response);
+			http_response_log_error_status(WLog_Get(TAG), WLOG_ERROR, response);
 
 			if (statusCode == HTTP_STATUS_DENIED)
 			{
-				freerdp_set_last_error_if_not(rpc->context, FREERDP_ERROR_AUTHENTICATION_FAILED);
+				rdpContext* context = transport_get_context(rpc->transport);
+				freerdp_set_last_error_if_not(context, FREERDP_ERROR_CONNECT_ACCESS_DENIED);
 			}
 
 			http_response_free(response);
@@ -563,12 +664,13 @@ static int rpc_client_default_out_channel_recv(rdpRpc* rpc)
 	}
 	else
 	{
-		wStream* fragment;
-		rpcconn_common_hdr_t* header;
-		fragment = rpc->client->ReceiveFragment;
+		wStream* fragment = rpc->client->ReceiveFragment;
 
 		while (1)
 		{
+			size_t pos = 0;
+			rpcconn_common_hdr_t header = { 0 };
+
 			while (Stream_GetPosition(fragment) < RPC_COMMON_FIELDS_LENGTH)
 			{
 				status = rpc_channel_read(&outChannel->common, fragment,
@@ -581,22 +683,27 @@ static int rpc_client_default_out_channel_recv(rdpRpc* rpc)
 					return 0;
 			}
 
-			header = (rpcconn_common_hdr_t*)Stream_Buffer(fragment);
+			pos = Stream_GetPosition(fragment);
+			Stream_SetPosition(fragment, 0);
 
-			if (header->frag_length > rpc->max_recv_frag)
+			/* Ignore errors, the PDU might not be complete. */
+			rts_read_common_pdu_header(fragment, &header, TRUE);
+			Stream_SetPosition(fragment, pos);
+
+			if (header.frag_length > rpc->max_recv_frag)
 			{
 				WLog_ERR(TAG,
 				         "rpc_client_recv: invalid fragment size: %" PRIu16 " (max: %" PRIu16 ")",
-				         header->frag_length, rpc->max_recv_frag);
+				         header.frag_length, rpc->max_recv_frag);
 				winpr_HexDump(TAG, WLOG_ERROR, Stream_Buffer(fragment),
 				              Stream_GetPosition(fragment));
 				return -1;
 			}
 
-			while (Stream_GetPosition(fragment) < header->frag_length)
+			while (Stream_GetPosition(fragment) < header.frag_length)
 			{
 				status = rpc_channel_read(&outChannel->common, fragment,
-				                          header->frag_length - Stream_GetPosition(fragment));
+				                          header.frag_length - Stream_GetPosition(fragment));
 
 				if (status < 0)
 				{
@@ -604,14 +711,12 @@ static int rpc_client_default_out_channel_recv(rdpRpc* rpc)
 					return -1;
 				}
 
-				if (Stream_GetPosition(fragment) < header->frag_length)
+				if (Stream_GetPosition(fragment) < header.frag_length)
 					return 0;
 			}
 
 			{
 				/* complete fragment received */
-				Stream_SealLength(fragment);
-				Stream_SetPosition(fragment, 0);
 				status = rpc_client_recv_fragment(rpc, fragment);
 
 				if (status < 0)
@@ -639,11 +744,11 @@ static int rpc_client_default_out_channel_recv(rdpRpc* rpc)
 	return status;
 }
 
-static int rpc_client_nondefault_out_channel_recv(rdpRpc* rpc)
+static SSIZE_T rpc_client_nondefault_out_channel_recv(rdpRpc* rpc)
 {
-	int status = -1;
-	HttpResponse* response;
-	RpcOutChannel* nextOutChannel;
+	SSIZE_T status = -1;
+	HttpResponse* response = NULL;
+	RpcOutChannel* nextOutChannel = NULL;
 	HANDLE nextOutChannelEvent = NULL;
 	nextOutChannel = rpc->VirtualConnection->NonDefaultOutChannel;
 	BIO_get_event(nextOutChannel->common.tls->bio, &nextOutChannelEvent);
@@ -662,17 +767,24 @@ static int rpc_client_nondefault_out_channel_recv(rdpRpc* rpc)
 				{
 					if (rpc_ncacn_http_send_out_channel_request(&nextOutChannel->common, TRUE))
 					{
-						rpc_ncacn_http_ntlm_uninit(&nextOutChannel->common);
-						status = rts_send_OUT_R1_A3_pdu(rpc);
-
-						if (status >= 0)
+						if (rpc_ncacn_http_is_final_request(&nextOutChannel->common))
 						{
-							rpc_out_channel_transition_to_state(
-							    nextOutChannel, CLIENT_OUT_CHANNEL_STATE_OPENED_A6W);
+							rpc_ncacn_http_auth_uninit(&nextOutChannel->common);
+
+							if (rts_send_OUT_R1_A3_pdu(rpc))
+							{
+								status = 1;
+								rpc_out_channel_transition_to_state(
+								    nextOutChannel, CLIENT_OUT_CHANNEL_STATE_OPENED_A6W);
+							}
+							else
+							{
+								WLog_ERR(TAG, "rts_send_OUT_R1/A3_pdu failure");
+							}
 						}
 						else
 						{
-							WLog_ERR(TAG, "rts_send_OUT_R1/A3_pdu failure");
+							status = 1;
 						}
 					}
 					else
@@ -687,11 +799,14 @@ static int rpc_client_nondefault_out_channel_recv(rdpRpc* rpc)
 
 				break;
 
+			case CLIENT_OUT_CHANNEL_STATE_INITIAL:
+			case CLIENT_OUT_CHANNEL_STATE_CONNECTED:
+			case CLIENT_OUT_CHANNEL_STATE_NEGOTIATED:
 			default:
 				WLog_ERR(TAG,
 				         "rpc_client_nondefault_out_channel_recv: Unexpected message %08" PRIx32,
 				         nextOutChannel->State);
-				return -1;
+				status = -1;
 		}
 
 		http_response_free(response);
@@ -702,7 +817,7 @@ static int rpc_client_nondefault_out_channel_recv(rdpRpc* rpc)
 
 int rpc_client_out_channel_recv(rdpRpc* rpc)
 {
-	int status;
+	SSIZE_T status = 0;
 	RpcVirtualConnection* connection = rpc->VirtualConnection;
 
 	if (connection->DefaultOutChannel)
@@ -727,9 +842,9 @@ int rpc_client_out_channel_recv(rdpRpc* rpc)
 int rpc_client_in_channel_recv(rdpRpc* rpc)
 {
 	int status = 1;
-	HttpResponse* response;
-	RpcInChannel* inChannel;
-	RpcOutChannel* outChannel;
+	HttpResponse* response = NULL;
+	RpcInChannel* inChannel = NULL;
+	RpcOutChannel* outChannel = NULL;
 	HANDLE InChannelEvent = NULL;
 	RpcVirtualConnection* connection = rpc->VirtualConnection;
 	inChannel = connection->DefaultInChannel;
@@ -764,24 +879,27 @@ int rpc_client_in_channel_recv(rdpRpc* rpc)
 				return -1;
 			}
 
-			rpc_ncacn_http_ntlm_uninit(&inChannel->common);
-			rpc_in_channel_transition_to_state(inChannel, CLIENT_IN_CHANNEL_STATE_NEGOTIATED);
-
-			/* Send CONN/B1 PDU over IN channel */
-
-			if (rts_send_CONN_B1_pdu(rpc) < 0)
+			if (rpc_ncacn_http_is_final_request(&inChannel->common))
 			{
-				WLog_ERR(TAG, "rpc_send_CONN_B1_pdu error!");
-				http_response_free(response);
-				return -1;
-			}
+				rpc_ncacn_http_auth_uninit(&inChannel->common);
+				rpc_in_channel_transition_to_state(inChannel, CLIENT_IN_CHANNEL_STATE_NEGOTIATED);
 
-			rpc_in_channel_transition_to_state(inChannel, CLIENT_IN_CHANNEL_STATE_OPENED);
+				/* Send CONN/B1 PDU over IN channel */
 
-			if (outChannel->State == CLIENT_OUT_CHANNEL_STATE_OPENED)
-			{
-				rpc_virtual_connection_transition_to_state(
-				    rpc, connection, VIRTUAL_CONNECTION_STATE_OUT_CHANNEL_WAIT);
+				if (!rts_send_CONN_B1_pdu(rpc))
+				{
+					WLog_ERR(TAG, "rpc_send_CONN_B1_pdu error!");
+					http_response_free(response);
+					return -1;
+				}
+
+				rpc_in_channel_transition_to_state(inChannel, CLIENT_IN_CHANNEL_STATE_OPENED);
+
+				if (outChannel->State == CLIENT_OUT_CHANNEL_STATE_OPENED)
+				{
+					rpc_virtual_connection_transition_to_state(
+					    rpc, connection, VIRTUAL_CONNECTION_STATE_OUT_CHANNEL_WAIT);
+				}
 			}
 
 			status = 1;
@@ -810,17 +928,15 @@ int rpc_client_in_channel_recv(rdpRpc* rpc)
 
 RpcClientCall* rpc_client_call_find_by_id(RpcClient* client, UINT32 CallId)
 {
-	int index;
-	int count;
 	RpcClientCall* clientCall = NULL;
 
 	if (!client)
 		return NULL;
 
 	ArrayList_Lock(client->ClientCallList);
-	count = ArrayList_Count(client->ClientCallList);
+	const size_t count = ArrayList_Count(client->ClientCallList);
 
-	for (index = 0; index < count; index++)
+	for (size_t index = 0; index < count; index++)
 	{
 		clientCall = (RpcClientCall*)ArrayList_GetItem(client->ClientCallList, index);
 
@@ -834,7 +950,7 @@ RpcClientCall* rpc_client_call_find_by_id(RpcClient* client, UINT32 CallId)
 
 RpcClientCall* rpc_client_call_new(UINT32 CallId, UINT32 OpNum)
 {
-	RpcClientCall* clientCall;
+	RpcClientCall* clientCall = NULL;
 	clientCall = (RpcClientCall*)calloc(1, sizeof(RpcClientCall));
 
 	if (!clientCall)
@@ -856,18 +972,26 @@ static void rpc_array_client_call_free(void* call)
 	rpc_client_call_free((RpcClientCall*)call);
 }
 
-int rpc_in_channel_send_pdu(RpcInChannel* inChannel, BYTE* buffer, UINT32 length)
+int rpc_in_channel_send_pdu(RpcInChannel* inChannel, const BYTE* buffer, size_t length)
 {
-	int status;
-	RpcClientCall* clientCall;
-	rpcconn_common_hdr_t* header;
+	SSIZE_T status = 0;
+	RpcClientCall* clientCall = NULL;
+	wStream s;
+	rpcconn_common_hdr_t header = { 0 };
+
 	status = rpc_channel_write(&inChannel->common, buffer, length);
 
 	if (status <= 0)
 		return -1;
 
-	header = (rpcconn_common_hdr_t*)buffer;
-	clientCall = rpc_client_call_find_by_id(inChannel->common.client, header->call_id);
+	Stream_StaticConstInit(&s, buffer, length);
+	if (!rts_read_common_pdu_header(&s, &header, FALSE))
+		return -1;
+
+	clientCall = rpc_client_call_find_by_id(inChannel->common.client, header.call_id);
+	if (!clientCall)
+		return -1;
+
 	clientCall->State = RPC_CLIENT_CALL_STATE_DISPATCHED;
 
 	/*
@@ -877,29 +1001,31 @@ int rpc_in_channel_send_pdu(RpcInChannel* inChannel, BYTE* buffer, UINT32 length
 	 * variables specified by this abstract data model.
 	 */
 
-	if (header->ptype == PTYPE_REQUEST)
+	if (header.ptype == PTYPE_REQUEST)
 	{
 		inChannel->BytesSent += status;
 		inChannel->SenderAvailableWindow -= status;
 	}
 
-	return status;
+	if (status > INT32_MAX)
+		return -1;
+	return (int)status;
 }
 
 BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 {
-	UINT32 offset;
+	size_t offset = 0;
 	BYTE* buffer = NULL;
-	UINT32 stub_data_pad;
-	SecBuffer Buffers[2] = { 0 };
-	SecBufferDesc Message;
+	UINT32 stub_data_pad = 0;
+	SecBuffer plaintext;
+	SecBuffer ciphertext = { 0 };
 	RpcClientCall* clientCall = NULL;
-	rdpNtlm* ntlm;
-	rpcconn_request_hdr_t* request_pdu = NULL;
-	RpcVirtualConnection* connection;
-	RpcInChannel* inChannel;
-	size_t length;
-	SSIZE_T size;
+	rdpCredsspAuth* auth = NULL;
+	rpcconn_request_hdr_t request_pdu = { 0 };
+	RpcVirtualConnection* connection = NULL;
+	RpcInChannel* inChannel = NULL;
+	size_t length = 0;
+	size_t size = 0;
 	BOOL rc = FALSE;
 
 	if (!s)
@@ -908,12 +1034,12 @@ BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 	if (!rpc)
 		goto fail;
 
-	ntlm = rpc->ntlm;
+	auth = rpc->auth;
 	connection = rpc->VirtualConnection;
 
-	if (!ntlm)
+	if (!auth)
 	{
-		WLog_ERR(TAG, "invalid ntlm context");
+		WLog_ERR(TAG, "invalid auth context");
 		goto fail;
 	}
 
@@ -928,93 +1054,74 @@ BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 	Stream_SealLength(s);
 	length = Stream_Length(s);
 
-	if (ntlm_client_query_auth_size(ntlm) < 0)
-		goto fail;
+	size = credssp_auth_trailer_size(auth);
 
-	request_pdu = (rpcconn_request_hdr_t*)calloc(1, sizeof(rpcconn_request_hdr_t));
-
-	if (!request_pdu)
-		goto fail;
-
-	size = ntlm_client_get_context_max_size(ntlm);
-
-	if (size < 0)
-		goto fail;
-
-	rpc_pdu_header_init(rpc, (rpcconn_hdr_t*)request_pdu);
-	request_pdu->ptype = PTYPE_REQUEST;
-	request_pdu->pfc_flags = PFC_FIRST_FRAG | PFC_LAST_FRAG;
-	request_pdu->auth_length = (UINT16)size;
-	request_pdu->call_id = rpc->CallId++;
-	request_pdu->alloc_hint = length;
-	request_pdu->p_cont_id = 0x0000;
-	request_pdu->opnum = opnum;
-	clientCall = rpc_client_call_new(request_pdu->call_id, request_pdu->opnum);
+	request_pdu.header = rpc_pdu_header_init(rpc);
+	request_pdu.header.ptype = PTYPE_REQUEST;
+	request_pdu.header.pfc_flags = PFC_FIRST_FRAG | PFC_LAST_FRAG;
+	request_pdu.header.auth_length = (UINT16)size;
+	request_pdu.header.call_id = rpc->CallId++;
+	request_pdu.alloc_hint = length;
+	request_pdu.p_cont_id = 0x0000;
+	request_pdu.opnum = opnum;
+	clientCall = rpc_client_call_new(request_pdu.header.call_id, request_pdu.opnum);
 
 	if (!clientCall)
 		goto fail;
 
-	if (ArrayList_Add(rpc->client->ClientCallList, clientCall) < 0)
+	if (!ArrayList_Append(rpc->client->ClientCallList, clientCall))
 	{
 		rpc_client_call_free(clientCall);
 		goto fail;
 	}
 
-	if (request_pdu->opnum == TsProxySetupReceivePipeOpnum)
-		rpc->PipeCallId = request_pdu->call_id;
+	// NOLINTNEXTLINE(clang-analyzer-unix.Malloc): ArrayList_Append takes ownership of clientCall
+	if (request_pdu.opnum == TsProxySetupReceivePipeOpnum)
+		rpc->PipeCallId = request_pdu.header.call_id;
 
-	request_pdu->stub_data = Stream_Buffer(s);
+	request_pdu.stub_data = Stream_Buffer(s);
 	offset = 24;
 	stub_data_pad = rpc_offset_align(&offset, 8);
 	offset += length;
-	request_pdu->auth_verifier.auth_pad_length = rpc_offset_align(&offset, 4);
-	request_pdu->auth_verifier.auth_type = RPC_C_AUTHN_WINNT;
-	request_pdu->auth_verifier.auth_level = RPC_C_AUTHN_LEVEL_PKT_INTEGRITY;
-	request_pdu->auth_verifier.auth_reserved = 0x00;
-	request_pdu->auth_verifier.auth_context_id = 0x00000000;
-	offset += (8 + request_pdu->auth_length);
-	request_pdu->frag_length = offset;
-	buffer = (BYTE*)calloc(1, request_pdu->frag_length);
+	request_pdu.auth_verifier.auth_pad_length = rpc_offset_align(&offset, 4);
+	request_pdu.auth_verifier.auth_type =
+	    rpc_auth_pkg_to_security_provider(credssp_auth_pkg_name(rpc->auth));
+	request_pdu.auth_verifier.auth_level = RPC_C_AUTHN_LEVEL_PKT_INTEGRITY;
+	request_pdu.auth_verifier.auth_reserved = 0x00;
+	request_pdu.auth_verifier.auth_context_id = 0x00000000;
+	offset += (8 + request_pdu.header.auth_length);
+	request_pdu.header.frag_length = offset;
+	buffer = (BYTE*)calloc(1, request_pdu.header.frag_length);
 
 	if (!buffer)
 		goto fail;
 
-	CopyMemory(buffer, request_pdu, 24);
+	CopyMemory(buffer, &request_pdu, 24);
 	offset = 24;
 	rpc_offset_pad(&offset, stub_data_pad);
-	CopyMemory(&buffer[offset], request_pdu->stub_data, length);
+	CopyMemory(&buffer[offset], request_pdu.stub_data, length);
 	offset += length;
-	rpc_offset_pad(&offset, request_pdu->auth_verifier.auth_pad_length);
-	CopyMemory(&buffer[offset], &request_pdu->auth_verifier.auth_type, 8);
+	rpc_offset_pad(&offset, request_pdu.auth_verifier.auth_pad_length);
+	CopyMemory(&buffer[offset], &request_pdu.auth_verifier.auth_type, 8);
 	offset += 8;
-	Buffers[0].BufferType = SECBUFFER_DATA;  /* auth_data */
-	Buffers[1].BufferType = SECBUFFER_TOKEN; /* signature */
-	Buffers[0].pvBuffer = buffer;
-	Buffers[0].cbBuffer = offset;
-	Buffers[1].cbBuffer = size;
-	Buffers[1].pvBuffer = calloc(1, Buffers[1].cbBuffer);
 
-	if (!Buffers[1].pvBuffer)
+	plaintext.pvBuffer = buffer;
+	plaintext.cbBuffer = offset;
+	plaintext.BufferType = SECBUFFER_READONLY;
+	if (!credssp_auth_encrypt(auth, &plaintext, &ciphertext, &size, rpc->SendSeqNum++))
 		goto fail;
 
-	Message.cBuffers = 2;
-	Message.ulVersion = SECBUFFER_VERSION;
-	Message.pBuffers = (PSecBuffer)&Buffers;
+	CopyMemory(&buffer[offset], ciphertext.pvBuffer, size);
+	offset += size;
 
-	if (!ntlm_client_encrypt(ntlm, 0, &Message, rpc->SendSeqNum++))
-		goto fail;
+	sspi_SecBufferFree(&ciphertext);
 
-	CopyMemory(&buffer[offset], Buffers[1].pvBuffer, Buffers[1].cbBuffer);
-	offset += Buffers[1].cbBuffer;
-
-	if (rpc_in_channel_send_pdu(inChannel, buffer, request_pdu->frag_length) < 0)
+	if (rpc_in_channel_send_pdu(inChannel, buffer, request_pdu.header.frag_length) < 0)
 		goto fail;
 
 	rc = TRUE;
 fail:
 	free(buffer);
-	free(Buffers[1].pvBuffer);
-	free(request_pdu);
 	Stream_Free(s, TRUE);
 	return rc;
 }
@@ -1022,16 +1129,16 @@ fail:
 static BOOL rpc_client_resolve_gateway(rdpSettings* settings, char** host, UINT16* port,
                                        BOOL* isProxy)
 {
-	struct addrinfo* result;
+	struct addrinfo* result = NULL;
 
 	if (!settings || !host || !port || !isProxy)
 		return FALSE;
 	else
 	{
-		const char* peerHostname = settings->GatewayHostname;
-		const char* proxyUsername = settings->ProxyUsername;
-		const char* proxyPassword = settings->ProxyPassword;
-		*port = settings->GatewayPort;
+		const char* peerHostname = freerdp_settings_get_string(settings, FreeRDP_GatewayHostname);
+		const char* proxyUsername = freerdp_settings_get_string(settings, FreeRDP_GatewayUsername);
+		const char* proxyPassword = freerdp_settings_get_string(settings, FreeRDP_GatewayPassword);
+		*port = (UINT16)freerdp_settings_get_uint32(settings, FreeRDP_GatewayPort);
 		*isProxy = proxy_prepare(settings, &peerHostname, port, &proxyUsername, &proxyPassword);
 		result = freerdp_tcp_resolve_host(peerHostname, *port, 0);
 
@@ -1047,6 +1154,7 @@ static BOOL rpc_client_resolve_gateway(rdpSettings* settings, char** host, UINT1
 
 RpcClient* rpc_client_new(rdpContext* context, UINT32 max_recv_frag)
 {
+	wObject* obj = NULL;
 	RpcClient* client = (RpcClient*)calloc(1, sizeof(RpcClient));
 
 	if (!client)
@@ -1087,10 +1195,14 @@ RpcClient* rpc_client_new(rdpContext* context, UINT32 max_recv_frag)
 	if (!client->ClientCallList)
 		goto fail;
 
-	ArrayList_Object(client->ClientCallList)->fnObjectFree = rpc_array_client_call_free;
+	obj = ArrayList_Object(client->ClientCallList);
+	obj->fnObjectFree = rpc_array_client_call_free;
 	return client;
 fail:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	rpc_client_free(client);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 
@@ -1105,7 +1217,7 @@ void rpc_client_free(RpcClient* client)
 		Stream_Free(client->ReceiveFragment, TRUE);
 
 	if (client->PipeEvent)
-		CloseHandle(client->PipeEvent);
+		(void)CloseHandle(client->PipeEvent);
 
 	ringbuffer_destroy(&(client->ReceivePipe));
 	DeleteCriticalSection(&(client->PipeLock));
