@@ -19,6 +19,7 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <limits.h>
 
 #include <openssl/err.h>
 
@@ -30,6 +31,7 @@
 #include "tcp.h"
 
 #include <winpr/assert.h>
+#include <winpr/sysinfo.h>
 #include <winpr/environment.h> /* For GetEnvironmentVariableA */
 
 #define CRLF "\r\n"
@@ -70,9 +72,9 @@ static const char* rplstat[] = { "succeeded",
 	                             "Command not supported",
 	                             "Address type not supported" };
 
-static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
+static BOOL http_proxy_connect(rdpContext* context, BIO* bufferedBio, const char* proxyUsername,
                                const char* proxyPassword, const char* hostname, UINT16 port);
-static BOOL socks_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
+static BOOL socks_proxy_connect(rdpContext* context, BIO* bufferedBio, const char* proxyUsername,
                                 const char* proxyPassword, const char* hostname, UINT16 port);
 static void proxy_read_environment(rdpSettings* settings, char* envname);
 
@@ -245,10 +247,9 @@ static BOOL check_no_proxy(rdpSettings* settings, const char* no_proxy)
 				if (rangedelim != NULL)
 				{
 					const char* range = rangedelim + 1;
-					unsigned sub = 0;
-					int rc = sscanf(range, "%u", &sub);
+					const unsigned long sub = strtoul(range, NULL, 0);
 
-					if ((rc == 1) && (rc >= 0) && (sub <= UINT8_MAX))
+					if ((errno == 0) && (sub <= UINT8_MAX))
 					{
 						*rangedelim = '\0';
 
@@ -328,14 +329,16 @@ BOOL proxy_parse_uri(rdpSettings* settings, const char* uri_in)
 	BOOL rc = FALSE;
 	const char* protocol = "";
 	UINT16 port = 0;
-	char* p = NULL;
-	char* atPtr = NULL;
+
+	if (!settings || !uri_in)
+		return FALSE;
+
 	char* uri_copy = _strdup(uri_in);
 	char* uri = uri_copy;
 	if (!uri)
 		goto fail;
 
-	p = strstr(uri, "://");
+	char* p = strstr(uri, "://");
 
 	if (p)
 	{
@@ -375,7 +378,7 @@ BOOL proxy_parse_uri(rdpSettings* settings, const char* uri_in)
 	}
 
 	/* uri is now [user:password@]hostname:port */
-	atPtr = strrchr(uri, '@');
+	char* atPtr = strrchr(uri, '@');
 
 	if (atPtr)
 	{
@@ -479,13 +482,19 @@ BOOL proxy_parse_uri(rdpSettings* settings, const char* uri_in)
 	rc = TRUE;
 
 fail:
+	if (!rc)
+		WLog_WARN(TAG, "Failed to parse proxy configuration: %s://%s:%" PRIu16, protocol, uri,
+		          port);
 	free(uri_copy);
 	return rc;
 }
 
-BOOL proxy_connect(rdpSettings* settings, BIO* bufferedBio, const char* proxyUsername,
+BOOL proxy_connect(rdpContext* context, BIO* bufferedBio, const char* proxyUsername,
                    const char* proxyPassword, const char* hostname, UINT16 port)
 {
+	WINPR_ASSERT(context);
+	rdpSettings* settings = context->settings;
+
 	switch (freerdp_settings_get_uint32(settings, FreeRDP_ProxyType))
 	{
 		case PROXY_TYPE_NONE:
@@ -493,10 +502,12 @@ BOOL proxy_connect(rdpSettings* settings, BIO* bufferedBio, const char* proxyUse
 			return TRUE;
 
 		case PROXY_TYPE_HTTP:
-			return http_proxy_connect(bufferedBio, proxyUsername, proxyPassword, hostname, port);
+			return http_proxy_connect(context, bufferedBio, proxyUsername, proxyPassword, hostname,
+			                          port);
 
 		case PROXY_TYPE_SOCKS:
-			return socks_proxy_connect(bufferedBio, proxyUsername, proxyPassword, hostname, port);
+			return socks_proxy_connect(context, bufferedBio, proxyUsername, proxyPassword, hostname,
+			                           port);
 
 		default:
 			WLog_ERR(TAG, "Invalid internal proxy configuration");
@@ -516,7 +527,7 @@ static const char* get_response_header(char* response)
 	return response;
 }
 
-static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
+static BOOL http_proxy_connect(rdpContext* context, BIO* bufferedBio, const char* proxyUsername,
                                const char* proxyPassword, const char* hostname, UINT16 port)
 {
 	BOOL rc = FALSE;
@@ -532,8 +543,11 @@ static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 	const char connect[] = "CONNECT ";
 	const char httpheader[] = " HTTP/1.1" CRLF "Host: ";
 
+	WINPR_ASSERT(context);
 	WINPR_ASSERT(bufferedBio);
 	WINPR_ASSERT(hostname);
+	const UINT32 timeout =
+	    freerdp_settings_get_uint32(context->settings, FreeRDP_TcpConnectTimeout);
 
 	_itoa_s(port, port_str, sizeof(port_str), 10);
 
@@ -568,7 +582,7 @@ static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 				const char basic[] = CRLF "Proxy-Authorization: Basic ";
 				char* base64 = NULL;
 
-				sprintf_s(creds, size, "%s:%s", proxyUsername, proxyPassword);
+				(void)sprintf_s(creds, size, "%s:%s", proxyUsername, proxyPassword);
 				base64 = crypto_base64_encode((const BYTE*)creds, size - 1);
 
 				if (!base64 || !Stream_EnsureRemainingCapacity(s, strlen(basic) + strlen(base64)))
@@ -591,7 +605,11 @@ static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 
 	Stream_Write(s, CRLF CRLF, 4);
 	ERR_clear_error();
-	status = BIO_write(bufferedBio, Stream_Buffer(s), Stream_GetPosition(s));
+	const size_t pos = Stream_GetPosition(s);
+	if (pos > INT32_MAX)
+		goto fail;
+
+	status = BIO_write(bufferedBio, Stream_Buffer(s), (int)pos);
 
 	if ((status < 0) || ((size_t)status != Stream_GetPosition(s)))
 	{
@@ -601,6 +619,7 @@ static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 
 	/* Read result until CR-LF-CR-LF.
 	 * Keep recv_buf a null-terminated string. */
+	const UINT64 start = GetTickCount64();
 	while (strstr(recv_buf, CRLF CRLF) == NULL)
 	{
 		if (resultsize >= sizeof(recv_buf) - 1)
@@ -608,15 +627,17 @@ static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 			WLog_ERR(TAG, "HTTP Reply headers too long: %s", get_response_header(recv_buf));
 			goto fail;
 		}
+		const size_t rdsize = sizeof(recv_buf) - resultsize - 1ULL;
 
 		ERR_clear_error();
-		status =
-		    BIO_read(bufferedBio, (BYTE*)recv_buf + resultsize, sizeof(recv_buf) - resultsize - 1);
+
+		WINPR_ASSERT(rdsize <= INT32_MAX);
+		status = BIO_read(bufferedBio, (BYTE*)recv_buf + resultsize, (int)rdsize);
 
 		if (status < 0)
 		{
 			/* Error? */
-			if (BIO_should_retry(bufferedBio))
+			if (!freerdp_shall_disconnect_context(context) && BIO_should_retry(bufferedBio))
 			{
 				USleep(100);
 				continue;
@@ -626,6 +647,18 @@ static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 			goto fail;
 		}
 		else if (status == 0)
+		{
+			const UINT64 now = GetTickCount64();
+			const UINT64 diff = now - start;
+			if (freerdp_shall_disconnect_context(context) || (now < start) || (diff > timeout))
+			{
+				/* Error? */
+				WLog_ERR(TAG, "Failed reading reply from HTTP proxy (BIO_read returned zero)");
+				goto fail;
+			}
+			Sleep(10);
+		}
+		else
 		{
 			/* Error? */
 			WLog_ERR(TAG, "Failed reading reply from HTTP proxy (BIO_read returned zero)");
@@ -652,7 +685,7 @@ static BOOL http_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 
 	recv_buf[7] = 'X';
 
-	if (strncmp(recv_buf, "HTTP/1.X 200", 12))
+	if (strncmp(recv_buf, "HTTP/1.X 200", 12) != 0)
 		goto fail;
 
 	rc = TRUE;
@@ -661,10 +694,16 @@ fail:
 	return rc;
 }
 
-static int recv_socks_reply(BIO* bufferedBio, BYTE* buf, int len, char* reason, BYTE checkVer)
+static int recv_socks_reply(rdpContext* context, BIO* bufferedBio, BYTE* buf, int len, char* reason,
+                            BYTE checkVer)
 {
 	int status = 0;
 
+	WINPR_ASSERT(context);
+
+	const UINT32 timeout =
+	    freerdp_settings_get_uint32(context->settings, FreeRDP_TcpConnectTimeout);
+	const UINT64 start = GetTickCount64();
 	for (;;)
 	{
 		ERR_clear_error();
@@ -677,7 +716,7 @@ static int recv_socks_reply(BIO* bufferedBio, BYTE* buf, int len, char* reason,
 		else if (status < 0)
 		{
 			/* Error? */
-			if (BIO_should_retry(bufferedBio))
+			if (!freerdp_shall_disconnect_context(context) && BIO_should_retry(bufferedBio))
 			{
 				USleep(100);
 				continue;
@@ -686,6 +725,19 @@ static int recv_socks_reply(BIO* bufferedBio, BYTE* buf, int len, char* reason,
 			WLog_ERR(TAG, "Failed reading %s reply from SOCKS proxy (Status %d)", reason, status);
 			return -1;
 		}
+		else if (status == 0)
+		{
+			const UINT64 now = GetTickCount64();
+			const UINT64 diff = now - start;
+			if (freerdp_shall_disconnect_context(context) || (now < start) || (diff > timeout))
+			{
+				/* Error? */
+				WLog_ERR(TAG, "Failed reading %s reply from SOCKS proxy (BIO_read returned zero)",
+				         reason);
+				return status;
+			}
+			Sleep(10);
+		}
 		else // if (status == 0)
 		{
 			/* Error? */
@@ -710,14 +762,14 @@ static int recv_socks_reply(BIO* bufferedBio, BYTE* buf, int len, char* reason,
 	return status;
 }
 
-static BOOL socks_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
+static BOOL socks_proxy_connect(rdpContext* context, BIO* bufferedBio, const char* proxyUsername,
                                 const char* proxyPassword, const char* hostname, UINT16 port)
 {
 	int status = 0;
 	int nauthMethods = 1;
 	int writeLen = 3;
-	BYTE buf[3 + 255 + 255]; /* biggest packet is user/pass auth */
-	size_t hostnlen = strnlen(hostname, 255);
+	BYTE buf[3 + 255 + 255] = { 0 }; /* biggest packet is user/pass auth */
+	const size_t hostnlen = strnlen(hostname, 255);
 
 	if (proxyUsername && proxyPassword)
 	{
@@ -742,7 +794,7 @@ static BOOL socks_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 		return FALSE;
 	}
 
-	status = recv_socks_reply(bufferedBio, buf, 2, "AUTH REQ", 5);
+	status = recv_socks_reply(context, bufferedBio, buf, 2, "AUTH REQ", 5);
 
 	if (status <= 0)
 		return FALSE;
@@ -758,8 +810,8 @@ static BOOL socks_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 				return FALSE;
 			else
 			{
-				int usernameLen = strnlen(proxyUsername, 255);
-				int userpassLen = strnlen(proxyPassword, 255);
+				const BYTE usernameLen = (BYTE)strnlen(proxyUsername, 255);
+				const BYTE userpassLen = (BYTE)strnlen(proxyPassword, 255);
 				BYTE* ptr = NULL;
 
 				if (nauthMethods < 2)
@@ -786,7 +838,7 @@ static BOOL socks_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 					return FALSE;
 				}
 
-				status = recv_socks_reply(bufferedBio, buf, 2, "AUTH REQ", 1);
+				status = recv_socks_reply(context, bufferedBio, buf, 2, "AUTH REQ", 1);
 
 				if (status < 2)
 					return FALSE;
@@ -810,13 +862,13 @@ static BOOL socks_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 	buf[1] = SOCKS_CMD_CONNECT; /* command */
 	buf[2] = 0;                 /* 3rd octet is reserved x00 */
 	buf[3] = SOCKS_ADDR_FQDN;   /* addr.type */
-	buf[4] = hostnlen;          /* DST.ADDR */
+	buf[4] = (BYTE)hostnlen;    /* DST.ADDR */
 	memcpy(buf + 5, hostname, hostnlen);
 	/* follows DST.PORT in netw. format */
 	buf[hostnlen + 5] = (port >> 8) & 0xff;
 	buf[hostnlen + 6] = port & 0xff;
 	ERR_clear_error();
-	status = BIO_write(bufferedBio, buf, hostnlen + 7U);
+	status = BIO_write(bufferedBio, buf, (int)hostnlen + 7);
 
 	if ((status < 0) || ((size_t)status != (hostnlen + 7U)))
 	{
@@ -824,7 +876,7 @@ static BOOL socks_proxy_connect(BIO* bufferedBio, const char* proxyUsername,
 		return FALSE;
 	}
 
-	status = recv_socks_reply(bufferedBio, buf, sizeof(buf), "CONN REQ", 5);
+	status = recv_socks_reply(context, bufferedBio, buf, sizeof(buf), "CONN REQ", 5);
 
 	if (status < 4)
 		return FALSE;
