@@ -41,10 +41,10 @@ static primitives_t* generic = NULL;
 
 #define CACHE_LINE_BYTES 64
 
-#define _mm_between_epi16(_val, _min, _max)                    \
-	do                                                         \
-	{                                                          \
-		_val = _mm_min_epi16(_max, _mm_max_epi16(_val, _min)); \
+#define mm_between_epi16(_val, _min, _max)                       \
+	do                                                           \
+	{                                                            \
+		(_val) = _mm_min_epi16(_max, _mm_max_epi16(_val, _min)); \
 	} while (0)
 
 #ifdef DO_PREFETCH
@@ -63,7 +63,7 @@ static inline void GNU_INLINE _mm_prefetch_buffer(char* WINPR_RESTRICT buffer, i
 
 /*---------------------------------------------------------------------------*/
 static pstatus_t
-sse2_yCbCrToRGB_16s16s_P3P3(const INT16* const WINPR_RESTRICT pSrc[3], int srcStep,
+sse2_yCbCrToRGB_16s16s_P3P3(const INT16* WINPR_RESTRICT pSrc[3], int srcStep,
                             INT16* WINPR_RESTRICT pDst[3], int dstStep,
                             const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
@@ -173,20 +173,20 @@ sse2_yCbCrToRGB_16s16s_P3P3(const INT16* const WINPR_RESTRICT pSrc[3], int srcSt
 			r = _mm_add_epi16(y, _mm_mulhi_epi16(cr, r_cr));
 			r = _mm_srai_epi16(r, 3);
 			/* r_buf[i] = CLIP(r); */
-			_mm_between_epi16(r, zero, max);
+			mm_between_epi16(r, zero, max);
 			_mm_store_si128(r_buf + i, r);
 			/* (y + HIWORD(cb*-5636) + HIWORD(cr*-11698)) >> 3 */
 			g = _mm_add_epi16(y, _mm_mulhi_epi16(cb, g_cb));
 			g = _mm_add_epi16(g, _mm_mulhi_epi16(cr, g_cr));
 			g = _mm_srai_epi16(g, 3);
 			/* g_buf[i] = CLIP(g); */
-			_mm_between_epi16(g, zero, max);
+			mm_between_epi16(g, zero, max);
 			_mm_store_si128(g_buf + i, g);
 			/* (y + HIWORD(cb*28999)) >> 3 */
 			b = _mm_add_epi16(y, _mm_mulhi_epi16(cb, b_cb));
 			b = _mm_srai_epi16(b, 3);
 			/* b_buf[i] = CLIP(b); */
-			_mm_between_epi16(b, zero, max);
+			mm_between_epi16(b, zero, max);
 			_mm_store_si128(b_buf + i, b);
 		}
 
@@ -203,7 +203,7 @@ sse2_yCbCrToRGB_16s16s_P3P3(const INT16* const WINPR_RESTRICT pSrc[3], int srcSt
 
 /*---------------------------------------------------------------------------*/
 static pstatus_t
-sse2_yCbCrToRGB_16s8u_P3AC4R_BGRX(const INT16* const WINPR_RESTRICT pSrc[3], UINT32 srcStep,
+sse2_yCbCrToRGB_16s8u_P3AC4R_BGRX(const INT16* WINPR_RESTRICT pSrc[3], UINT32 srcStep,
                                   BYTE* WINPR_RESTRICT pDst, UINT32 dstStep,
                                   const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
@@ -214,9 +214,9 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_BGRX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 	const __m128i g_cr = _mm_set1_epi16(-11698); /* -0.714 << 14 */
 	const __m128i b_cb = _mm_set1_epi16(28999);  /*  1.770 << 14 */
 	const __m128i c4096 = _mm_set1_epi16(4096);
-	const INT16* y_buf = (const INT16*)pSrc[0];
-	const INT16* cb_buf = (const INT16*)pSrc[1];
-	const INT16* cr_buf = (const INT16*)pSrc[2];
+	const INT16* y_buf = pSrc[0];
+	const INT16* cb_buf = pSrc[1];
+	const INT16* cr_buf = pSrc[2];
 	const UINT32 pad = roi->width % 16;
 	const UINT32 step = sizeof(__m128i) / sizeof(INT16);
 	const UINT32 imax = (roi->width - pad) * sizeof(INT16) / sizeof(__m128i);
@@ -294,18 +294,18 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_BGRX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 			r1 = _mm_add_epi16(y1, _mm_mulhi_epi16(cr1, r_cr));
 			r1 = _mm_srai_epi16(r1, 3);
 			/* r_buf[i] = CLIP(r); */
-			_mm_between_epi16(r1, zero, max);
+			mm_between_epi16(r1, zero, max);
 			/* (y + HIWORD(cb*-5636) + HIWORD(cr*-11698)) >> 3 */
 			g1 = _mm_add_epi16(y1, _mm_mulhi_epi16(cb1, g_cb));
 			g1 = _mm_add_epi16(g1, _mm_mulhi_epi16(cr1, g_cr));
 			g1 = _mm_srai_epi16(g1, 3);
 			/* g_buf[i] = CLIP(g); */
-			_mm_between_epi16(g1, zero, max);
+			mm_between_epi16(g1, zero, max);
 			/* (y + HIWORD(cb*28999)) >> 3 */
 			b1 = _mm_add_epi16(y1, _mm_mulhi_epi16(cb1, b_cb));
 			b1 = _mm_srai_epi16(b1, 3);
 			/* b_buf[i] = CLIP(b); */
-			_mm_between_epi16(b1, zero, max);
+			mm_between_epi16(b1, zero, max);
 			y2 = _mm_load_si128((const __m128i*)y_buf);
 			y_buf += step;
 			y2 = _mm_add_epi16(y2, c4096);
@@ -320,18 +320,18 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_BGRX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 			r2 = _mm_add_epi16(y2, _mm_mulhi_epi16(cr2, r_cr));
 			r2 = _mm_srai_epi16(r2, 3);
 			/* r_buf[i] = CLIP(r); */
-			_mm_between_epi16(r2, zero, max);
+			mm_between_epi16(r2, zero, max);
 			/* (y + HIWORD(cb*-5636) + HIWORD(cr*-11698)) >> 3 */
 			g2 = _mm_add_epi16(y2, _mm_mulhi_epi16(cb2, g_cb));
 			g2 = _mm_add_epi16(g2, _mm_mulhi_epi16(cr2, g_cr));
 			g2 = _mm_srai_epi16(g2, 3);
 			/* g_buf[i] = CLIP(g); */
-			_mm_between_epi16(g2, zero, max);
+			mm_between_epi16(g2, zero, max);
 			/* (y + HIWORD(cb*28999)) >> 3 */
 			b2 = _mm_add_epi16(y2, _mm_mulhi_epi16(cb2, b_cb));
 			b2 = _mm_srai_epi16(b2, 3);
 			/* b_buf[i] = CLIP(b); */
-			_mm_between_epi16(b2, zero, max);
+			mm_between_epi16(b2, zero, max);
 			{
 				__m128i R0;
 				__m128i R1;
@@ -401,7 +401,7 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_BGRX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 
 /*---------------------------------------------------------------------------*/
 static pstatus_t
-sse2_yCbCrToRGB_16s8u_P3AC4R_RGBX(const INT16* const WINPR_RESTRICT pSrc[3], UINT32 srcStep,
+sse2_yCbCrToRGB_16s8u_P3AC4R_RGBX(const INT16* WINPR_RESTRICT pSrc[3], UINT32 srcStep,
                                   BYTE* WINPR_RESTRICT pDst, UINT32 dstStep,
                                   const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
@@ -412,9 +412,9 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_RGBX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 	const __m128i g_cr = _mm_set1_epi16(-11698); /* -0.714 << 14 */
 	const __m128i b_cb = _mm_set1_epi16(28999);  /*  1.770 << 14 */
 	const __m128i c4096 = _mm_set1_epi16(4096);
-	const INT16* y_buf = (const INT16*)pSrc[0];
-	const INT16* cb_buf = (const INT16*)pSrc[1];
-	const INT16* cr_buf = (const INT16*)pSrc[2];
+	const INT16* y_buf = pSrc[0];
+	const INT16* cb_buf = pSrc[1];
+	const INT16* cr_buf = pSrc[2];
 	const UINT32 pad = roi->width % 16;
 	const UINT32 step = sizeof(__m128i) / sizeof(INT16);
 	const UINT32 imax = (roi->width - pad) * sizeof(INT16) / sizeof(__m128i);
@@ -492,18 +492,18 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_RGBX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 			r1 = _mm_add_epi16(y1, _mm_mulhi_epi16(cr1, r_cr));
 			r1 = _mm_srai_epi16(r1, 3);
 			/* r_buf[i] = CLIP(r); */
-			_mm_between_epi16(r1, zero, max);
+			mm_between_epi16(r1, zero, max);
 			/* (y + HIWORD(cb*-5636) + HIWORD(cr*-11698)) >> 3 */
 			g1 = _mm_add_epi16(y1, _mm_mulhi_epi16(cb1, g_cb));
 			g1 = _mm_add_epi16(g1, _mm_mulhi_epi16(cr1, g_cr));
 			g1 = _mm_srai_epi16(g1, 3);
 			/* g_buf[i] = CLIP(g); */
-			_mm_between_epi16(g1, zero, max);
+			mm_between_epi16(g1, zero, max);
 			/* (y + HIWORD(cb*28999)) >> 3 */
 			b1 = _mm_add_epi16(y1, _mm_mulhi_epi16(cb1, b_cb));
 			b1 = _mm_srai_epi16(b1, 3);
 			/* b_buf[i] = CLIP(b); */
-			_mm_between_epi16(b1, zero, max);
+			mm_between_epi16(b1, zero, max);
 			y2 = _mm_load_si128((const __m128i*)y_buf);
 			y_buf += step;
 			y2 = _mm_add_epi16(y2, c4096);
@@ -518,18 +518,18 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_RGBX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 			r2 = _mm_add_epi16(y2, _mm_mulhi_epi16(cr2, r_cr));
 			r2 = _mm_srai_epi16(r2, 3);
 			/* r_buf[i] = CLIP(r); */
-			_mm_between_epi16(r2, zero, max);
+			mm_between_epi16(r2, zero, max);
 			/* (y + HIWORD(cb*-5636) + HIWORD(cr*-11698)) >> 3 */
 			g2 = _mm_add_epi16(y2, _mm_mulhi_epi16(cb2, g_cb));
 			g2 = _mm_add_epi16(g2, _mm_mulhi_epi16(cr2, g_cr));
 			g2 = _mm_srai_epi16(g2, 3);
 			/* g_buf[i] = CLIP(g); */
-			_mm_between_epi16(g2, zero, max);
+			mm_between_epi16(g2, zero, max);
 			/* (y + HIWORD(cb*28999)) >> 3 */
 			b2 = _mm_add_epi16(y2, _mm_mulhi_epi16(cb2, b_cb));
 			b2 = _mm_srai_epi16(b2, 3);
 			/* b_buf[i] = CLIP(b); */
-			_mm_between_epi16(b2, zero, max);
+			mm_between_epi16(b2, zero, max);
 			{
 				__m128i R0;
 				__m128i R1;
@@ -598,7 +598,7 @@ sse2_yCbCrToRGB_16s8u_P3AC4R_RGBX(const INT16* const WINPR_RESTRICT pSrc[3], UIN
 }
 
 static pstatus_t
-sse2_yCbCrToRGB_16s8u_P3AC4R(const INT16* const WINPR_RESTRICT pSrc[3], UINT32 srcStep,
+sse2_yCbCrToRGB_16s8u_P3AC4R(const INT16* WINPR_RESTRICT pSrc[3], UINT32 srcStep,
                              BYTE* WINPR_RESTRICT pDst, UINT32 dstStep, UINT32 DstFormat,
                              const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
@@ -628,7 +628,7 @@ sse2_yCbCrToRGB_16s8u_P3AC4R(const INT16* const WINPR_RESTRICT pSrc[3], UINT32 s
  * numbers. See the general code above.
  */
 static pstatus_t
-sse2_RGBToYCbCr_16s16s_P3P3(const INT16* const WINPR_RESTRICT pSrc[3], int srcStep,
+sse2_RGBToYCbCr_16s16s_P3P3(const INT16* WINPR_RESTRICT pSrc[3], int srcStep,
                             INT16* WINPR_RESTRICT pDst[3], int dstStep,
                             const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
@@ -734,21 +734,21 @@ sse2_RGBToYCbCr_16s16s_P3P3(const INT16* const WINPR_RESTRICT pSrc[3], int srcSt
 			y = _mm_add_epi16(y, _mm_mulhi_epi16(b, y_b));
 			y = _mm_add_epi16(y, min);
 			/* y_r_buf[i] = MINMAX(y, 0, (255 << 5)) - (128 << 5); */
-			_mm_between_epi16(y, min, max);
+			mm_between_epi16(y, min, max);
 			_mm_store_si128(y_buf + i, y);
 			/* cb = HIWORD(r*cb_r) + HIWORD(g*cb_g) + HIWORD(b*cb_b) */
 			cb = _mm_mulhi_epi16(r, cb_r);
 			cb = _mm_add_epi16(cb, _mm_mulhi_epi16(g, cb_g));
 			cb = _mm_add_epi16(cb, _mm_mulhi_epi16(b, cb_b));
 			/* cb_g_buf[i] = MINMAX(cb, (-128 << 5), (127 << 5)); */
-			_mm_between_epi16(cb, min, max);
+			mm_between_epi16(cb, min, max);
 			_mm_store_si128(cb_buf + i, cb);
 			/* cr = HIWORD(r*cr_r) + HIWORD(g*cr_g) + HIWORD(b*cr_b) */
 			cr = _mm_mulhi_epi16(r, cr_r);
 			cr = _mm_add_epi16(cr, _mm_mulhi_epi16(g, cr_g));
 			cr = _mm_add_epi16(cr, _mm_mulhi_epi16(b, cr_b));
 			/* cr_b_buf[i] = MINMAX(cr, (-128 << 5), (127 << 5)); */
-			_mm_between_epi16(cr, min, max);
+			mm_between_epi16(cr, min, max);
 			_mm_store_si128(cr_buf + i, cr);
 		}
 
@@ -765,11 +765,11 @@ sse2_RGBToYCbCr_16s16s_P3P3(const INT16* const WINPR_RESTRICT pSrc[3], int srcSt
 
 /*---------------------------------------------------------------------------*/
 static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_BGRX(
-    const INT16* const WINPR_RESTRICT pSrc[3], /* 16-bit R,G, and B arrays */
-    UINT32 srcStep,                            /* bytes between rows in source data */
-    BYTE* WINPR_RESTRICT pDst,                 /* 32-bit interleaved ARGB (ABGR?) data */
-    UINT32 dstStep,                            /* bytes between rows in dest data */
-    const prim_size_t* WINPR_RESTRICT roi)     /* region of interest */
+    const INT16* WINPR_RESTRICT pSrc[3],   /* 16-bit R,G, and B arrays */
+    UINT32 srcStep,                        /* bytes between rows in source data */
+    BYTE* WINPR_RESTRICT pDst,             /* 32-bit interleaved ARGB (ABGR?) data */
+    UINT32 dstStep,                        /* bytes between rows in dest data */
+    const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
 	const UINT16* pr = (const UINT16*)(pSrc[0]);
 	const UINT16* pg = (const UINT16*)(pSrc[1]);
@@ -779,7 +779,7 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_BGRX(
 	BYTE* out = NULL;
 	UINT32 srcbump = 0;
 	UINT32 dstbump = 0;
-	out = (BYTE*)pDst;
+	out = pDst;
 	srcbump = (srcStep - (roi->width * sizeof(UINT16))) / sizeof(UINT16);
 	dstbump = (dstStep - (roi->width * sizeof(UINT32)));
 
@@ -876,11 +876,11 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_BGRX(
 }
 
 static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_RGBX(
-    const INT16* const WINPR_RESTRICT pSrc[3], /* 16-bit R,G, and B arrays */
-    UINT32 srcStep,                            /* bytes between rows in source data */
-    BYTE* WINPR_RESTRICT pDst,                 /* 32-bit interleaved ARGB (ABGR?) data */
-    UINT32 dstStep,                            /* bytes between rows in dest data */
-    const prim_size_t* WINPR_RESTRICT roi)     /* region of interest */
+    const INT16* WINPR_RESTRICT pSrc[3],   /* 16-bit R,G, and B arrays */
+    UINT32 srcStep,                        /* bytes between rows in source data */
+    BYTE* WINPR_RESTRICT pDst,             /* 32-bit interleaved ARGB (ABGR?) data */
+    UINT32 dstStep,                        /* bytes between rows in dest data */
+    const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
 	const UINT16* pr = (const UINT16*)(pSrc[0]);
 	const UINT16* pg = (const UINT16*)(pSrc[1]);
@@ -890,7 +890,7 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_RGBX(
 	BYTE* out = NULL;
 	UINT32 srcbump = 0;
 	UINT32 dstbump = 0;
-	out = (BYTE*)pDst;
+	out = pDst;
 	srcbump = (srcStep - (roi->width * sizeof(UINT16))) / sizeof(UINT16);
 	dstbump = (dstStep - (roi->width * sizeof(UINT32)));
 
@@ -987,11 +987,11 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_RGBX(
 }
 
 static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_XBGR(
-    const INT16* const WINPR_RESTRICT pSrc[3], /* 16-bit R,G, and B arrays */
-    UINT32 srcStep,                            /* bytes between rows in source data */
-    BYTE* WINPR_RESTRICT pDst,                 /* 32-bit interleaved ARGB (ABGR?) data */
-    UINT32 dstStep,                            /* bytes between rows in dest data */
-    const prim_size_t* WINPR_RESTRICT roi)     /* region of interest */
+    const INT16* WINPR_RESTRICT pSrc[3],   /* 16-bit R,G, and B arrays */
+    UINT32 srcStep,                        /* bytes between rows in source data */
+    BYTE* WINPR_RESTRICT pDst,             /* 32-bit interleaved ARGB (ABGR?) data */
+    UINT32 dstStep,                        /* bytes between rows in dest data */
+    const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
 	const UINT16* pr = (const UINT16*)(pSrc[0]);
 	const UINT16* pg = (const UINT16*)(pSrc[1]);
@@ -1001,7 +1001,7 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_XBGR(
 	BYTE* out = NULL;
 	UINT32 srcbump = 0;
 	UINT32 dstbump = 0;
-	out = (BYTE*)pDst;
+	out = pDst;
 	srcbump = (srcStep - (roi->width * sizeof(UINT16))) / sizeof(UINT16);
 	dstbump = (dstStep - (roi->width * sizeof(UINT32)));
 
@@ -1098,11 +1098,11 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_XBGR(
 }
 
 static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_XRGB(
-    const INT16* const WINPR_RESTRICT pSrc[3], /* 16-bit R,G, and B arrays */
-    UINT32 srcStep,                            /* bytes between rows in source data */
-    BYTE* WINPR_RESTRICT pDst,                 /* 32-bit interleaved ARGB (ABGR?) data */
-    UINT32 dstStep,                            /* bytes between rows in dest data */
-    const prim_size_t* WINPR_RESTRICT roi)     /* region of interest */
+    const INT16* WINPR_RESTRICT pSrc[3],   /* 16-bit R,G, and B arrays */
+    UINT32 srcStep,                        /* bytes between rows in source data */
+    BYTE* WINPR_RESTRICT pDst,             /* 32-bit interleaved ARGB (ABGR?) data */
+    UINT32 dstStep,                        /* bytes between rows in dest data */
+    const prim_size_t* WINPR_RESTRICT roi) /* region of interest */
 {
 	const UINT16* pr = (const UINT16*)(pSrc[0]);
 	const UINT16* pg = (const UINT16*)(pSrc[1]);
@@ -1112,7 +1112,7 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_XRGB(
 	BYTE* out = NULL;
 	UINT32 srcbump = 0;
 	UINT32 dstbump = 0;
-	out = (BYTE*)pDst;
+	out = pDst;
 	srcbump = (srcStep - (roi->width * sizeof(UINT16))) / sizeof(UINT16);
 	dstbump = (dstStep - (roi->width * sizeof(UINT32)));
 
@@ -1209,7 +1209,7 @@ static pstatus_t sse2_RGBToRGB_16s8u_P3AC4R_XRGB(
 }
 
 static pstatus_t
-sse2_RGBToRGB_16s8u_P3AC4R(const INT16* const WINPR_RESTRICT pSrc[3], /* 16-bit R,G, and B arrays */
+sse2_RGBToRGB_16s8u_P3AC4R(const INT16* WINPR_RESTRICT pSrc[3], /* 16-bit R,G, and B arrays */
                            UINT32 srcStep,            /* bytes between rows in source data */
                            BYTE* WINPR_RESTRICT pDst, /* 32-bit interleaved ARGB (ABGR?) data */
                            UINT32 dstStep,            /* bytes between rows in dest data */
