@@ -333,6 +333,10 @@ static BOOL copy(WCHAR** dst, ULONG* dstLen, const WCHAR* what, size_t len)
 	*dst = NULL;
 	*dstLen = 0;
 
+	if (!what && (len != 0))
+		return FALSE;
+	if (len == 0)
+		return TRUE;
 	*dst = calloc(sizeof(WCHAR), len + 1);
 	if (!*dst)
 		return FALSE;
@@ -949,7 +953,7 @@ static BOOL WINPR_init(void)
 
 	for (size_t x = 0; x < ARRAYSIZE(SecurityFunctionTableA_NAME_LIST); x++)
 	{
-		const SecurityFunctionTableA_NAME* const cur = &SecurityFunctionTableA_NAME_LIST[x];
+		const SecurityFunctionTableA_NAME* cur = &SecurityFunctionTableA_NAME_LIST[x];
 		InitializeConstWCharFromUtf8(cur->Name, BUFFER_NAME_LIST_W[x],
 		                             ARRAYSIZE(BUFFER_NAME_LIST_W[x]));
 	}
@@ -987,7 +991,7 @@ void sspi_GlobalFinish(void)
 
 static const SecurityFunctionTableA* sspi_GetSecurityFunctionTableAByNameA(const SEC_CHAR* Name)
 {
-	size_t cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
 
 	for (size_t index = 0; index < cPackages; index++)
 	{
@@ -1002,7 +1006,7 @@ static const SecurityFunctionTableA* sspi_GetSecurityFunctionTableAByNameA(const
 
 static const SecurityFunctionTableW* sspi_GetSecurityFunctionTableWByNameW(const SEC_WCHAR* Name)
 {
-	size_t cPackages = sizeof(SecPkgInfoW_LIST) / sizeof(*(SecPkgInfoW_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoW_LIST);
 
 	for (size_t index = 0; index < cPackages; index++)
 	{
@@ -1073,12 +1077,10 @@ static void sspi_ContextBufferFree(void* contextBuffer)
 static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesW(ULONG* pcPackages,
                                                                   PSecPkgInfoW* ppPackageInfo)
 {
-	size_t size = 0;
-	UINT32 cPackages = 0;
-	SecPkgInfoW* pPackageInfo = NULL;
-	cPackages = sizeof(SecPkgInfoW_LIST) / sizeof(*(SecPkgInfoW_LIST));
-	size = sizeof(SecPkgInfoW) * cPackages;
-	pPackageInfo = (SecPkgInfoW*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
+	size_t cPackages = ARRAYSIZE(SecPkgInfoW_LIST);
+	size_t size = sizeof(SecPkgInfoW) * cPackages;
+	SecPkgInfoW* pPackageInfo =
+	    (SecPkgInfoW*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
 
 	if (!pPackageInfo)
 		return SEC_E_INSUFFICIENT_MEMORY;
@@ -1101,12 +1103,10 @@ static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesW(ULONG* pcPacka
 static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesA(ULONG* pcPackages,
                                                                   PSecPkgInfoA* ppPackageInfo)
 {
-	size_t size = 0;
-	UINT32 cPackages = 0;
-	SecPkgInfoA* pPackageInfo = NULL;
-	cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
-	size = sizeof(SecPkgInfoA) * cPackages;
-	pPackageInfo = (SecPkgInfoA*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
+	size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
+	size_t size = sizeof(SecPkgInfoA) * cPackages;
+	SecPkgInfoA* pPackageInfo =
+	    (SecPkgInfoA*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
 
 	if (!pPackageInfo)
 		return SEC_E_INSUFFICIENT_MEMORY;
@@ -1134,9 +1134,8 @@ static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesA(ULONG* pcPacka
 
 static void FreeContextBuffer_EnumerateSecurityPackages(void* contextBuffer)
 {
-	UINT32 cPackages = 0;
 	SecPkgInfoA* pPackageInfo = (SecPkgInfoA*)contextBuffer;
-	cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
 
 	if (!pPackageInfo)
 		return;
@@ -1163,16 +1162,14 @@ SecurityFunctionTableA* SEC_ENTRY winpr_InitSecurityInterfaceA(void)
 static SECURITY_STATUS SEC_ENTRY winpr_QuerySecurityPackageInfoW(SEC_WCHAR* pszPackageName,
                                                                  PSecPkgInfoW* ppPackageInfo)
 {
-	size_t size = 0;
-	SecPkgInfoW* pPackageInfo = NULL;
-	size_t cPackages = sizeof(SecPkgInfoW_LIST) / sizeof(*(SecPkgInfoW_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoW_LIST);
 
 	for (size_t index = 0; index < cPackages; index++)
 	{
 		if (_wcscmp(pszPackageName, SecPkgInfoW_LIST[index]->Name) == 0)
 		{
-			size = sizeof(SecPkgInfoW);
-			pPackageInfo =
+			size_t size = sizeof(SecPkgInfoW);
+			SecPkgInfoW* pPackageInfo =
 			    (SecPkgInfoW*)sspi_ContextBufferAlloc(QuerySecurityPackageInfoIndex, size);
 
 			if (!pPackageInfo)
@@ -1196,16 +1193,14 @@ static SECURITY_STATUS SEC_ENTRY winpr_QuerySecurityPackageInfoW(SEC_WCHAR* pszP
 static SECURITY_STATUS SEC_ENTRY winpr_QuerySecurityPackageInfoA(SEC_CHAR* pszPackageName,
                                                                  PSecPkgInfoA* ppPackageInfo)
 {
-	size_t size = 0;
-	SecPkgInfoA* pPackageInfo = NULL;
-	size_t cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
 
 	for (size_t index = 0; index < cPackages; index++)
 	{
 		if (strcmp(pszPackageName, SecPkgInfoA_LIST[index]->Name) == 0)
 		{
-			size = sizeof(SecPkgInfoA);
-			pPackageInfo =
+			size_t size = sizeof(SecPkgInfoA);
+			SecPkgInfoA* pPackageInfo =
 			    (SecPkgInfoA*)sspi_ContextBufferAlloc(QuerySecurityPackageInfoIndex, size);
 
 			if (!pPackageInfo)
@@ -1698,7 +1693,7 @@ static SECURITY_STATUS SEC_ENTRY winpr_DeleteSecurityContext(PCtxtHandle phConte
 		return SEC_E_UNSUPPORTED_FUNCTION;
 	}
 
-	const UINT32 status = table->DeleteSecurityContext(phContext);
+	const SECURITY_STATUS status = table->DeleteSecurityContext(phContext);
 
 	if (IsSecurityStatusError(status))
 	{
