@@ -49,24 +49,29 @@
 static char* winpr_read_unix_timezone_identifier_from_file(FILE* fp)
 {
 	const INT CHUNK_SIZE = 32;
-	INT64 rc = 0;
-	INT64 read = 0;
-	INT64 length = CHUNK_SIZE;
-	char* tzid = NULL;
+	size_t rc = 0;
+	size_t read = 0;
+	size_t length = CHUNK_SIZE;
 
-	tzid = (char*)malloc((size_t)length);
+	char* tzid = malloc(length);
 	if (!tzid)
 		return NULL;
 
 	do
 	{
-		rc = fread(tzid + read, 1, length - read - 1, fp);
+		rc = fread(tzid + read, 1, length - read - 1UL, fp);
 		if (rc > 0)
 			read += rc;
 
-		if (read < (length - 1))
+		if (read < (length - 1UL))
 			break;
 
+		if (read > length - 1UL)
+		{
+			free(tzid);
+			return NULL;
+		}
+
 		length += CHUNK_SIZE;
 		char* tmp = (char*)realloc(tzid, length);
 		if (!tmp)
@@ -541,11 +546,11 @@ static BOOL map_iana_id(const char* iana, LPDYNAMIC_TIME_ZONE_INFORMATION tz)
 	const char* winDst = TimeZoneIanaToWindows(iana, TIME_ZONE_NAME_DAYLIGHT);
 
 	if (winStd)
-		ConvertUtf8ToWChar(winStd, tz->StandardName, ARRAYSIZE(tz->StandardName));
+		(void)ConvertUtf8ToWChar(winStd, tz->StandardName, ARRAYSIZE(tz->StandardName));
 	if (winDst)
-		ConvertUtf8ToWChar(winDst, tz->DaylightName, ARRAYSIZE(tz->DaylightName));
+		(void)ConvertUtf8ToWChar(winDst, tz->DaylightName, ARRAYSIZE(tz->DaylightName));
 	if (winId)
-		ConvertUtf8ToWChar(winId, tz->TimeZoneKeyName, ARRAYSIZE(tz->TimeZoneKeyName));
+		(void)ConvertUtf8ToWChar(winId, tz->TimeZoneKeyName, ARRAYSIZE(tz->TimeZoneKeyName));
 
 	return winId != NULL;
 }
@@ -595,7 +600,7 @@ static void log_print(wLog* log, DWORD level, const char* file, const char* fkt,
 	if (!WLog_IsLevelActive(log, level))
 		return;
 
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, line);
 	WLog_PrintMessageVA(log, WLOG_MESSAGE_TEXT, level, line, file, fkt, ap);
 	va_end(ap);
@@ -613,21 +618,21 @@ static void log_timezone_(const DYNAMIC_TIME_ZONE_INFORMATION* tzif, DWORD resul
 	log_print(log, level, file, fkt, line, "DYNAMIC_TIME_ZONE_INFORMATION {");
 
 	log_print(log, level, file, fkt, line, "  Bias=%" PRIu32, tzif->Bias);
-	ConvertWCharNToUtf8(tzif->StandardName, ARRAYSIZE(tzif->StandardName), buffer,
-	                    ARRAYSIZE(buffer));
+	(void)ConvertWCharNToUtf8(tzif->StandardName, ARRAYSIZE(tzif->StandardName), buffer,
+	                          ARRAYSIZE(buffer));
 	log_print(log, level, file, fkt, line, "  StandardName=%s", buffer);
 	log_print(log, level, file, fkt, line, "  StandardDate=%s",
 	          systemtime2str(&tzif->StandardDate, buffer, sizeof(buffer)));
 	log_print(log, level, file, fkt, line, "  StandardBias=%" PRIu32, tzif->StandardBias);
 
-	ConvertWCharNToUtf8(tzif->DaylightName, ARRAYSIZE(tzif->DaylightName), buffer,
-	                    ARRAYSIZE(buffer));
+	(void)ConvertWCharNToUtf8(tzif->DaylightName, ARRAYSIZE(tzif->DaylightName), buffer,
+	                          ARRAYSIZE(buffer));
 	log_print(log, level, file, fkt, line, "  DaylightName=%s", buffer);
 	log_print(log, level, file, fkt, line, "  DaylightDate=%s",
 	          systemtime2str(&tzif->DaylightDate, buffer, sizeof(buffer)));
 	log_print(log, level, file, fkt, line, "  DaylightBias=%" PRIu32, tzif->DaylightBias);
-	ConvertWCharNToUtf8(tzif->TimeZoneKeyName, ARRAYSIZE(tzif->TimeZoneKeyName), buffer,
-	                    ARRAYSIZE(buffer));
+	(void)ConvertWCharNToUtf8(tzif->TimeZoneKeyName, ARRAYSIZE(tzif->TimeZoneKeyName), buffer,
+	                          ARRAYSIZE(buffer));
 	log_print(log, level, file, fkt, line, "  TimeZoneKeyName=%s", buffer);
 	log_print(log, level, file, fkt, line, "  DynamicDaylightTimeDisabled=DST-%s",
 	          tzif->DynamicDaylightTimeDisabled ? "disabled" : "enabled");
@@ -729,7 +734,7 @@ static int dynamic_time_zone_from_localtime(const struct tm* local_time,
 	{
 		/* DST bias is the difference between standard time and DST in minutes */
 		const LONG d = get_bias(local_time, TRUE);
-		tz->DaylightBias = -1 * labs(tz->Bias - d);
+		tz->DaylightBias = -1 * (LONG)labs(tz->Bias - d);
 		if (!get_transition_date(local_time, FALSE, &tz->StandardDate))
 			rc |= HAVE_NO_STANDARD_TRANSITION_DATE;
 		if (!get_transition_date(local_time, TRUE, &tz->DaylightDate))
@@ -751,7 +756,7 @@ DWORD GetDynamicTimeZoneInformation(PDYNAMIC_TIME_ZONE_INFORMATION tz)
 	WINPR_ASSERT(tz);
 
 	*tz = empty;
-	ConvertUtf8ToWChar(defaultName, tz->StandardName, ARRAYSIZE(tz->StandardName));
+	(void)ConvertUtf8ToWChar(defaultName, tz->StandardName, ARRAYSIZE(tz->StandardName));
 
 	const time_t t = time(NULL);
 	struct tm tres = { 0 };
@@ -858,14 +863,14 @@ DWORD EnumDynamicTimeZoneInformation(const DWORD dwIndex,
 		return ERROR_NO_MORE_ITEMS;
 
 	if (entry->DaylightName)
-		ConvertUtf8ToWChar(entry->DaylightName, lpTimeZoneInformation->DaylightName,
-		                   ARRAYSIZE(lpTimeZoneInformation->DaylightName));
+		(void)ConvertUtf8ToWChar(entry->DaylightName, lpTimeZoneInformation->DaylightName,
+		                         ARRAYSIZE(lpTimeZoneInformation->DaylightName));
 	if (entry->StandardName)
-		ConvertUtf8ToWChar(entry->StandardName, lpTimeZoneInformation->StandardName,
-		                   ARRAYSIZE(lpTimeZoneInformation->StandardName));
+		(void)ConvertUtf8ToWChar(entry->StandardName, lpTimeZoneInformation->StandardName,
+		                         ARRAYSIZE(lpTimeZoneInformation->StandardName));
 	if (entry->Id)
-		ConvertUtf8ToWChar(entry->Id, lpTimeZoneInformation->TimeZoneKeyName,
-		                   ARRAYSIZE(lpTimeZoneInformation->TimeZoneKeyName));
+		(void)ConvertUtf8ToWChar(entry->Id, lpTimeZoneInformation->TimeZoneKeyName,
+		                         ARRAYSIZE(lpTimeZoneInformation->TimeZoneKeyName));
 
 	const time_t t = time(NULL);
 	struct tm tres = { 0 };
@@ -901,8 +906,10 @@ DWORD EnumDynamicTimeZoneInformation(const DWORD dwIndex,
 	return ERROR_SUCCESS;
 }
 
+// NOLINTBEGIN(readability-non-const-parameter)
 DWORD GetDynamicTimeZoneInformationEffectiveYears(
     const PDYNAMIC_TIME_ZONE_INFORMATION lpTimeZoneInformation, LPDWORD FirstYear, LPDWORD LastYear)
+// NOLINTEND(readability-non-const-parameter)
 {
 	WINPR_UNUSED(lpTimeZoneInformation);
 	WINPR_UNUSED(FirstYear);
