@@ -736,8 +736,7 @@ BOOL WSAResetEvent(HANDLE hEvent)
 
 BOOL WSACloseEvent(HANDLE hEvent)
 {
-	BOOL status = 0;
-	status = CloseHandle(hEvent);
+	BOOL status = CloseHandle(hEvent);
 
 	if (!status)
 		SetLastError(6);
@@ -762,7 +761,7 @@ int WSAEventSelect(SOCKET s, WSAEVENT hEventObject, LONG lNetworkEvents)
 	if (lNetworkEvents & FD_WRITE)
 		mode |= WINPR_FD_WRITE;
 
-	if (SetEventFileDescriptor(hEventObject, s, mode) < 0)
+	if (SetEventFileDescriptor(hEventObject, (int)s, mode) < 0)
 		return SOCKET_ERROR;
 
 	return 0;
@@ -1043,6 +1042,7 @@ int _connect(SOCKET s, const struct sockaddr* name, int namelen)
 	return status;
 }
 
+// NOLINTNEXTLINE(readability-non-const-parameter)
 int _ioctlsocket(SOCKET s, long cmd, u_long* argp)
 {
 	int fd = (int)s;
@@ -1060,9 +1060,9 @@ int _ioctlsocket(SOCKET s, long cmd, u_long* argp)
 			return SOCKET_ERROR;
 
 		if (*argp)
-			fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+			(void)fcntl(fd, F_SETFL, flags | O_NONBLOCK);
 		else
-			fcntl(fd, F_SETFL, flags & ~(O_NONBLOCK));
+			(void)fcntl(fd, F_SETFL, flags & ~(O_NONBLOCK));
 	}
 
 	return 0;
